#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Fri Mar 16 15:50:15 2018

@author: lenaboysen

Time of Emergence (ToE)

first method:
based on approach by Sarah Schlunegger, 2019, NCC: DOI: 10.1038/s41558-019-0553-2
--> based on trends over the ensemble dimension of deforest-globe
signal: meand of trends from t0 to ti
noise: std over trends from t0 to ti
SNR = 2



second method: 
based on 'traditional approach'
--> 
signal =  running mean over time independent of ensemble size  
noise = standard deviation of reference period, here piControl
SNR: 1 (2 was too large to detect anything)



"""


import os
import numpy as np
np.set_printoptions(threshold=100) # 'nan'
#import os.path
import sys as sys
import pickle

#import cdo
#cdo = cdo.Cdo()
#cdo.setCdo('/opt/local/bin/cdo')
import xarray as xr
#from matplotlib import gridspec
#from datetime import datetime, date, time, timedelta

#plot
import matplotlib.pyplot as plt
import matplotlib as mpl
import matplotlib.colors as colors
import seaborn as sns; sns.set(color_codes=True)
import cartopy.crs as ccrs

#import plotly.plotly as py
#import plotly.graph_objs as go
import pandas as pd



new_path = '/Users/lenaboysen/Google Drive/python/'
if new_path not in sys.path:
    sys.path.append(new_path)
    
#from mod_tTest import *
import mod_tTest as ttest
import zerodivcolormap as zcmap
 
## Constants
kg_C_per_kg_CO2 = 0.272918
kg_C_perMol = 0.012011
C_to_K = 273.15
MPa_to_mm = 1022.8

#%% FUNCTIONS

def whitelinefix(lons):
    return np.append(lons,360)

def make_ticklabels_invisible(fig):
    for i, ax in enumerate(fig.axes):
        ax.text(0.5, 0.5, "ax%d" % (i+1), va="center", ha="center")
        for tl in ax.get_xticklabels() + ax.get_yticklabels():
            tl.set_visible(False)
      
       
def read_data(var,timstat,model,path,runs,lat=None,lon=None):
#    var = 'treeFrac'
#    timstat = 'year'
#    path = "/Users/lenaboysen/mistral/LUMIP/IPSL/"
#    length = 1
#    var = [var]
#    vv = 0;
#    runs = ['r1i1p1f1','r2i1p1f1','r3i1p1f1','r4i1p1f1',\
#                               'r5i1p1f1','r6i1p1f1','r7i1p1f1']

    if type(var) == str:
        length = 1
        var = [var]
    elif type(var) == list:
        length = np.shape(var)[0]
        
    ds_data = None
    for vv in range(0,length):       
        for rr in runs: 
            
            print 'read in exp = '+ model+ ' '+ rr + ' ' + ' '+ var[vv]
            ds_new = xr.open_dataset(path + rr +'/'\
                                     +var[vv]+'_'+model+'_'+timstat+'.nc',\
                                     cache=False, decode_times=False)
            ds_new.close()
    
#            print ds_new[var[vv]].shape # dsA.cveg.coords, dsA.cveg.dims, dsA.info
            ### Data must be in shape  lon, lat, time!!
            nt = np.shape(ds_new['time'])[0]
            if timstat == 'year':
                ff = 'y'
            else:
                ff = 'm'
            
#            if lat is None and (rr == runs[0] or rr == 'piControl'):
#                    lat = ds_new['lat'].values
#            if lon is None and (rr == runs[0] or rr == 'piControl'):
#                    lon = ds_new['lon'].values 
                    
            lat = def_mask_regions[model][model+'_deffrac_glob'].lat
#                lat = ds_new['lat'].values
            lon = ds_new['lon'].values

            startyear = 1850
            time = pd.date_range(str(startyear)+'-01-01', periods=nt, freq=ff)
            ds_new2 = xr.Dataset({var[vv]:(('time', 'lat', 'lon'),ds_new[var[vv]])},
                                 {'time': time,
                                  'lat':lat, #ds_new['lat'],
                                  'lon':lon}) # ds_new['lon']})
            ds_new2 = ds_new2.transpose('lon','lat','time')

            if ds_data is None:
                ds_data = xr.Dataset({var[vv]+'_'+rr:ds_new2[var[vv]]})
            else:
                ds_new3 = xr.Dataset({var[vv]+'_'+rr:ds_new2[var[vv]]})
                ds_data = xr.merge([ds_data,ds_new3])
                del ds_new3
            
        print ds_data
    return(ds_data)
    

def shiftmergetime(ds, var, dt):
#    ds = DEF_30.copy()
#    dt = 30
#    var = 'tas'
#    
    ds_new = None
    items = [s for s in ds.keys() if var in s]
    
    j = 0
    for i in items:
        if ds_new is None:
            ds_new = ds[i]
            j = j + 1
        else: 
            firstyear = ds['time'].dt.year.data[0]
            
            if len(items)>5:
                firstyear = firstyear - dt*len(items)/2
                
            time = pd.date_range(str(firstyear)+'-01-01', 
                                 periods=dt, 
                                 freq='Y').shift(dt*j,freq='Y')
            ds['time'] = time

            ds_new = xr.concat([ds_new,ds[i]],dim='time')
            j = j + 1
            
    ds_new2 = xr.Dataset({var:(('lon', 'lat', 'time'), ds_new.values)},
                          {'lon': ds_new['lon'],
                           'lat': ds_new['lat'],
                           'time': ds_new['time']})            
    
    return ds_new2


def myround(x, base=0.5):
        return round(base*round(x/base), 2)        




def ensmean(variables,data):
    data_ensmean = None
    for vv in variables:
        items = [s for s in data.keys() if vv in s]
        data_mean2 = data[items].to_array().mean(axis=0).to_dataset(name=vv)
        if data_ensmean is None:
            data_ensmean = data_mean2
        else: 
            data_ensmean = xr.merge([data_ensmean,data_mean2])
    return data_ensmean


def ensmeans(variables,data,models):
#    variables = ['treeFrac']
#    data = DEF_mean
#    models = models
    
    data_ensmean = None
    for model in models:            
        itemsM = [s for s in data.keys() if model in s] # all keys with model 
    
        for vv in variables:
            itemsV = [s for s in itemsM if vv in s] # thereof all keys with var
            
            data_mean2 = data[itemsV].to_array().mean(axis=0).to_dataset(name=vv+'_'+model)
            
            if data_ensmean is None:
                data_ensmean = data_mean2
            else: 
                data_ensmean = xr.merge([data_ensmean,data_mean2])
    return data_ensmean
        


def disarrange(a, axis=-1):
    """
    Shuffle `a` in-place along the given axis.

    Apply numpy.random.shuffle to the given axis of `a`.
    Each one-dimensional slice is shuffled independently.
    """
    b = a.swapaxes(axis, -1)
    # Shuffle `b` in-place along the last axis.  `b` is a view of `a`,
    # so `a` is shuffled in place, too.
    shp = b.shape[:-1]
    for ndx in np.ndindex(shp):
        np.random.shuffle(b[ndx])
    return


def spatial_trend(data):
#    data = ds
    ## from http://atedstone.github.io/rate-of-change-maps/
    data = data.transpose('time','lon','lat')
    years = range(0,len(data['time']))
    vals = data.values
    vals2 = vals.reshape(len(years),-1)
    ## polyfit returns slope = coeff and intersect
#    if not np.any(np.isfinite(vals2)):
    reg =  np.polyfit(years, vals2, 1)
    # Get the coefficients back
    trends = reg[0,:].reshape(vals.shape[1], vals.shape[2])
#    else:
##        idx = np.isfinite(vals2[np.shape(vals2)[0]-1,:])
#        idx = np.where(~np.isinf(vals2[np.shape(vals2)[0]-1,:])) or \
#                np.where(~np.isnan(vals2[np.shape(vals2)[0]-1,:]))
#                
##        reg =  np.polyfit(years, vals2[idx], 1)
#        reg = np.empty(shape=(np.shape(vals2)[1]))
#        
#        for xx in idx[0]:
#            idxx = np.where(~np.isinf(vals2[:,xx]))
#            
#            reg[idx] =  np.polyfit(np.asarray(years)[idxx],
#                                             vals2[:,xx][idxx], 1)[0]
        # Get the coefficients back
#        trends = reg.reshape(vals.shape[1], vals.shape[2])
        
    return(trends)
    

        
def calc_ToE_Sarah(data, ctl, var, model, meta, rw, 
                   add_members=True, noisetag = 'DEF', DF = None):

##    ### VERSION 1
#    data = DEF.copy()
#    var='tas'
#    ctl = CTL
#    meta = meta
#    option = 1
#    model = 'MPI'
#    noisetag = 'CTL'
#    DF = None #defarea['diff_t']
#    rr = 'r1i1p1f1'
#    i = 2
###    
    print 'noisetag = ' + noisetag
    
    if type(var) == list:
        var = var[0]
        
    imax = meta.loc[model]['imax']
    runs = meta.loc[model]['runs']
    lat = DEF.lat.values
    lon = DEF.lon.values
    istart = 2
    
    if noisetag == 'DEF':
        SNR = 2
        cix = rw/2
        for i in range(istart,imax): 
            print 'Timestep = ' + str(i)
            
            trends = None          
            for rr in runs:
                data_var = data[var+'_'+rr].isel(time=range(0,i)) - \
                                        ctl[var+'_piControl'].mean('time')
#                ds = spatial_trend(data_var) 
                                        
                if not DF is None:
                    DFt = DF.copy().isel(time=range(0,i))
#                    DFt = DFt.where(DFt > 0.01, 0)
#                    dsF = spatial_trend(DFt)
                    ds = data_var.copy() / DFt
#                    ds = ds / dsF
                    ## polyfit cannot handle nan or inf
                    ds = ds.where(np.isfinite(ds),0) 
#                    ds = np.where(np.isfinite(ds),ds,0)
#                    ds = xr.where(ds.isnull(), ds, np.nan)
                else:
                    ds = data_var.copy()          
#                plt.pcolormesh((ds[:,:,38]))#.isel(time=38))
#                plt.colorbar()
#                plt.show()

                ds = spatial_trend(ds)  
  
                if trends is None:
                    trends = xr.Dataset({rr:(('lon','lat'),ds)})
                    #del(ds)
                else:
                    dummy = xr.Dataset({rr:(('lon','lat'),ds)})
                    trends = trends.merge(dummy)
                    del(dummy);# del(ds)

                if add_members == True:    
                    if rr == runs[len(runs)-1] and len(runs) < 5:
                        for nn in range(1,6-(len(runs))):
                            if i < imax-6:
                                data_var = data[var+'_'+rr].isel(time=range(nn,i+nn)) - \
                                                ctl[var+'_piControl'].mean('time')
                            else:
                                data_var = data[var+'_'+rr].isel(time=range(nn,imax-nn)) - \
                                                ctl[var+'_piControl'].mean('time')
                                                
                            ds = spatial_trend(data_var)
     
                            if not DF is None:
                                DFt = DF.copy().isel(time=range(0,i))/100.*(-1)
                                DFt = DFt.where(DFt > 0.01, 0)
                                dsF = spatial_trend(DFt)
                                ds = ds / dsF
                            
                            dummy = xr.Dataset({rr+str(nn):(('lon','lat'),ds)})
                            trends = trends.merge(dummy)
                            del(dummy); del(ds)                     
                    
                signal = trends.to_array(dim="ensemble").mean("ensemble") 
                noise = trends.to_array(dim="ensemble").std("ensemble")
                
                mask_ts = xr.where(np.abs(signal) < SNR*np.abs(noise), np.nan, i)
                if i == istart:
                    mask_merged = mask_ts
                else:
                    mask_merged = xr.concat([mask_merged, mask_ts],dim='time')
                
                
                
        for i in range(istart,imax-5):
            print 'i = ' + str(i)
            toe_certain = xr.where(np.isnan(mask_merged.isel(time=range(i,i+5))).any(axis=0),0,1)
            if i == istart:
                toe = toe_certain
            else:
                toe = toe + toe_certain
        
        
        
        toe_final = xr.Dataset({'ToE': (('lon', 'lat'), (imax - toe))},
                                         {'lon': lon, 'lat': lat})             
                


    ### =============================================================
    elif noisetag == 'CTL':
    ### noise = std over ref period
    ### signal = runmean
        cix = rw/2
        ref = ctl[var+'_piControl'].isel(time=range(0,30))
        SNR = 1
        
        for i in range(cix,imax-cix): 
            print 'Timestep = ' + str(i)
        
            trends = None          
            for rr in runs:
                
                ds = data[var+'_'+rr].isel(time=range(i-cix,i+cix)).mean('time') - \
                    ref.mean('time')
                    
                if trends is None:
                    trends = xr.Dataset({rr:(('lon','lat'),ds)})
                    #del(ds)
                else:
                    dummy = xr.Dataset({rr:(('lon','lat'),ds)})
                    trends = trends.merge(dummy)
                    del(dummy);# del(ds)  
                    
            signal = trends.to_array(dim="ensemble").mean("ensemble")  
            noise =  ref.std("time")
            
            mask_ts = xr.where(np.abs(signal) < SNR*np.abs(noise), np.nan, i)
            if i == cix:
                mask_merged = mask_ts
            else:
                mask_merged = xr.concat([mask_merged, mask_ts],dim='time')

        for i in range(0,len(mask_merged.time)-cix):
            toe_certain = xr.where(np.isnan(mask_merged.isel(time=range(i,i+cix))).any(axis=0),0,1)
            if i == 0:
                toe = toe_certain
            else:
                toe = toe + toe_certain
            
        toe_final = xr.Dataset({'ToE': (('lon', 'lat'), (imax - toe))},
                                         {'lon': lon, 'lat': lat})
    

    return(toe_final)
 
    
    
        
#%%  DEFINE VARIABLES
    
regions = ['_glob','_soam','_africa','_sea_aust','_noam_temp','_noam_bor','_eura_bor','_eura_temp']
regions_names = ['Global','South America','Africa','SE Asia & Aust.','North Am. temp.','North Am. bor.','Eurasia bor.','Eurasia temp.']

models = ['MPI','IPSL','CESM2','Can','CNRM','BCC','MIROC','UKESM','EC']
models_name = pd.DataFrame({'IPSL': ['IPSL-CM6A-LR'],
                            'MPI': ['MPI-ESM-1.2.0'],
                             'CESM2': ['CESM2'],
                             'Can': ['CanESM5'],
                             'CNRM': ['CNRM-ESM2-1'],
                             'BCC': ['BCC-CSM2-MR'],
                             'MIROC': ['MIROC-ES2L'],
                             'UKESM':['UKESM1-0-LL'],
                             'EC':['EC-Earth3-Veg']})


timstat = 'year'

meta = pd.DataFrame({'model':['MPI','IPSL','CESM2',\
                              'Can','CNRM','BCC','MIROC','UKESM','EC'],
                     'imax':[150, 80,  80, 90, 80, 81,150, 80, 80],
                     'imaxctl': [90, 90, 90, 90, 90, 90,90,90, 90],
                     'nruns':[7,3, 3, 2, 1, 1, 1, 1, 1],
                     'runs': [['r1i1p1f1','r2i1p1f1','r3i1p1f1','r4i1p1f1',\
                               'r5i1p1f1','r6i1p1f1','r7i1p1f1'],\
                              ['r1i1p1f1','r2i1p1f1','r3i1p1f1'],\
                              ['r1i1p1f1','r2i1p1f1','r3i1p1f1'],\
                              ['r1i1p2f1'],\
                              ['r1i1p1f2'],\
                              ['r1i1p1f1'],\
                              ['r1i1p1f1'],\
                              ['r1i1p1f2'],\
                              ['r1i1p1f1']]\
                               }).set_index('model')
    
metaCTL = pd.DataFrame({'model':['MPI','IPSL','CESM2',\
                                 'Can','CNRM','BCC','MIROC','UKESM','EC'],
                     'imax':[150, 80,  80, 90, 80, 81, 90, 90, 80],
                     'imaxctl': [90, 90, 90, 90, 90, 90, 90, 90, 90],
                     'nruns':[ 7, 3, 1, 2, 1, 1, 1, 1, 1],
                     'runs': [['piControl'],
                              ['piControl'],
                              ['piControl'],\
                              ['piControl'],\
                              ['piControl'],
                              ['piControl'],
                              ['piControl'],
                              ['piControl'],
                              ['piControl']]}).set_index('model')
rr = 0
timstat = 'year'
typ = '_'

varinfo = pd.DataFrame({'var':['tas',
                               'pr',
                               'gpp',
                               'cLand',
                               'cVeg','cSoil','cLitter',
                               'cSoilFast','cSoilSlow','cSoilMedium',
                               'evspsbl','npp','rh','fFire',
                               'ts','hfss','hfls','rsus','rsds','rlds'],
                     'name':['$\Delta$Near Surface Temperature',
                             '$\Delta$Precipitation',
                             '$\Delta$GPP',
                             '$\Delta$Total Land Carbon',
                             '$\Delta$Vegetation Carbon',
                             '$\Delta$Soil Carbon',
                             '$\Delta$Litter Carbon',
                             '$\Delta$Soil Carbon (fast)',
                             '$\Delta$Soil Carbon (slow)',
                             '$\Delta$Soil Carbon (medium)',
                             '$\Delta$Evapotranspiration',
                             '$\Delta$NPP',
                             '$\Delta$Het. respiration',
                             '$\Delta$Fire Emissions',
                             '$\Delta$Surface Temperature',
                             '$\Delta$Sensible Heat Flux',
                             '$\Delta$Latent Heat Flux',
                             '$\Delta$Outgoing Shortwave Radiation',
                             '$\Delta$Incoming Shortwave Radation',
                             '$\Delta$Incoming Longwave Radiation'] ,
                     'name_short': ['$\Delta$Tas',
                             '$\Delta$Pr',
                             '$\Delta$GPP',
                             '$\Delta$cLand',
                             '$\Delta$cVeg',
                             '$\Delta$cSoil',
                             '$\Delta$cLitter',
                             '$\Delta$cSoil (fast)',
                             '$\Delta$cSoil (slow)',
                             '$\Delta$cSoil (medium)',
                             '$\Delta$ET',
                             '$\Delta$NPP',
                             '$\Delta$rh',
                             '$\Delta$fFire',
                             '$\Delta$T$_s$',
                             '$\Delta$Q$_S$',
                             '$\Delta$Q$_H$',
                             '$\Delta$SW$_U$',
                             '$\Delta$SW$_D$',
                             '$\Delta$LW$_D$'] ,
                     'unit': ['$^\circ$C',
                              'mm '+timstat+'$^{-1}$',
                              'kg m$^{-2}$'+timstat+'$^{-1}$',
                              'kg m$^{-2}$','kg m$^{-2}$','kg m$^{-2}$','kg m$^{-2}$',
                              'kg m$^{-2}$','kg m$^{-2}$','kg m$^{-2}$',
                              'mm '+timstat+'$^{-1}$',
                              'kg m$^{-2}$ '+timstat+'$^{-1}$',
                              'kg m$^{-2}$ '+timstat+'$^{-1}$',
                              'kg m$^{-2}$ '+timstat+'$^{-1}$',
                              'K','W m$^{-2}$','W m$^2{-2}$',
                              'W m$^{-2}$','W m$^{-2}$','W m$^{-2}$'],
                     'rm':[30,30,10,10,10,10,10,10,10,10,
                           30,10,10,10,
                           30,30,30,30,30,30],
                    'minmax':[2.5,500,1.0,20.0,
                              15,5,2,0.6,0.6,0.6,
                              400,0.5,0.5,0.25,
                              2.5,50,50,
                              100,100,100],
                    'minmax_scaled':[8.0,500,1.0,30.00,
                                     20,10,5,0.6,0.6,0.6,
                                     400,0.5,0.5,0.25,
                                     2.5,100,100,
                                     100,100,100]\
                               }).set_index('var')  
    
#%% Read in mask of deforestation --> Overlay maps with contour
mistral = False
    ### All models in one xarray: 
    ### lat and lon don't match but xarray merges anyway! -
    ### -> can still be used for time series
mm = 0 ## model counter

#for model in ['MPI','IPSL','CESM2']
for model in models: #['MIROC','UKESM','EC']: # 'Can','CNRM','BCC',
    ### =============================================
    #### Create DEFORESTATION MASK
    ### =============================================
   
    if mistral == True:
        savepath = "/Users/lenaboysen/mistral/LUMIP/" 
        path="/Users/lenaboysen/mistral/LUMIP/"+model+"/"
    else:
        savepath = "/Users/lenaboysen/Documents/Crescendo_LUMIP/LUMIP/DATA/"
        path = "/Users/lenaboysen/Documents/Crescendo_LUMIP/LUMIP/DATA/"+model+"/"

    file_out = savepath + 'def_mask_regions.obj'
    if os.path.isfile(file_out):
        print 'loading def mask'
        def_mask_regions = pickle.load(open(file_out,'rb'))
        def_mask = def_mask_regions[model][model+'_defmask_glob']
        deffrac = def_mask_regions[model][model+'_deffrac_glob']/100.*(-1)
        ca = def_mask_regions[model][model+'_cellarea']
        
    file_out = savepath + 'deffrac_TS_regions.obj'
    if os.path.isfile(file_out):
        diff_t = pickle.load(open(file_out,'rb')) 
        deffrac_t = diff_t[model]/100.*(-1)
            
            
#    defarea =  deffrac_t * ca / 10**9 ## 10**3 km2   
#        
#    plt.pcolormesh(defarea['diff_t'].isel(time=38))
#    plt.colorbar()
#    plt.show()
    ### =============================================
    #### READ in VARIABLES 
    ### =============================================
    
    variables = ['cLand', 'tas', 'gpp']
    
    rr = 0
    timstat = 'year'
    typ = '_'
    
    if type(variables) == str:
        length = 1
        var = [variables]
    elif type(variables) == list:
        length = np.shape(variables)[0]
        var = variables
    
    for vv in [1]: #[0, 1]: 
    

        DEF = read_data(var[vv],timstat,model,path,meta.loc[model]['runs'])
        CTL = read_data(var[vv],timstat,model,path,['piControl'])
    
        lat = DEF.lat.values
        lon = DEF.lon.values
        
        if 'kg' in varinfo.loc[var[vv]]['unit'] :
            DEF = DEF.fillna(0)
            CTL = CTL.fillna(0)
        
        ## ==========    ***   =====================
        option = 1 ## 1=0:i; 2=0:(i+30/len(runs))
        noisetag = 'DEF' ## DEFens or CTL30years
        add_members = False
        ## ==========    ***   =====================   

#        var = variables #['tas']
        option = 1
        DF = deffrac_t['diff_t'] ##  None
        ToE = calc_ToE_Sarah(DEF.copy(), CTL, var[vv], model, meta, 
                             varinfo.loc[var[vv]]['rm'], 
                             add_members=add_members, noisetag=noisetag, 
                             DF = None) 
        
    
        plt.pcolormesh(ToE['ToE']) 
        plt.colorbar()
        plt.show()

        FoE = xr.Dataset({'FoE':(('lon','lat'),np.zeros(shape=(len(DEF.lon),len(DEF.lat))))}
                ,{'lon':DEF.lon, 'lat':DEF.lat})
       
#        tmax = meta.loc[model]['imax']
#        if model == 'BCC':
#            tmax = 50
#        for i in range(0,30):
#            ix =  ToE.where(ToE==i,drop=True)
#            FoE['FoE'].loc[dict(lat=ix.lat,lon=ix.lon)] =\
#                defarea['diff_t'].isel(time=i).sel(lat=ix.lat,lon=ix.lon)
#
#            plt.pcolormesh(FoE['FoE']) 
#            plt.colorbar()
#            plt.show()
            
            
        for la in range(0,len(lat)):
            for lo in range(0,len(lon)):
               ix = ToE['ToE'].isel(lon=lo, lat=la)#.value
               if model == 'BCC' and ix > 50:
                   continue
               
#               FoE['FoE'].loc[dict(lat=lat[la],lon=lon[lo])]= defarea['diff_t'].isel(time=ix-1,lon=lo, lat=la)

               FoE['FoE'].loc[dict(lat=lat[la],lon=lon[lo])]= deffrac_t['diff_t'].isel(time=ix-1,lon=lo, lat=la)            
            
#        
#        var = ['tas']
#        ### signal and noise normalized by deforestation fraction 
#        ### --> hints to advection of the signal (def signal is equal)        
#        ToE_scaled = calc_ToE_Sarah(DEF.copy(), CTL, var[vv], model, meta, 
#                                    varinfo.loc[var[vv]]['rm'],  option, 
#                                    noisetag='DEF', DF=defarea['diff_t'])
#
        plt.pcolormesh(FoE['FoE']) 
        plt.colorbar()
        plt.show()
#        
#        
#        plt.pcolormesh(ToE_scaled['ToE']-ToE['ToE']) 
#        plt.colorbar()
#        plt.show()
        
        
     
        ### =============================================
        #### ADJUST DATA FORMAT
        ### =============================================
        
        
        DEF_30 = DEF.isel(time=slice(meta.loc[model]['imax']-varinfo.loc[var[vv]]['rm'],meta.loc[model]['imax']))
        DEF_90 = shiftmergetime(DEF_30, var[vv], dt=varinfo.loc[var[vv]]['rm'])
        
        CTL_30 = CTL.isel(time=slice(meta.loc[model]['imaxctl']-varinfo.loc[var[vv]]['rm'],meta.loc[model]['imaxctl']))
        CTL_90 = CTL.rename({var[vv]+'_piControl': var[vv]})     
        
        CTL_data = CTL_90[var[vv]].data
        DEF_data = DEF_90[var[vv]].data
        varid = var[vv]    

        lat = DEF_90[DEF_90.keys()[0]]['lat'].data
        lon = DEF_90[DEF_90.keys()[0]]['lon'].data
        
        ### =============================================
        ### modified t-Test (t-statistic = t^)
        ### =============================================
        
        test3d = ttest.modTtest3d(CTL_data,DEF_data,
                                  alternative="two_sided",
                                  conf_level=0.95) 
        pval = test3d['pvalue'][:,:]
        mask = np.where((pval>0.025) & (pval<0.975), True, False) # True is masked out
        
        var_masked = np.ma.masked_array(DEF_data.mean(-1)-CTL_data.mean(axis=-1), mask)

            
        years_masked = np.ma.masked_array(ToE['ToE'], mask)
        years_masked[np.isnan(years_masked)] = 0
        
#        years_scaled_masked = np.ma.masked_array(ToE_scaled['ToE'], mask)
#        years_scaled_masked[np.isnan(years_scaled_masked)] = 0       
        
        ttest_contour = np.zeros(np.shape(pval))
        ttest_contour[(pval>0.025) & (pval<0.975)] = 1 ## shade non-significant areas
        ttest_contour[(pval<=0.025) & (pval>=0.975)] = np.nan 
    
        ToE_shade = xr.where(ToE['ToE'] > meta.loc[model]['imax'], 
                             np.nan, ToE['ToE'])  
#        ToE_scaled_shade = xr.where(ToE_scaled['ToE'] > meta.loc[model]['imax'], 
#                             np.nan, ToE_scaled['ToE'])          
        
        ToE_stats = xr.Dataset({'ToE':(('lon', 'lat'), ToE['ToE']),
                                'ToE_shade':(('lon', 'lat'), ToE_shade),
                                  'ToE_masked': (('lon', 'lat'), years_masked),
                                  'ttest_contour': (('lon', 'lat'), ttest_contour),
                                  'def_mask':(('lon','lat'), def_mask),
                                  'FoE': (('lon', 'lat'), -FoE['FoE'])},#,
#                                  'ToE_scaled':(('lon', 'lat'), ToE_scaled['ToE']),
#                                 'ToE_scaled_shade':(('lon', 'lat'), ToE_scaled_shade),
#                                 'ToE_scaled_masked': (('lon', 'lat'), years_scaled_masked)},
                                         {'lon': lon, 'lat': lat})
        
        if add_members == True:
            file_out = open(savepath + var[vv] + '_' + model +'_ToE_newmembers.obj','w')
            pickle.dump(ToE_stats, file_out)
            file_out.close() 
        else:
            file_out = open(savepath + var[vv] + '_' + model +'_ToE_'+noisetag+'.obj','w')
            pickle.dump(ToE_stats, file_out)
            file_out.close()  





#%% 
### =============================================
### PLOT ALL in One ToE
### =============================================  
add_members = False
noisetag = 'DEF'
var = ['tas','gpp','cLand']

for vv in [0]: #,1,2]:

    cexx2 = 14
    cexx = 12
    
    var_min = 0
    var_max = 150 #meta.loc[model]['imax'] ###--> set the same for all models
    
    lons = whitelinefix(lon)
    lons, lats = np.meshgrid(lons, lat)
    
    ### --- COLORBAR SPECIFICATIONS ----*
    inc = 10
    tcks = np.arange(0,160,10)
    blues = plt.cm.Blues(np.linspace(0., 1, 12))[[3,4,6,7,8,9,10,11],:]
    grey = [0.6, 0.6, 0.6, 1.]  ## light grey
    reds = plt.cm.YlOrRd(np.linspace(0, 1, 12))[[0,4,6,8,11],:] #6,8,9,11
    greens = plt.cm.Greens(np.linspace(0, 1, 12))[[2,7,11],:]
    greensB = plt.cm.Greens(np.linspace(0, 1, 12))[[11],:]
    redB = plt.cm.YlOrRd(np.linspace(0, 1, 12))[[11],:]
    
    # combine them and build a new colormap
    cols = np.vstack((reds, greens,blues))
    cmapA = mpl.colors.LinearSegmentedColormap.from_list('my_colormap', cols)   
    
    #for j in range(240,251):
    cmaplist = [cmapA(i) for i in range(cmapA.N)]
    cmaplist[72] = (0.5, 0.0, 0.15, 1.)   # dark red redB
    cmaplist[248] = (0.66, 0.66, 0.66, 1.)  # light grey
    
    # create the new map
    cmap = cmapA.from_list('Custom cmap', cmaplist, cmapA.N)
    norm = mpl.colors.BoundaryNorm(tcks, cmap.N) 
    cmap.set_over('grey')
    
    
    ###=============
    reds2 = plt.cm.YlOrRd(np.linspace(0, 1, 12))[[0,4,6,8,11],:] #6,8,9,11
    greens2 = plt.cm.Greens(np.linspace(0, 1, 12))[[2,7,11],:]
    
    tcks2 = np.arange(0,90,10)
    cols2 = np.vstack((reds2, greens2))
    cmap2 = mpl.colors.LinearSegmentedColormap.from_list('my_colormap', cols2)    
    
    
    #for j in range(210,221):
    cmaplist2 = [cmap2(i) for i in range(cmap2.N)]
    cmaplist2[241] = (0.66, 0.66, 0.66, 1.) # light grey
    cmaplist2[128] = (0.5, 0.0, 0.15, 1.)   # dark red redB
    cmaplist2[156] = (0.86, 0.94 , 0.82, 1.) # light green   
    cmaplist2[184] = (0.25, 0.66 , 0.36, 1.) # mid green  
    cmaplist2[213] = (0.0, 0.27 , 0.11, 1.) # dark green      
    
    
    # create the new map
    cmap2 = cmap2.from_list('Custom cmap', cmaplist2, cmap2.N)
    norm2 = mpl.colors.BoundaryNorm(tcks2, cmap2.N) 
    #    cmap2.set_over('grey')
    
    rgba = (0.3,0.3,0.3,1.0) 
    rgba2 = (0.8,0.8,0.8,0.3) 
    
    fig_scale = 2
    fig = plt.figure(figsize=(4*fig_scale,3*fig_scale))
    gs1 = plt.GridSpec(4, 3, height_ratios=[1, 0.05, 1, 0.05])
    
    gs1.update(wspace=0.05, hspace=0.025) 
      
    i = 0; j = 0; m = 0
    for model in ['MPI','IPSL','CESM2']:#models:#['MPI']:#
        print model + ' i = ' + str(i) + ' j = ' + str(j)
        
        #### created with time series scripts (or ToE_ts)
        if mistral == True:
            savepath = "/Users/lenaboysen/mistral/LUMIP/" 
            path="/Users/lenaboysen/mistral/LUMIP/"+model+"/"
        else:
            savepath = "/Users/lenaboysen/Documents/Crescendo_LUMIP/LUMIP/DATA/"
            path = "/Users/lenaboysen/Documents/Crescendo_LUMIP/LUMIP/DATA/"+model+"/"
        
        if add_members == True:
            file_out = savepath + var[vv] + '_' + model +'_ToE_newmembers.obj'
        else:
            file_out = savepath + var[vv] + '_' + model +'_ToE_'+noisetag+'.obj'
        
        if os.path.isfile(file_out):
            ToE_stats = pickle.load(open(file_out,'rb'))        
                
        def_file = savepath + 'def_mask_regions.obj'
        if os.path.isfile(def_file):
            print 'loading def mask for ' + model
            init = def_mask_regions[model][model+'_init_forest']
            if model == 'MPI':
                init = init.where(init>0,np.nan)
            init = init.where(np.isnan(init),1) ## make lsm mask
            
#            plt.pcolormesh(init) 
#            plt.colorbar()
#            plt.show()
                 
        axes = plt.subplot(gs1[j,i], 
                           projection=ccrs.Robinson(central_longitude=0.0))
        
        
        ToEplot = ToE_stats['ToE']*init
        ToEplotmasked = ToE_stats['ToE_masked']*init

#        plt.pcolormesh(ToE_stats['ToE']*init) 
#        plt.colorbar()
#        plt.show()

            
        #       ### shade non-significant areas
        if model in ['MPI','MIROC']:
            colormap = cmap
            colornorm = norm
            var_max = 150
            bnd=np.append(tcks2,[160])
        else:
            colormap = cmap #2
            colornorm = norm2
            #var_max = 80
            var_max = 150
            bnd=np.append(tcks2,[90])
            
    #    plot = ToEplot.T.plot.pcolormesh(transform = ccrs.PlateCarree(),
    #                                 add_colorbar=False,
    #                                 cmap=colormap,alpha=0.1, #norm=colornorm, 
    #                                 vmin = 0.0, vmax = var_max, 
    #                                 ax=axes)
    #    
        ### normal plotting of only significant areas
#        axes.set_ylim(-90,90)

        
        plot = ToEplotmasked.T.plot.pcolormesh(transform = ccrs.PlateCarree(),
                                     add_colorbar=False,
                                     cmap=colormap, #norm=colornorm, 
                                     vmin = 0.0, vmax = var_max, 
                                     ax=axes)   
#        plt.show()
#        plt.close() 
        
        
        
        cp = ToE_stats['def_mask'].T.plot.contour(ax=axes,levels=1, 
                      transform = ccrs.PlateCarree(),
                      add_colorbar=False,
                      vmin = 0, vmax = 1,
                      colors='k', origin="lower" ,linewidths=0.5)
        
#        axes.set_extent([-180, 180, -90, 180])
        
        axes.text(0.5,1.07,
            list(models_name[model])[0],
            color='slategrey',
            horizontalalignment='center',
            verticalalignment='center',
            transform=axes.transAxes,
            fontsize=cexx) #,
        
        axes.coastlines(color='grey',linewidth=0.5)
        axes.outline_patch.set_linewidth(0.25)
        axes.gridlines(color='grey',linewidth=0.5, alpha=0.5, linestyle='--')
        
        axes.text(0,1.07,['(a)','(b)','(c)','(d)'][m],
                        color='k',
                        horizontalalignment='left',
                        verticalalignment='center',
                        transform=axes.transAxes,
                        fontsize=cexx,
                        bbox={'facecolor':'white', 
                              'edgecolor':'white',
                              'alpha':0.7, 'pad':5})                   
        i = i + 1
        if m in [2,5,8]:
            i = 0
            j = j + 1
        
        m = m + 1        
            
            
                
    ### --- Colorbar ----
    axes = plt.subplot(gs1[1, :])
    axes.set_visible(False)
    #cbar_ax = fig.add_axes([0.12, 0.34, 0.6, 0.03]) ## x0, y0, width, height
    cbar_ax = fig.add_axes([0.22, 0.53, 0.6, 0.02]) ## x0, y0, width, height
    bar = mpl.colorbar.ColorbarBase(cbar_ax, cmap=cmap,
                            norm=norm,
                            boundaries=np.append(tcks,160),
                            ticks=tcks, 
                            extend='max',
                            spacing='uniform',
                            orientation='horizontal')
    
    bar.ax.tick_params(labelsize=10,length=2, width=1)
    bar.set_label('ToE [years] of '+varinfo.loc[var[vv]]['name'],
                  fontsize=14,labelpad=-40)
    #        bar.set_label(j,fontsize=14,labelpad=-50)        
    
    for label in bar.ax.xaxis.get_ticklabels()[::2]:
        label.set_visible(False)
    
    bar.ax.tick_params(labelsize=cexx)
    bar.outline.set_visible(False)
    
    
    
    ### --- LEGEND -------------
    import matplotlib.patches as mpatches
    p_fancy = mpatches.FancyBboxPatch((0.9, 0.1),
                             abs(0.3), abs(0.5),
                             boxstyle="round, pad=0.1",
                             fc='white',
                             ec='black',
                             label = '$\Delta$F',
                             linewidth = 0.5)
    p_fancy2 = mpatches.FancyBboxPatch((0.9, 0.1),
                             abs(0.3), abs(0.5),
                             boxstyle="round, pad=0.1",
                             fc=rgba,
                             ec=rgba,
                             label = 'sign.',
                             linewidth = 0.5)   
    p_fancy3 = mpatches.FancyBboxPatch((0.9, 0.1),
                             abs(0.3), abs(0.5),
                             boxstyle="round, pad=0.1",
                             fc=rgba2,
                             ec=rgba2,
                             label = 'non-sign.',
                             linewidth = 0.5)    
    #
    #plt.legend(handles=[p_fancy], fontsize=cexx-2,
    #           bbox_to_anchor=(0.02, 2.5), loc=2, borderaxespad=0.)       
    #plt.legend(handles=[p_fancy,p_fancy2,p_fancy3], fontsize=cexx-2,
    #           bbox_to_anchor=(1.03, 1.4), loc=2, borderaxespad=0.) 
    #plt.legend(handles=[p_fancy], fontsize=cexx-2,
    #           bbox_to_anchor=(1.03, 1.4), loc=2, borderaxespad=0.) 
    
    
    ### =============================================
    ### PLOT ALL in One FoE
    ### =============================================  
    
    var_min = -60
    cmapF = plt.cm.get_cmap('YlOrBr_r')
    cmaplistF = [cmapF(i) for i in range(cmapF.N)]
    cmapF = cmapF.from_list('Custom cmap', cmaplistF, cmapF.N)
    cmapF.set_over('white')
            
      
    i = 0; j = 2; m = 0
    for model in ['MPI','IPSL','CESM2']:#models:#['MPI']:#
        print model + ' i = ' + str(i) + ' j = ' + str(j)
        
        #### created with time series scripts (or ToE_ts)
        if mistral == True:
            savepath = "/Users/lenaboysen/mistral/LUMIP/" 
            path="/Users/lenaboysen/mistral/LUMIP/"+model+"/"
        else:
            savepath = "/Users/lenaboysen/Documents/Crescendo_LUMIP/LUMIP/DATA/"
            path = "/Users/lenaboysen/Documents/Crescendo_LUMIP/LUMIP/DATA/"+model+"/"
        
        if add_members == True:
            file_out = savepath + var[vv] + '_' + model +'_ToE_newmembers.obj'
        else:
            file_out = savepath + var[vv] + '_' + model +'_ToE_'+noisetag+'.obj'
        if os.path.isfile(file_out):
            ToE_stats = pickle.load(open(file_out,'rb'))        
                
        file_out = savepath + 'def_mask_regions.obj'
        if os.path.isfile(file_out):
            print 'loading def mask for ' +  model
            def_mask_regions = pickle.load(open(file_out,'rb'))
            def_mask = def_mask_regions[model][model+'_defmask_glob']
            diff = def_mask_regions[model][model+'_deffrac_glob']
            cellarea = def_mask_regions[model][model+'_cellarea']
    #            lat = deffrac.lat
            
        diffplot = -diff.where(diff < -0.1)
        defarea = -diff* cellarea /100/ 10**9 #10**3 km2            
                
        
        axes = plt.subplot(gs1[j,i], 
                           projection=ccrs.Robinson(central_longitude=0.0))
        
        FoEplot = ToE_stats['FoE']*100
        ToEplotmasked = ToE_stats['ToE_masked']
        FoEplot = FoEplot.where(~np.isnan(ToEplotmasked),np.nan)
        
        
        ### Difference FoE and dF
        FoE_diff = diffplot - ToE_stats['FoE']*(-1)*100 
        FoE_diffplot = FoE_diff.where(~np.isnan(ToEplotmasked),np.nan)
        
    #    plt.pcolormesh(ToE_stats['FoE']*100) 
    #    plt.colorbar()
    #    plt.show()
    #    
    #    plt.pcolormesh(FoEplot) 
    #    plt.colorbar()
    #    plt.show()    
        
        plot = FoEplot.T.plot.pcolormesh(transform = ccrs.PlateCarree(),
                                         add_colorbar=False,
                                         cmap=cmapF, #alpha=0.3, #norm=norm, 
                                         vmin = var_min, vmax = -1, 
                                         ax=axes)
        
        cp = ToE_stats['def_mask'].T.plot.contour(ax=axes,levels=1, 
                      transform = ccrs.PlateCarree(),
                      add_colorbar=False,
                      vmin = 0, vmax = 1,
                      colors='k', origin="lower" ,linewidths=0.5)
        
        axes.text(0.5,1.07,
            list(models_name[model])[0],
            color='slategrey',
            horizontalalignment='center',
            verticalalignment='center',
            transform=axes.transAxes,
            fontsize=cexx) #,
        
        axes.coastlines(color='grey',linewidth=0.5)
        axes.outline_patch.set_linewidth(0.25)
        axes.gridlines(color='grey',linewidth=0.5, alpha=0.5, linestyle='--')
                    
    
        axes.text(0,1.07,['(d)','(e)','(f)'][m],
                    color='k',
                    horizontalalignment='left',
                    verticalalignment='center',
                    transform=axes.transAxes,
                    fontsize=cexx,
                    bbox={'facecolor':'white',
                          'edgecolor':'white',
                          'alpha':0.7, 'pad':5}) 
    
        
        i = i + 1
        
        if m in [2,5,8]:
            i = 0
            j = j + 1
        
        m = m + 1        
            
    
            
                
    ### --- Colorbar ----
    axes = plt.subplot(gs1[3, :])
    axes.set_visible(False)
    #cbar_ax = fig.add_axes([0.15, 0.34, 0.6, 0.03]) ## x0, y0, width, height
    cbar_ax = fig.add_axes([0.22, 0.15, 0.6, 0.02])
    
    inc = 5
    tcks = np.arange(myround(var_min, base=10),0+inc,inc) #-0.1
    tcks = np.arange(var_min,-1,5)
    norm = mpl.colors.BoundaryNorm(tcks, cmapF.N)         
    cols = plt.cm.YlOrRd(np.linspace(0, 1, 12))[[11],:]
       
    bar2 = mpl.colorbar.ColorbarBase(cbar_ax, cmap=cmapF,
                            norm=norm,
                            boundaries=tcks,
                            ticks=tcks, 
                            extend='both',
                            spacing='uniform',
                            orientation='horizontal')
    
    bar2.ax.tick_params(labelsize=10,length=2, width=1)
    bar2.set_label('FoE [%] of '+varinfo.loc[var[vv]]['name'],fontsize=14,labelpad=-40) 
    for label in bar2.ax.xaxis.get_ticklabels()[::2]:
        label.set_visible(False)
    bar2.ax.tick_params(labelsize=cexx)
    bar2.outline.set_visible(False)
    
    ### --- LEGEND -------------
    import matplotlib.patches as mpatches
    p_fancy = mpatches.FancyBboxPatch((0.9, 0.1),
                             abs(0.3), abs(0.5),
                             boxstyle="round, pad=0.1",
                             fc='white',
                             ec='black',
                             label = '$\Delta$F',
                             linewidth = 0.5)
    p_fancy2 = mpatches.FancyBboxPatch((0.9, 0.1),
                             abs(0.3), abs(0.5),
                             boxstyle="round, pad=0.1",
                             fc=rgba,
                             ec=rgba,
                             label = 'sign.',
                             linewidth = 0.5)   
    p_fancy3 = mpatches.FancyBboxPatch((0.9, 0.1),
                             abs(0.3), abs(0.5),
                             boxstyle="round, pad=0.1",
                             fc=rgba2,
                             ec=rgba2,
                             label = 'non-sign.',
                             linewidth = 0.5)    
    #
    plt.legend(handles=[p_fancy], fontsize=cexx-2,
               bbox_to_anchor=(1.03, 1.2), loc=2, borderaxespad=0.)       
    #plt.legend(handles=[p_fancy], fontsize=cexx-2,
    #           bbox_to_anchor=(0.72, 7), loc=2, borderaxespad=0.) 
    
    
    ### --------------------------------
    plt.show()
    fig.tight_layout()            
    
    if add_members == True:
        pngfile='/Users/lenaboysen/Google Drive/LUMIP/Sigma_noise/ToE_FoE_'+\
                                    var[vv]+'_ALL_newmembers.png'  
    else:
        pngfile='/Users/lenaboysen/Google Drive/LUMIP/Sigma_noise/ToE_FoE_'+\
                                    var[vv]+'_ALL_'+noisetag+'3.png'                                        
    fig.savefig(pngfile,dpi=300, bbox_inches='tight',width=400,height=300)
    plt.close()            

### =============================================
### =============================================
### =============================================








#%% 
### =============================================
### PLOT ALL in One ToE
### =============================================  
cexx2 = 14
cexx = 12

var_min = 0
var_max = 150 #meta.loc[model]['imax'] ###--> set the same for all models

lons = whitelinefix(lon)
lons, lats = np.meshgrid(lons, lat)

### --- COLORBAR SPECIFICATIONS ----*
inc = 10
tcks = np.arange(0,160,10)
blues = plt.cm.Blues(np.linspace(0., 1, 12))[[3,4,6,7,8,9,10,11],:]
grey = [0.6, 0.6, 0.6, 1.]  ## light grey
reds = plt.cm.YlOrRd(np.linspace(0, 1, 12))[[0,4,6,8,11],:] #6,8,9,11
greens = plt.cm.Greens(np.linspace(0, 1, 12))[[2,7,11],:]
greensB = plt.cm.Greens(np.linspace(0, 1, 12))[[11],:]
redB = plt.cm.YlOrRd(np.linspace(0, 1, 12))[[11],:]

# combine them and build a new colormap
cols = np.vstack((reds, greens,blues))
cmapA = mpl.colors.LinearSegmentedColormap.from_list('my_colormap', cols)   

#for j in range(240,251):
cmaplist = [cmapA(i) for i in range(cmapA.N)]
cmaplist[72] = (0.5, 0.0, 0.15, 1.)   # dark red redB
cmaplist[248] = (0.66, 0.66, 0.66, 1.)  # light grey

# create the new map
cmap = cmapA.from_list('Custom cmap', cmaplist, cmapA.N)
norm = mpl.colors.BoundaryNorm(tcks, cmap.N) 
cmap.set_over('grey')


###=============
reds2 = plt.cm.YlOrRd(np.linspace(0, 1, 12))[[0,4,6,8,11],:] #6,8,9,11
greens2 = plt.cm.Greens(np.linspace(0, 1, 12))[[2,7,11],:]

tcks2 = np.arange(0,90,10)
cols2 = np.vstack((reds2, greens2))
cmap2 = mpl.colors.LinearSegmentedColormap.from_list('my_colormap', cols2)    


#for j in range(210,221):
cmaplist2 = [cmap2(i) for i in range(cmap2.N)]
cmaplist2[241] = (0.66, 0.66, 0.66, 1.) # light grey
cmaplist2[128] = (0.5, 0.0, 0.15, 1.)   # dark red redB
cmaplist2[156] = (0.86, 0.94 , 0.82, 1.) # light green   
cmaplist2[184] = (0.25, 0.66 , 0.36, 1.) # mid green  
cmaplist2[213] = (0.0, 0.27 , 0.11, 1.) # dark green      


# create the new map
cmap2 = cmap2.from_list('Custom cmap', cmaplist2, cmap2.N)
norm2 = mpl.colors.BoundaryNorm(tcks2, cmap2.N) 
#    cmap2.set_over('grey')

rgba = (0.3,0.3,0.3,1.0) 
rgba2 = (0.8,0.8,0.8,0.3) 
    

## ===================================
add_members = False
var = ['tas','cLand','gpp']
vv = 2


fig_scale = 2
fig = plt.figure(figsize=(4*fig_scale,3*fig_scale))
gs1 = plt.GridSpec(4, 3, height_ratios=[1,1,1, 0.05])

gs1.update(wspace=0.05, hspace=0.025) 
  
i = 0; j = 0; m = 0; mm = 0
for model in models:
    print model + ' i = ' + str(i) + ' j = ' + str(j)
    
    #### created with time series scripts (or ToE_ts)
    if mistral == True:
        savepath = "/Users/lenaboysen/mistral/LUMIP/" 
        path="/Users/lenaboysen/mistral/LUMIP/"+model+"/"
    else:
        savepath = "/Users/lenaboysen/Documents/Crescendo_LUMIP/LUMIP/DATA/"
        path = "/Users/lenaboysen/Documents/Crescendo_LUMIP/LUMIP/DATA/"+model+"/"
    
    if add_members == True:
        file_out = savepath + var[vv] + '_' + model +'_ToE_newmembers.obj'
    else:
        file_out = savepath + var[vv] + '_' + model +'_ToE_'+noisetag+'.obj'
    if os.path.isfile(file_out):
        ToE_stats = pickle.load(open(file_out,'rb'))        


    def_file = savepath + 'def_mask_regions.obj'
    if os.path.isfile(def_file):
        print 'loading def mask for ' + model
        init = def_mask_regions[model][model+'_init_forest']
        if model == 'MPI':
            init = init.where(init>0,np.nan)
            
        if model in ['Can','EC','MIROC']:
            init = init/100
            init = init.where(init==0,1)
            init = init.where(init==1,np.nan)
        else:
            init = init.where(np.isnan(init),1) ## make lsm mask
        
        init['lon'] = ToE_stats.lon     
            
    axes = plt.subplot(gs1[j,i], 
                       projection=ccrs.Robinson(central_longitude=0.0))
#    
#    
#    plt.pcolormesh(init) 
#    plt.colorbar()
#    plt.show()    
##    
    
    
    ToEplot = ToE_stats['ToE']*init
    ToEplotmasked = ToE_stats['ToE_masked']*init
        
    
    colormap = cmap
    colornorm = norm
    var_max = 150
    bnd=np.append(tcks2,[160])
        
        
    ### normal plotting of only significant areas
    plot = ToEplotmasked.T.plot.pcolormesh(transform = ccrs.PlateCarree(),
                                 add_colorbar=False,
                                 cmap=colormap, #norm=colornorm, 
                                 vmin = 0.0, vmax = var_max, ax=axes)     
    
    
    cp = ToE_stats['def_mask'].T.plot.contour(ax=axes,levels=1, 
                  transform = ccrs.PlateCarree(),
                  add_colorbar=False,
                  vmin = 0, vmax = 1,
                  colors='k', origin="lower" ,linewidths=0.5)
    
    axes.text(0.5,1.07,
        list(models_name[model])[0],
        color='slategrey',
        horizontalalignment='center',
        verticalalignment='center',
        transform=axes.transAxes,
        fontsize=cexx) #,
    
    axes.coastlines(color='grey',linewidth=0.5)
    axes.outline_patch.set_linewidth(0.25)
    axes.gridlines(color='grey',linewidth=0.5, alpha=0.5, linestyle='--')
                    
    i = i + 1
    if m in [2,5]:
        i = 0
        j = j + 1
    
    m = m + 1        
        
        
### --- Colorbar ----
axes = plt.subplot(gs1[3, :])
axes.set_visible(False)
#cbar_ax = fig.add_axes([0.12, 0.34, 0.6, 0.03]) ## x0, y0, width, height
cbar_ax = fig.add_axes([0.12, 0.1, 0.6, 0.02]) ## x0, y0, width, height
bar = mpl.colorbar.ColorbarBase(cbar_ax, cmap=cmap,
                        norm=norm,
                        boundaries=np.append(tcks,160),
                        ticks=tcks, 
                        extend='max',
                        spacing='uniform',
                        orientation='horizontal')

bar.ax.tick_params(labelsize=10,length=2, width=1)
bar.set_label('ToE [years] of '+varinfo.loc[var[vv]]['name'],
              fontsize=14,labelpad=-40)
#        bar.set_label(j,fontsize=14,labelpad=-50)        

for label in bar.ax.xaxis.get_ticklabels()[::2]:
    label.set_visible(False)

bar.ax.tick_params(labelsize=cexx)
bar.outline.set_visible(False)



### --- LEGEND -------------
import matplotlib.patches as mpatches
p_fancy = mpatches.FancyBboxPatch((0.9, 0.1),
                         abs(0.3), abs(0.5),
                         boxstyle="round, pad=0.1",
                         fc='white',
                         ec='black',
                         label = '$\Delta$F',
                         linewidth = 0.5)
p_fancy2 = mpatches.FancyBboxPatch((0.9, 0.1),
                         abs(0.3), abs(0.5),
                         boxstyle="round, pad=0.1",
                         fc=rgba,
                         ec=rgba,
                         label = 'sign.',
                         linewidth = 0.5)   
p_fancy3 = mpatches.FancyBboxPatch((0.9, 0.1),
                         abs(0.3), abs(0.5),
                         boxstyle="round, pad=0.1",
                         fc=rgba2,
                         ec=rgba2,
                         label = 'non-sign.',
                         linewidth = 0.5)    
#
#plt.legend(handles=[p_fancy], fontsize=cexx-2,
#           bbox_to_anchor=(0.02, 2.5), loc=2, borderaxespad=0.)       
plt.legend(handles=[p_fancy,p_fancy2,p_fancy3], fontsize=cexx-2,
           bbox_to_anchor=(1.03, 2), loc=2, borderaxespad=0.) 


### --------------------------------
plt.show()
fig.tight_layout()                
mm = mm + 1 ## model counter
pngfile='/Users/lenaboysen/Google Drive/LUMIP/Sigma_noise/ToE_'+\
                                var[vv]+'_ALL_'+noisetag+'.png'                                         
fig.savefig(pngfile,dpi=300, bbox_inches='tight',width=400,height=350)
plt.close()            
        

#%%

### =============================================
### PLOT ALL in One FoE
### =============================================  

var_min = -45


cmap = plt.cm.get_cmap('YlOrBr_r')
cmaplist = [cmap(i) for i in range(cmap.N)]
cmap = cmap.from_list('Custom cmap', cmaplist, cmap.N)
cmap.set_over('white')
        
fig_scale = 2
fig = plt.figure(figsize=(4*fig_scale,3*fig_scale))
gs1 = plt.GridSpec(4, 3, height_ratios=[1, 1, 1, 0.05])

gs1.update(wspace=0.05, hspace=0.025) 
  
i = 0; j = 0; m = 0
for model in models:#['MPI']:#
    print model + ' i = ' + str(i) + ' j = ' + str(j)
    
    #### created with time series scripts (or ToE_ts)
    if mistral == True:
        savepath = "/Users/lenaboysen/mistral/LUMIP/" 
        path="/Users/lenaboysen/mistral/LUMIP/"+model+"/"
    else:
        savepath = "/Users/lenaboysen/Documents/Crescendo_LUMIP/LUMIP/DATA/"
        path = "/Users/lenaboysen/Documents/Crescendo_LUMIP/LUMIP/DATA/"+model+"/"
    
    if add_members == True:
        file_out = savepath + var[vv] + '_' + model +'_ToE_newmembers.obj'
    else:
        file_out = savepath + var[vv] + '_' + model +'_ToE_'+noisetag+'.obj'
    if os.path.isfile(file_out):
        ToE_stats = pickle.load(open(file_out,'rb'))        
            
            
    axes = plt.subplot(gs1[j,i], 
                       projection=ccrs.Robinson(central_longitude=0.0))
    
    
    ToEplot = ToE_stats['FoE']*100
    ToEplotmasked = ToE_stats['ToE_masked']
        
#    plt.pcolormesh(ToE_stats['FoE']*100) 
#    plt.colorbar()
#    plt.show()
#            
    
    
    #       ### shade non-significant areas
    plot = ToEplot.T.plot.pcolormesh(transform = ccrs.PlateCarree(),
                                     add_colorbar=False,
                                     cmap=cmap, #alpha=0.3, #norm=norm, 
                                     vmin = -50, vmax = -1, 
                                     ax=axes)
    
    
    cp = ToE_stats['def_mask'].T.plot.contour(ax=axes,levels=1, 
                  transform = ccrs.PlateCarree(),
                  add_colorbar=False,
                  vmin = 0, vmax = 1,
                  colors='k', origin="lower" ,linewidths=0.5)
    
    axes.text(0.5,1.07,
        list(models_name[model])[0],
        color='slategrey',
        horizontalalignment='center',
        verticalalignment='center',
        transform=axes.transAxes,
        fontsize=cexx) #,
    
    axes.coastlines(color='grey',linewidth=0.5)
    axes.outline_patch.set_linewidth(0.25)
    axes.gridlines(color='grey',linewidth=0.5, alpha=0.5, linestyle='--')
                    
    i = i + 1
    
    if m in [2,5,8]:
        i = 0
        j = j + 1
    
    m = m + 1        
        

        
            
### --- Colorbar ----
axes = plt.subplot(gs1[3, :])
axes.set_visible(False)
#cbar_ax = fig.add_axes([0.15, 0.34, 0.6, 0.03]) ## x0, y0, width, height
cbar_ax = fig.add_axes([0.22, 0.1, 0.6, 0.02])

inc = 5
tcks = np.arange(myround(var_min, base=10),0+inc,inc) #-0.1
tcks = np.arange(-50,-1,5)
norm = mpl.colors.BoundaryNorm(tcks, cmap.N)         
cols = plt.cm.YlOrRd(np.linspace(0, 1, 12))[[11],:]
   


bar = mpl.colorbar.ColorbarBase(cbar_ax, cmap=cmap,
                        norm=norm,
                        boundaries=tcks,
                        ticks=tcks, 
                        extend='both',
                        spacing='uniform',
                        orientation='horizontal')


bar.ax.tick_params(labelsize=10,length=2, width=1)
#bar.set_label('$\Delta$FoE [10$^3$ km$^2$] of '+varinfo.loc[var[vv]]['name'],fontsize=14,labelpad=-50) 
bar.set_label('$\Delta$FoE [frac] of '+varinfo.loc[var[vv]]['name'],fontsize=14,labelpad=-40) 

for label in bar.ax.xaxis.get_ticklabels()[::2]:
    label.set_visible(False)

bar.ax.tick_params(labelsize=cexx)
bar.outline.set_visible(False)

### --- LEGEND -------------
import matplotlib.patches as mpatches
p_fancy = mpatches.FancyBboxPatch((0.9, 0.1),
                         abs(0.3), abs(0.5),
                         boxstyle="round, pad=0.1",
                         fc='white',
                         ec='black',
                         label = '$\Delta$F',
                         linewidth = 0.5)
p_fancy2 = mpatches.FancyBboxPatch((0.9, 0.1),
                         abs(0.3), abs(0.5),
                         boxstyle="round, pad=0.1",
                         fc=rgba,
                         ec=rgba,
                         label = 'sign.',
                         linewidth = 0.5)   
p_fancy3 = mpatches.FancyBboxPatch((0.9, 0.1),
                         abs(0.3), abs(0.5),
                         boxstyle="round, pad=0.1",
                         fc=rgba2,
                         ec=rgba2,
                         label = 'non-sign.',
                         linewidth = 0.5)    
#
plt.legend(handles=[p_fancy], fontsize=cexx-2,
           bbox_to_anchor=(1.03, 1.2), loc=2, borderaxespad=0.)       
#plt.legend(handles=[p_fancy,p_fancy2,p_fancy3], fontsize=cexx-2,
#           bbox_to_anchor=(0.72, 7), loc=2, borderaxespad=0.) 


### --------------------------------
plt.show()
fig.tight_layout()                
mm = mm + 1 ## model counter
pngfile='/Users/lenaboysen/Google Drive/LUMIP/Sigma_noise/FoE_'+\
                                var[vv]+'_ALL_'+noisetag+'.png'                                         
fig.savefig(pngfile,dpi=300, bbox_inches='tight',width=400,height=300)
plt.close()            






        
#%% DIFF FoE and DeltaF
       

### =============================================
### PLOT ALL in One FoE
### =============================================  
var = ['tas','gpp','cLand']

for vv in [0,1,2]:

    var_min = 0#.1 
    var_max = 45
    cmap = plt.cm.get_cmap('YlOrBr_r')
    cmaplist = [cmap(i) for i in range(cmap.N)][::-1]
    cmap = cmap.from_list('Custom cmap', cmaplist, cmap.N)
    cmap.set_under('white')
            
    fig_scale = 2
    fig = plt.figure(figsize=(4*fig_scale,3*fig_scale))
    gs1 = plt.GridSpec(4, 3, height_ratios=[1,1,1, 0.05])
    
    gs1.update(wspace=0.05, hspace=0.025) 
      
    i = 0; j = 0; m = 0
    for model in models:#['MPI']:#
        print model + ' i = ' + str(i) + ' j = ' + str(j)
        
        #### created with time series scripts (or ToE_ts)
        if mistral == True:
            savepath = "/Users/lenaboysen/mistral/LUMIP/" 
            path="/Users/lenaboysen/mistral/LUMIP/"+model+"/"
        else:
            savepath = "/Users/lenaboysen/Documents/Crescendo_LUMIP/LUMIP/DATA/"
            path = "/Users/lenaboysen/Documents/Crescendo_LUMIP/LUMIP/DATA/"+model+"/"
        
    if add_members == True:
        file_out = savepath + var[vv] + '_' + model +'_ToE_newmembers.obj'
    else:
        file_out = savepath + var[vv] + '_' + model +'_ToE_'+noisetag+'.obj'
        if os.path.isfile(file_out):
            ToE_stats = pickle.load(open(file_out,'rb'))  
            
        file_out = savepath + 'def_mask_regions.obj'
        if os.path.isfile(file_out):
            print 'loading def mask for ' +  model
            def_mask_regions = pickle.load(open(file_out,'rb'))
            def_mask = def_mask_regions[model][model+'_defmask_glob']
            diff = def_mask_regions[model][model+'_deffrac_glob']
            cellarea = def_mask_regions[model][model+'_cellarea']
    #            lat = deffrac.lat
            
        diffplot = -diff.where(diff < -0.1)
        defarea = -diff* cellarea /100/ 10**9 #10**3 km2            
                
        
        axes = plt.subplot(gs1[j,i], 
                           projection=ccrs.Robinson(central_longitude=0.0))
        
        
        ToEplot = diffplot - ToE_stats['FoE']*(-1)*100
    #    
    #    plt.pcolormesh(diffplot - ToE_stats['FoE']*(-1)*100) 
    #    plt.colorbar()
    #    plt.show()    
    #    
      
        #       ### shade non-significant areas
        plot = ToEplot.T.plot.pcolormesh(transform = ccrs.PlateCarree(),
                                         add_colorbar=False,
                                         cmap=cmap, #alpha=0.3, #norm=norm, 
                                         vmin = var_min, vmax = var_max, 
                                         ax=axes)
        
        
        cp = ToE_stats['def_mask'].T.plot.contour(ax=axes,levels=1, 
                      transform = ccrs.PlateCarree(),
                      add_colorbar=False,
                      vmin = 0, vmax = 1,
                      colors='k', origin="lower" ,linewidths=0.5)
        
        axes.text(0.5,1.07,
            list(models_name[model])[0],
            color='slategrey',
            horizontalalignment='center',
            verticalalignment='center',
            transform=axes.transAxes,
            fontsize=cexx) #,
        
        axes.coastlines(color='grey',linewidth=0.5)
        axes.outline_patch.set_linewidth(0.25)
        axes.gridlines(color='grey',linewidth=0.5, alpha=0.5, linestyle='--')
                        
        i = i + 1
        
        if m in [2,5,8]:
            i = 0
            j = j + 1
        
        m = m + 1      
            
                
    ### --- Colorbar ----
    axes = plt.subplot(gs1[3, :])
    axes.set_visible(False)
    cbar_ax = fig.add_axes([0.12, 0.34, 0.6, 0.03]) ## x0, y0, width, height
    
    inc = 5
    #tcks = np.arange(myround(var_min, var_max, base=10),0+inc,inc) #-0.1
    var_min = 0
    tcks = np.arange(var_min,var_max,inc)
    bnds = np.arange(var_min,var_max+inc,inc)
    bar = mpl.colorbar.ColorbarBase(cbar_ax, cmap=cmap,
                            boundaries=bnds,
                            ticks=tcks, 
                            extend='max',
                            spacing='uniform',
                            orientation='horizontal')
    
    
    bar.ax.tick_params(labelsize=14)
    #bar.set_label('$\Delta$F - $\Delta$FoE [10$^3$ km$^2$] of '+varinfo.loc[var[vv]]['name'],fontsize=14,labelpad=-50)    
    bar.set_label('$\Delta$F - FoE [frac] of '+varinfo.loc[var[vv]]['name'],fontsize=14,labelpad=-50)    
    
    #for label in bar.ax.xaxis.get_ticklabels()[::2]:
    #    label.set_visible(False)
    
    bar.ax.tick_params(labelsize=cexx)
    bar.outline.set_visible(False)
    
    ### --- LEGEND -------------
    import matplotlib.patches as mpatches
    p_fancy = mpatches.FancyBboxPatch((0.9, 0.1),
                             abs(0.3), abs(0.5),
                             boxstyle="round, pad=0.1",
                             fc='white',
                             ec='black',
                             label = '$\Delta$F',
                             linewidth = 0.5)
    p_fancy2 = mpatches.FancyBboxPatch((0.9, 0.1),
                             abs(0.3), abs(0.5),
                             boxstyle="round, pad=0.1",
                             fc=rgba,
                             ec=rgba,
                             label = 'sign.',
                             linewidth = 0.5)   
    p_fancy3 = mpatches.FancyBboxPatch((0.9, 0.1),
                             abs(0.3), abs(0.5),
                             boxstyle="round, pad=0.1",
                             fc=rgba2,
                             ec=rgba2,
                             label = 'non-sign.',
                             linewidth = 0.5)    
    #
    plt.legend(handles=[p_fancy], fontsize=cexx-2,
               bbox_to_anchor=(1.03, 1.2), loc=2, borderaxespad=0.)       
    #plt.legend(handles=[p_fancy,p_fancy2,p_fancy3], fontsize=cexx-2,
    #           bbox_to_anchor=(0.72, 7), loc=2, borderaxespad=0.) 
    
    
    ### --------------------------------
    plt.show()
    fig.tight_layout()                
    mm = mm + 1 ## model counter
    pngfile='/Users/lenaboysen/Google Drive/LUMIP/Sigma_noise/FoE_'+\
                                    var[vv]+'_DiffDF_ALL_'+noisetag+'.png'                                         
    fig.savefig(pngfile,dpi=300, bbox_inches='tight',width=400,height=350)
    plt.close()  


        #%%
        ### =============================================
        ### PLOT ToE
        ### =============================================  
        cexx2 = 14
        cexx = 12
        
        var_min = 0
        var_max = 150 #meta.loc[model]['imax'] ###--> set the same for all models
        
        lons = whitelinefix(lon)
        lons, lats = np.meshgrid(lons, lat)
        
        ### --- COLORBAR SPECIFICATIONS ----*
        inc = 10
        tcks = np.arange(myround(var_min,base=20),var_max+inc,inc) #-0.1
#        tcks = tcks.insert(8, 81) 
        tcks2 = np.append(tcks[0:9], 81)
        tcks3 = np.append(tcks2, tcks[9:len(tcks)])
        tcks = tcks3
        
        
#        for j in range(128,138):
        colors1a = plt.cm.YlOrRd(np.linspace(0, 1, 12))[[11],:]
        colors1 = plt.cm.YlGnBu(np.linspace(0., 1, 12))[[0,1,2,5,6,7,8,11],:]
        colors1b = [0.6, 0.6, 0.6, 1.]  ## light grey
        
        colors2 = plt.cm.YlOrRd(np.linspace(0, 1, 12))[[0,1,2,3,4,5,6,8,9,11],:]
        colors2b = [128/256, 0., 0., 1.]   ## dark red     
        
        # combine them and build a new colormap
        cols = np.vstack((colors2, colors1, colors1b))
        cmap = mpl.colors.LinearSegmentedColormap.from_list('my_colormap', cols)        
        
        # extract all colors from the .jet map
        cmaplist = [cmap(i) for i in range(cmap.N)]
        #    # force the last color entry to be white
#            cmaplist[128] = colors1b  
#            cmaplist[248] = (0.8,0.8,0.8,1.0)
        
#        cmaplist[137] = colors1b  
        cmaplist[136] = colors1b  
        cmaplist[248] = (0.8,0.8,0.8,1.0)
        
        # create the new map
        cmap = cmap.from_list('Custom cmap', cmaplist, cmap.N)
        norm = mpl.colors.BoundaryNorm(tcks, cmap.N) 
        
        
        #    =================================================
        fig = plt.figure(figsize=(8, 3))
        ax = fig.add_axes([0.05, 0.80, 0.9, 0.15])
        bar = mpl.colorbar.ColorbarBase(ax, cmap=cmap,
                                boundaries=np.append(tcks,160),
                                extend='max',
                                ticks=tcks,
                                orientation='horizontal')
#            bar.set_label(j)

        rgba = (0.3,0.3,0.3,1.0) 
        rgba2 = (0.8,0.8,0.8,0.3) 
        ### --------------------------------*
    
    
        fig_scale = 2
        fig = plt.figure(figsize=(4*fig_scale,3*fig_scale))
        gs1 = plt.GridSpec(2, 1,height_ratios=[1, 0.05])        
        
        axes = plt.subplot(gs1[0,0], 
                           projection=ccrs.Robinson(central_longitude=0.0))

        ToEplot = ToE_stats['ToE']
        ToEplotmasked = ToE_stats['ToE_masked']
            
#       ### shade non-significant areas
        plot = ToEplot.T.plot.pcolormesh(transform = ccrs.PlateCarree(),
                                     add_colorbar=False,
                                     cmap=cmap,alpha=0.3, #norm=norm, 
                                     vmin = 0.0, vmax = var_max, 
                                     ax=axes)
        
        ### normal plotting of only significant areas
        plot = ToEplotmasked.T.plot.pcolormesh(transform = ccrs.PlateCarree(),
                                     add_colorbar=False,
                                     cmap=cmap, #norm=norm, 
                                     vmin = 0.0, vmax = var_max, 
                                     ax=axes)   
        

        cp = ToE_stats['def_mask'].T.plot.contour(ax=axes,levels=1, 
                      transform = ccrs.PlateCarree(),
                      add_colorbar=False,
                      vmin = 0, vmax = 1,
                      colors='k', origin="lower" ,linewidths=2)
        
        axes.coastlines(color='grey',linewidth=0.5)
        axes.outline_patch.set_linewidth(0.75)
    #    axes.set_title('ToE [years] of '+name,fontsize=14)
        axes.gridlines(color='grey',linewidth=0.5, alpha=0.5, linestyle='--')
        
    
    
    
    
        if i == 0:
            ax.text(0.05,0.95,
                list(models_name[model])[0],
                color='slategrey',
                horizontalalignment='left',
                verticalalignment='center',
                transform=ax.transAxes,
                fontsize=cexx2,
                bbox={'facecolor':'white', 'alpha':0.7, 'pad':5})
            
            
            ### --- LEGEND -------------
            import matplotlib.patches as mpatches
            p_fancy = mpatches.FancyBboxPatch((0.9, 0.1),
                                     abs(0.3), abs(0.5),
                                     boxstyle="round, pad=0.1",
                                     fc='white',
                                     ec='black',
                                     label = '$\Delta$F',
                                     linewidth = 2)
            p_fancy2 = mpatches.FancyBboxPatch((0.9, 0.1),
                                     abs(0.3), abs(0.5),
                                     boxstyle="round, pad=0.1",
                                     fc=rgba,
                                     ec=rgba,
                                     label = 'sign.',
                                     linewidth = 2)   
            p_fancy3 = mpatches.FancyBboxPatch((0.9, 0.1),
                                     abs(0.3), abs(0.5),
                                     boxstyle="round, pad=0.1",
                                     fc=rgba2,
                                     ec=rgba2,
                                     label = 'non-sign.',
                                     linewidth = 2)    
            
            plt.legend(handles=[p_fancy,p_fancy2,p_fancy3], fontsize=cexx-2,
                       bbox_to_anchor=(0.16, 0.03), loc=1, borderaxespad=0.)
        
        
        
        
        ### --- Colorbar ----
        axes = plt.subplot(gs1[1, :])
        axes.set_visible(False)
        
        if var[vv] == 'fFire':
            cbar_ax = fig.add_axes([0.1, 0.1, 0.8, 0.03]) ## x0, y0, width, height
        else:
            cbar_ax = fig.add_axes([0.1, 0.1, 0.8, 0.03])

        inc = 10
        tcks = np.arange(myround(var_min,base=20),var_max+inc,inc) 
        colors1a = plt.cm.YlOrRd(np.linspace(0, 1, 12))[[11],:]
        colors1 = plt.cm.YlGnBu(np.linspace(0., 1, 12))[[0,1,2,5,6,7,8,11],:]
        colors1b = [0.6, 0.6, 0.6, 1.]  ## light grey
        colors2 = plt.cm.YlOrRd(np.linspace(0, 1, 12))[[0,1,2,3,4,5,6,8,9,11],:]
   
        # combine them and build a new colormap
        cols = np.vstack((colors2, colors1, colors1b))
                
        cmap = mpl.colors.LinearSegmentedColormap.from_list('my_colormap', cols)        
        cmaplist = [cmap(i) for i in range(cmap.N)]
        cmaplist[248] = (0.8,0.8,0.8,1.0)
        cmap = cmap.from_list('Custom cmap', cmaplist, cmap.N)

        bar = mpl.colorbar.ColorbarBase(cbar_ax, cmap=cmap,
                                norm=norm,
                                boundaries=np.append(tcks,160),
                                ticks=tcks, 
                                extend='max',
                                spacing='uniform',
                                orientation='horizontal')
        
        
        bar.ax.tick_params(labelsize=14)
        bar.set_label('ToE [years] of '+varinfo.loc[var[vv]]['name'],fontsize=14,labelpad=-50)
#        bar.set_label(j,fontsize=14,labelpad=-50)        
        bar.outline.set_edgecolor('gray')
        
#        
        ### --------------------------------
        plt.show()
        fig.tight_layout()                
        mm = mm + 1 ## model counter
        pngfile='/Users/lenaboysen/Google Drive/LUMIP/Sigma_noise/ToE_'+\
                                        var[vv]+'_'+model+'.png'                                         
        fig.savefig(pngfile,dpi=300, bbox_inches='tight',width=400,height=350)
        plt.close()              
             
#    
    
    
    ### ================ FRACTION OF EMERGENCE ============================
    
        var_min = -40
        cmap = plt.cm.get_cmap('YlOrBr_r')
        cmaplist = [cmap(i) for i in range(cmap.N)]
        cmap = cmap.from_list('Custom cmap', cmaplist, cmap.N)
        cmap.set_over('white')
    
        fig_scale = 2
        fig = plt.figure(figsize=(4*fig_scale,3*fig_scale))
        gs1 = plt.GridSpec(2, 1,height_ratios=[1, 0.05])
        axes = plt.subplot(gs1[0,0], 
                           projection=ccrs.Robinson(central_longitude=0.0))
        
        
        plot = ToE_stats['FoE'].T.plot.pcolormesh(transform = ccrs.PlateCarree(),
                                     add_colorbar=False,
                                     cmap=cmap, #alpha=0.3, #norm=norm, 
                                     vmin = var_min, vmax = -0.1, 
                                     ax=axes)
        
        cp = ToE_stats['def_mask'].T.plot.contour(ax=axes,levels=1, 
                      transform = ccrs.PlateCarree(),
                      add_colorbar=False,
                      vmin = 0, vmax = 1,
                      colors='k', origin="lower" ,linewidths=2)
        
        axes.coastlines(color='grey',linewidth=0.5)
        axes.outline_patch.set_linewidth(0.75)
        axes.gridlines(color='grey',linewidth=0.5, alpha=0.5, linestyle='--')
        
        
        ax.text(0.05,0.95,
            list(models_name[model])[0],
            color='slategrey',
            horizontalalignment='left',
            verticalalignment='center',
            transform=ax.transAxes,
            fontsize=cexx2,
            bbox={'facecolor':'white', 'alpha':0.7, 'pad':5})
        
        
        ### --- LEGEND -------------
#        import matplotlib.patches as mpatches
        p_fancy = mpatches.FancyBboxPatch((0.9, 0.1),
                                 abs(0.3), abs(0.5),
                                 boxstyle="round, pad=0.1",
                                 fc='white',
                                 ec='black',
                                 label = '$\Delta$F',
                                 linewidth = 2)

        
        plt.legend(handles=[p_fancy], fontsize=cexx-2,
                   bbox_to_anchor=(0.16, 0.03), loc=1, borderaxespad=0.)
        
        
        
        ### --- Colorbar ----
        axes = plt.subplot(gs1[1, :])
        axes.set_visible(False)
        cbar_ax = fig.add_axes([0.1, 0.1, 0.8, 0.03])
        
        inc = 5
        tcks = np.arange(myround(var_min, base=20),0+inc,inc) #-0.1
        norm = mpl.colors.BoundaryNorm(tcks, cmap.N)         
        cols = plt.cm.YlOrRd(np.linspace(0, 1, 12))[[11],:]
   
    
    
        bar = mpl.colorbar.ColorbarBase(cbar_ax, cmap=cmap,
                                norm=norm,
                                boundaries=tcks,
                                ticks=tcks, 
                                extend='max',
                                spacing='uniform',
                                orientation='horizontal')
        
        
        bar.ax.tick_params(labelsize=14)
        bar.set_label('$\Delta$FoE [10$^3$ km$^2$] of '+varinfo.loc[var[vv]]['name'],fontsize=14,labelpad=-50)    
        bar.outline.set_edgecolor('gray')
        
        
        ### --------------------------------
        plt.show()    
        fig.tight_layout()                
        mm = mm + 1 ## model counter
        
        pngfile='/Users/lenaboysen/Google Drive/LUMIP/Sigma_noise/FoE_'+\
                                        var[vv]+'_'+model+'.png'                                         
        fig.savefig(pngfile,dpi=300, bbox_inches='tight',width=400,height=350)
        plt.close()       
    

### ================================================
### ================================================


    
