#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Sat Oct 12 15:36:05 2019

@author: lenaboysen

Analysis of non-linear relationship between dF and dTas or dcLand

based on scatter plots and regression


Data needed: LUMIP_MAP_read_save_data.py

"""


import os
import numpy as np
np.set_printoptions(threshold=100) # 'nan'
#import os.path
import sys as sys
import pickle

#import cdo
#cdo = cdo.Cdo()
#cdo.setCdo('/opt/local/bin/cdo')
import xarray as xr
#from matplotlib import gridspec
#from datetime import datetime, date, time, timedelta

#plot
import matplotlib.pyplot as plt
import matplotlib as mpl
import matplotlib.colors as mcolors
import seaborn as sns; sns.set(color_codes=True)

#import plotly.plotly as py
#import plotly.graph_objs as go
import pandas as pd
from scipy.optimize import curve_fit


new_path = '/Users/lenaboysen/Google Drive/python/'
if new_path not in sys.path:
    sys.path.append(new_path)
    
 


## Constants
kg_C_per_kg_CO2 = 0.272918
kg_C_perMol = 0.012011
C_to_K = 273.15
MPa_to_mm = 1022.8

#%% FUNCTIONS
            
def func_log(x, a, b, c):
  #return a * np.exp(-b * x) + c
  return a * np.log(b * x) + c  
def func_exp(x, a, b, c):
    #Return values from a general exponential function.
    return a * np.exp(b * x) + c



#%%

rr = 0
timstat = 'year'
timstr = 'yr'
typ = '_'

varinfo = pd.DataFrame({'var':['tas',
                               'pr',
                               'gpp',
                               'cLand',
                               'cVeg','cSoil','cLitter',
                               'cSoilFast','cSoilSlow','cSoilMedium',
                               'evspsbl','npp','rh','fFire'],
                     'name':['$\Delta$Near Surface Temperature',
                             '$\Delta$Precipitation',
                             '$\Delta$GPP',
                             '$\Delta$Total Land Carbon',
                             '$\Delta$Vegetation Carbon',
                             '$\Delta$Soil Carbon',
                             '$\Delta$Litter Carbon',
                             '$\Delta$Soil Carbon (fast)',
                             '$\Delta$Soil Carbon (slow)',
                             '$\Delta$Soil Carbon (medium)',
                             '$\Delta$Evapotranspiration',
                             '$\Delta$NPP',
                             '$\Delta$Het. respiration',
                             '$\Delta$Fire Emissions'] ,
                     'name_short': ['$\Delta$Tas',
                             '$\Delta$Pr',
                             '$\Delta$GPP',
                             '$\Delta$cLand',
                             '$\Delta$cVeg',
                             '$\Delta$cSoil',
                             '$\Delta$cLitter',
                             '$\Delta$cSoil (fast)',
                             '$\Delta$cSoil (slow)',
                             '$\Delta$cSoil (medium)',
                             '$\Delta$ET',
                             '$\Delta$NPP',
                             '$\Delta$rh',
                             '$\Delta$fFire'] ,
                     'unit': ['$^\circ$C',
                              'mm '+timstr+'$^{-1}$',
                              'GtC '+timstr+'$^{-1}$',
                              'GtC','GtC','GtC','GtC','GtC','GtC','GtC',
                              'mm '+timstr+'$^{-1}$',
                              'GtC '+timstr+'$^{-1}$','GtC '+timstr+'$^{-1}$','GtC '+timstr+'$^{-1}$'],
                     'rm':[30,30,10,10,10,10,10,10,10,10,
                           30,10,10,10],
                    'minmax':[2.5,500,1.0,20.0,
                              15,5,2,0.6,0.6,0.6,
                              400,0.5,0.5,0.25],
                    'minmax_scaled':[5.0,500,1.0,40.0,
                                     20,10,5,0.6,0.6,0.6,
                                     400,0.5,0.5,0.25]\
                               }).set_index('var')    


mistral = False          
regions = ['_glob','_soam','_africa','_sea_aust','_noam_temp','_noam_bor','_eura_bor','_eura_temp']
regions_names = ['Global','South America','Africa','SE Asia & Aust.','North Am. temp.','North Am. bor.','Eurasia bor.','Eurasia temp.']

models = ['MPI','IPSL','CESM2','Can','CNRM','BCC','MIROC','UKESM','EC']
models_name = pd.DataFrame({'IPSL': ['IPSL-CM6A-LR'],
                            'MPI': ['MPI-ESM-1.2.0'],
                             'CESM2': ['CESM2'],
                             'Can': ['CanESM5'],
                             'CNRM': ['CNRM-ESM2-1'],
                             'BCC': ['BCC-CSM2-MR'],
                             'MIROC': ['MIROC-ES2L'],
                             'UKESM':['UKESM1-0-LL'],
                             'EC':['EC-Earth3-Veg']})


timstat = 'year'

meta = pd.DataFrame({'model':['MPI','IPSL','CESM2',\
                              'Can','CNRM','BCC','MIROC','UKESM','EC'],
                     'imax':[151, 80,  80, 90, 80, 81,150, 80, 80],
                     'imaxctl': [90, 90, 90, 90, 90, 90,90,90, 90],
                     'nruns':[7,3, 3, 2, 1, 1, 1, 1, 1],
                     'runs': [['r1i1p1f1','r2i1p1f1','r3i1p1f1','r4i1p1f1',\
                               'r5i1p1f1','r6i1p1f1','r7i1p1f1'],\
                              ['r1i1p1f1','r2i1p1f1','r3i1p1f1'],\
                              ['r1i1p1f1','r2i1p1f1','r3i1p1f1'],\
                              ['r1i1p2f1'],\
                              ['r1i1p1f2'],\
                              ['r1i1p1f1'],\
                              ['r1i1p1f1'],\
                              ['r1i1p1f2'],\
                              ['r1i1p1f1']]\
                               }).set_index('model')
    
metaCTL = pd.DataFrame({'model':['MPI','IPSL','CESM2',\
                                 'Can','CNRM','BCC','MIROC','UKESM','EC'],
                     'imax':[151, 80,  80, 90, 80, 81, 90, 90, 80],
                     'imaxctl': [90, 90, 90, 90, 90, 90, 90, 90, 90],
                     'nruns':[ 7, 3, 1, 2, 1, 1, 1, 1, 1],
                     'runs': [['piControl'],
                              ['piControl'],
                              ['piControl'],\
                              ['piControl'],\
                              ['piControl'],
                              ['piControl'],
                              ['piControl'],
                              ['piControl'],
                              ['piControl']]}).set_index('model')

## index = rownames; select with meta.loc['IPSL']    
cols = pd.DataFrame({'IPSL': ['orchid','purple'],
                             'CESM': ['orchid','orchid'],
                              'MPI': ['indianred','indianred'],
                              'CESM2':['dodgerblue','dodgerblue'],
                              'Can': ['orange','orange'],
                              'BCC': ['mediumblue','mediumblue'],
                              'CNRM' : ['seagreen','seagreen'],
                              'MIROC' : ['gray','gray'],
                              'UKESM': ['pink','pink'],
                              'EC':['turquoise','turquoise']})      
cexx = 10
cexx2 = 12   

#%% DTas against final deffrac

var = ['tas','cLand']
for vv in [1]:
    
    if var[vv] == 'tas':
        tag  = 'Clim'
        unit = '$^{\circ}$C'
    else:
        tag = 'Carbon'
        unit = 'kg m$^{-2}$'
        
    rrix = [0]
    
    fig_scale = 2
    fig = plt.figure(figsize=(3*fig_scale,4*fig_scale))
    gs1 = plt.GridSpec(4,3, height_ratios=[1,1,1,0.05],width_ratios=[1,1,1])
    
    gs1.update(wspace=0.1, hspace=0.2) 
    i = 0; j = 0;  p = 0
    
    zero_lat = pd.DataFrame({'model':models})
    zero_GD_lat = pd.DataFrame({'model':models})

    for model in models:
        print model
        print 'i = ' + str(i) + ' j = ' + str(j)
        
        if model in ['Can','BCC'] and vv == 1:
            i = i+1
            p = p+1 
            if model == 'BCC':
                j =  j + 1
                i = 0
                
            continue
        
        
        ax = plt.subplot(gs1[j,i])
        plt.axhline(y=0, color='k', linestyle='-',linewidth= 1)   
                
        sns.set_context("notebook", font_scale=1.2, rc={"lines.linewidth": 1.2})
        sns.set_style("whitegrid")
        custom_style = {
                'grid.color': '0.8',
                'grid.linestyle': '--',
                'grid.linewidth': 0.5,
                }
        sns.set_style(custom_style)  
        if var[vv] == 'tas':
            ax.set_ylim(-4, 4)        
            yticks = np.arange(-4,4.5,0.5) 
        else:
            ax.set_ylim(-20, 4)        
            yticks = np.arange(-20,4.5,2.5) 

        
        
        xticks = np.arange(0,100,10)   
        ax.set_xlim(xticks[1],xticks[len(xticks)-1])                                
        plt.axvline(x=0, color='k', linestyle='-',linewidth= 1) 
    
        
        if mistral == True:
            savepath = "/Users/lenaboysen/mistral/LUMIP/" 
            path="/Users/lenaboysen/mistral/LUMIP/"+model+"/"
        else:
            savepath = "/Users/lenaboysen/Documents/Crescendo_LUMIP/LUMIP/DATA/"
            path = "/Users/lenaboysen/Documents/Crescendo_LUMIP/LUMIP/DATA/"+model+"/"
        
        file_out = savepath + var[vv] + '_' + model +'_map.obj'
        if os.path.isfile(file_out):
            ds = pickle.load(open(file_out,'rb'))
    
#   #   #   #                
        biome_GD = None; def_GD = None
        for rr in rrix: #range(0,8): 
            print regions[rr]
            def_file = savepath + 'def_mask_regions.obj'
            if os.path.isfile(def_file):
                print 'loading def mask for ' + model
                def_mask_regions = pickle.load(open(def_file,'rb'))
                def_mask = def_mask_regions[model][model+'_defmask'+regions[rr]]
                deffrac = def_mask_regions[model][model+'_deffrac'+regions[rr]]/100.*(-1)
                deffrac = deffrac.where(deffrac > 0, 0)
                cellarea = def_mask_regions[model][model+'_cellarea']
                reg = def_mask_regions[model][model+regions[rr]]
                init = def_mask_regions[model][model+'_init_forest']
                
            lon = deffrac.lon
            lat = deffrac.lat   
            latdiff = np.append(np.diff(lat),np.diff(lat)[0])   
            
            deffrac = deffrac.where(deffrac > 0.1, 0)  
            
                             
                            
            ds_GD = np.ma.masked_array(ds['var_masked'].copy(),xr.where(deffrac > 0, 0, 1)) 
            ds_GD = xr.Dataset({'var_masked':(('lon','lat'),ds_GD)},
                                        {'lon': lon, 'lat': lat}) 
               
            deffrac = deffrac.to_dataset(name='deffrac').where(deffrac>0, np.nan)*100
            
            if biome_GD is None:
                biome_GD = ds_GD
                def_GD = deffrac
            else: 
                biome_GD = xr.merge([biome_GD, ds_GD], join='outer')
                def_GD = xr.merge([def_GD, deffrac])
#   #   #   #                
        merge = xr.merge([biome_GD,def_GD])    
        merge = merge.where(~merge['deffrac'].isnull(), drop=True)
        
        ## -- global
        merge = merge.sel(lat=slice(-70,70))
        work1d = merge.stack(z=('lat', 'lon'))
        work_sort = work1d.sortby('deffrac') ## = deffrac
        df = work_sort.to_dataframe().dropna(axis=0).reset_index()
        
        ## -- tropical 
        mergetrop = merge.sel(lat=slice(-23,23))
        work1dtrop = mergetrop.stack(z=('lat', 'lon'))
        worktrop_sort = work1dtrop.sortby('deffrac') ## = deffrac
        dftrop = worktrop_sort.to_dataframe().dropna(axis=0).reset_index()        
        
        ## -- temperate
        mergetemp1 = merge.sel(lat=slice(23,50))
        mergetemp2 = merge.sel(lat=slice(-23,-50))
        mergetemp = xr.merge([mergetemp1,mergetemp2])
        work1dtemp = mergetemp.stack(z=('lat', 'lon'))
        worktemp_sort = work1dtemp.sortby('deffrac') ## = deffrac
        dftemp = worktemp_sort.to_dataframe().dropna(axis=0).reset_index()          
        
        ## -- boreal 
        mergebor = merge.sel(lat=slice(50,70))
        work1dbor = mergebor.stack(z=('lat', 'lon'))
        workbor_sort = work1dbor.sortby('deffrac') ## = deffrac
        dfbor = workbor_sort.to_dataframe().dropna(axis=0).reset_index()  
        

        
        vmax = 70
        vmin = -70
        cmap = 'terrain_r'#'gist_rainbow'
        
        c1 = plt.cm.Greens_r(np.linspace(0., 1, 10))[[2,4,6,7,8],:]
        c2 = plt.cm.RdPu(np.linspace(0, 1, 10))[[6,9],:]
        c2b = plt.cm.autumn(np.linspace(0, 1, 10))[[3,5],:]
        c3 = plt.cm.summer_r(np.linspace(0., 1, 10))[[1,4,8],:] 
        c4 = plt.cm.GnBu(np.linspace(0., 1, 10))[[7,9],:]
        c4 = [[0./255., 191./255., 225./255.,1.],
              [0./255., 0./255., 11./255.,1.]] #deepskyblue1, blue	
        colors = np.vstack((c1, c2, c2b, c3, c4))
        cmap = mcolors.LinearSegmentedColormap.from_list('my_colormap', colors)
        
    
        sc = df.plot.scatter(
                x = 'deffrac', y = 'var_masked',
                alpha=0.2,c='lat',edgecolors='none',colormap=cmap,
                vmax=vmax,vmin=vmin,colorbar=False, ax=ax)
        

        x = dftrop['deffrac']
        y = dftrop['var_masked']
        
        fit = np.poly1d(np.polyfit(x, y, 1))(x)
        plt.plot(x,fit,linestyle='-',color='red',label='tropics')  
    
        ### 2nd order fit
#        fit = np.poly1d(np.polyfit(x, y, 2))(x)
#        ax.plot(x,fit,linestyle='--',color='red',label='tropics')
#        popt, pcov = curve_fit(func_log, x, y)
#        ax.plot(x, func_log(x, *popt),linestyle='--', color='red', label = 'tropics')    
            
        x = dftemp['deffrac']
        y = dftemp['var_masked']
       
        fit = np.poly1d(np.polyfit(x, y, 1))(x)
        plt.plot(x,fit,linestyle='-',color='greenyellow',label='temperate')         
       
    ### 2nd order fit
#        fit = np.poly1d(np.polyfit(x, y, 2))(x)
#        ax.plot(x,fit,linestyle='--',color='greenyellow',label='temperate')
#        popt, pcov = curve_fit(func_log, x, y)
#        ax.plot(x, func_log(x, *popt),linestyle='--', color='greenyellow', label = 'temperate') 
            
        x = dfbor['deffrac']
        y = dfbor['var_masked']
      
        fit = np.poly1d(np.polyfit(x, y, 1))(x)
        plt.plot(x,fit,linestyle='-',color='blue',label='boreal') 
      
    ### 2nd order fit
#        fit = np.poly1d(np.polyfit(x, y, 2))(x)
#        ax.plot(x,fit,linestyle='--',color='blue',label='boreal')        
#        popt, pcov = curve_fit(func_log, x, y)
#        ax.plot(x, func_log(x, *popt),linestyle='--', color='blue', label = 'boreal')         
        
        
        handlesX, labelsX = ax.get_legend_handles_labels()
    
        ax.text(0.5,1.05,
                list(models_name[model])[0],
                color='grey',
                horizontalalignment='center',
                verticalalignment='center',
                transform=ax.transAxes,
                fontsize=cexx+2,
                bbox={'facecolor':'white', 'alpha':0.7, 'pad':5})
        
        ax.text(-0.05,1.05,['(a)','(b)','(c)','(d)','(e)','(f)','(g)','(h)','(i)'][p],
                color='k',
                horizontalalignment='left',
                verticalalignment='center',
                transform=ax.transAxes,
                fontsize=cexx,
                bbox={'facecolor':'white', 'alpha':0.7, 'pad':5})
    
        
        ax.set_yticks(yticks, minor=False)                
        plt.yticks(fontsize=cexx) 
        
        if var[vv] == 'tas':
            ix = 5
        else: 
            ix = 2
            
        for index, label in enumerate(ax.yaxis.get_ticklabels()):
            if index % ix != 0: # show only every 5th label
                label.set_visible(False)
                
        ax.set_xticks(xticks, minor=False) 
        plt.xticks(fontsize=cexx)
        ax.tick_params(axis='x', labelsize=cexx)
        
        for label in [ax.xaxis.get_ticklabels()[0]]:
                label.set_visible(False)    
        
        
        if p in range(0,6):
            dummy = [item.get_text() for item in ax.get_xticklabels()]
            ax.set_xticklabels(['']*len(dummy))
            ax.set_xlabel('',fontsize=cexx)
        else:
            for index, label in enumerate(ax.xaxis.get_ticklabels()):
                if index % 2 != 0: # show only every nth label
                    label.set_visible(False)
            
            ax.set_xlabel('$\Delta$F [%]',fontsize=cexx)
    
    
        if var[vv] == 'cLand':
            ix = [1,2,5,7,8]
        else:
            [1,2,4,5,7,8]
            
        if p in ix:
            dummy = [item.get_text() for item in ax.get_yticklabels()]
            ax.set_yticklabels(['']*len(dummy))
            ax.set_ylabel('',fontsize=cexx)
 
    
        if var[vv] == 'cLand': 
            ix = [0,4,6]
        else: 
            ix = [0,3,6]
            
        if p in ix:
            string = varinfo.loc[var[vv]]['name_short']+' ['+unit+']'
            ax.set_ylabel(string,fontsize=cexx2)
        
        
        # legend
        if p == 6:
            leg = ax.legend(handlesX, #np.take(handlesX,[0,2,4]),
                          labelsX, #np.take(labelsX,[0,2,4]),
                          ncol=1,
                          fontsize=8,
                          prop={'size':8},
                          title='',
                          loc='center left', 
                          bbox_to_anchor=(-0.5, -0.3), borderaxespad=0)
            for line in leg.get_lines():
                line.set_linewidth(2.0)   
        
        
        
        
        i = i + 1

        if p in [2, 5]:
            i = 0
            j = j + 1
        p = p + 1     
        
        
    ### --- Colorbar ----
    def myround(x, base=5):
        return int(base * round(float(x)/base))
    
    
    nt = 10
    #tcks = np.arange(myround(vmin), myround(vmax), nt)    
    tcks = [-70,-60,-50,-40,-30,-23,-10,0,10,23,30,40,50,60,70 ]
    
    axes = plt.subplot(gs1[3, :])
    axes.set_visible(False)
    #    cbar_ax = fig.add_axes([0.12, 0.1, 0.8, 0.03]) ## x0, y0, width, height
    cbar_ax = fig.add_axes([0.12, 0.05, 0.7, 0.02]) ## x0, y0, width, height 
    cbar = mpl.colorbar.ColorbarBase(cbar_ax, cmap=cmap,
                                     ticks=tcks, boundaries=tcks,
                       orientation='horizontal',extend='both')
    
    #        for label in bar.ax.xaxis.get_ticklabels()[::2]:
    #            label.set_visible(False)
    #        
    cbar.ax.tick_params(labelsize=10,length=2, width=1)
    cbar.outline.set_visible(False)   
    cbar.set_label('Latitude [deg]',fontsize=10,labelpad=-40)  

    fig.show()
    fig.tight_layout()
    pngfile='/Users/lenaboysen/Google Drive/LUMIP/nonlinear/'+tag+'_Deffrac_nonlinear_global_ALL.png' 
    fig.savefig(pngfile,dpi=300, bbox_inches='tight')
    plt.close()             

#%% DTas against final deffrac

var = ['tas','cLand']
for vv in [0]:
    
    if var[vv] == 'tas':
        tag  = 'Clim'
        unit = '$^{\circ}$C'
    else:
        tag = 'Carbon'
        unit = 'kg m$^{-2}$'
    
    
    biomes = ['tropics','temperate','boreal','global']
#   #   
    for bb in biomes:
        if bb == 'tropics':
            rrix = [0] #[1,2,3]
            latmin = -23
            latmax = 23   
        elif bb == 'temperate':
            rrix = [0] #[1,2,3]
            latmin = 23
            latmax = 50  
        elif bb == 'boreal':
            rrix = [0] #[1,2,3]
            latmin = 50
            latmax = 70              
        elif bb == 'global':
            rrix = [0] #[1,2,3]
            latmin = -50
            latmax = 70  
            
            
        fig_scale = 2
        fig = plt.figure(figsize=(3*fig_scale,4*fig_scale))
        gs1 = plt.GridSpec(4,3, height_ratios=[1,1,1,0.05],width_ratios=[1,1,1])
        
        gs1.update(wspace=0.1, hspace=0.2) 
        i = 0; j = 0;  p = 0
        
        zero_lat = pd.DataFrame({'model':models})
        zero_GD_lat = pd.DataFrame({'model':models})

        for model in models:
            print model
            print 'i = ' + str(i) + ' j = ' + str(j)
            
            if model in ['Can','BCC'] and vv == 1:
                i = i+1
                
                if model == 'BCC':
                    j = 0
                    
                continue
            
            
            ax = plt.subplot(gs1[j,i])
            plt.axhline(y=0, color='k', linestyle='-',linewidth= 1)   
                    
            sns.set_context("notebook", font_scale=1.2, rc={"lines.linewidth": 1.2})
            sns.set_style("whitegrid")
            custom_style = {
                    'grid.color': '0.8',
                    'grid.linestyle': '--',
                    'grid.linewidth': 0.5,
                    }
            sns.set_style(custom_style)  
            if var[vv] == 'tas':
                ax.set_ylim(-4, 4)        
                yticks = np.arange(-4,4.5,0.5) 
            else:
                ax.set_ylim(-20, 4)        
                yticks = np.arange(-20,4.5,2.5) 

            
            
            xticks = np.arange(0,100,10)   
            ax.set_xlim(xticks[1],xticks[len(xticks)-1])                                
            plt.axvline(x=0, color='k', linestyle='-',linewidth= 1) 
        
            
            if mistral == True:
                savepath = "/Users/lenaboysen/mistral/LUMIP/" 
                path="/Users/lenaboysen/mistral/LUMIP/"+model+"/"
            else:
                savepath = "/Users/lenaboysen/Documents/Crescendo_LUMIP/LUMIP/DATA/"
                path = "/Users/lenaboysen/Documents/Crescendo_LUMIP/LUMIP/DATA/"+model+"/"
            
            file_out = savepath + var[vv] + '_' + model +'_map.obj'
            if os.path.isfile(file_out):
                ds = pickle.load(open(file_out,'rb'))
        
#   #   #   #                
            biome_GD = None; def_GD = None
            for rr in rrix: #range(0,8): 
                print regions[rr]
                def_file = savepath + 'def_mask_regions.obj'
                if os.path.isfile(def_file):
                    print 'loading def mask for ' + model
                    def_mask_regions = pickle.load(open(def_file,'rb'))
                    def_mask = def_mask_regions[model][model+'_defmask'+regions[rr]]
                    deffrac = def_mask_regions[model][model+'_deffrac'+regions[rr]]/100.*(-1)
                    deffrac = deffrac.where(deffrac > 0, 0)
                    cellarea = def_mask_regions[model][model+'_cellarea']
                    reg = def_mask_regions[model][model+regions[rr]]
                    init = def_mask_regions[model][model+'_init_forest']
                    
                lon = deffrac.lon
                lat = deffrac.lat   
                latdiff = np.append(np.diff(lat),np.diff(lat)[0])   
                
                deffrac = deffrac.where(deffrac > 0.1, 0)  
                
                
                
                ## select only where init deffrac == initfrac; 
#                mask = deffrac.where(init-deffrac*100 < 40, 0 )
                ## 
                
#                plt.pcolormesh(init,vmin=0,vmax=100)
#                plt.colorbar()
#                plt.show()    
#                plt.pcolormesh(deffrac*100,vmin=0,vmax=100)
#                plt.colorbar()
#                plt.show()                    
#                
#                
#                plt.pcolormesh(init-deffrac*100,vmin=0,vmax=100)
#                plt.colorbar()
#                plt.show()                   
#                
#                plt.pcolormesh(mask)
#                plt.colorbar()
#                plt.show()                   
                                
                ds_GD = np.ma.masked_array(ds['var_masked'].copy(),xr.where(deffrac > 0, 0, 1)) 
                ds_GD = xr.Dataset({'var_masked':(('lon','lat'),ds_GD)},
                                            {'lon': lon, 'lat': lat}) 
                
#                dummy = init.copy()
#                dummy = dummy.where((dummy.lat > 50) & (dummy.lat < latmax),2)
#  
#                plt.pcolormesh(dummy.T)
#                plt.colorbar()
#                plt.show()    
                                 
                deffrac = deffrac.to_dataset(name='deffrac').where(deffrac>0, np.nan)*100
                
                if biome_GD is None:
                    biome_GD = ds_GD
                    def_GD = deffrac
                else: 
                    biome_GD = xr.merge([biome_GD, ds_GD], join='outer')
                    def_GD = xr.merge([def_GD, deffrac])
#   #   #   #                
            work = xr.merge([biome_GD,def_GD])    
            
            ### merge all tropical 
    #        plt.pcolormesh(work['var_masked'].T)
    #        plt.colorbar()
    #        plt.show()  
    #        plt.pcolormesh(work['reg'].T)
    #        plt.colorbar()
    #        plt.show()      
        
            work = work.sel(lat=slice(latmin,latmax))
            work = work.where(~work['deffrac'].isnull(), drop=True)
        
            ## ==== sort deffrac increasing, get according tas ===
            print 'Sort *** '
            ## turn into 1d array
            work1d = work.stack(z=('lat', 'lon'))
            work_sort = work1d.sortby('deffrac') ## = deffrac
            
            df_work = work_sort.to_dataframe().dropna().reset_index()
            
            vmax = latmax
            vmin = latmin
            cmap = 'gnuplot'
            sc = df_work.plot.scatter(
                    x = 'deffrac', y = 'var_masked',
                    alpha=0.2,c='lat',edgecolors='none',colormap=cmap,
                    vmax=vmax,vmin=vmin,colorbar=False, ax=ax)
            
            x = df_work['deffrac']
            y = df_work['var_masked']

            popt, pcov = curve_fit(func_log, x, y)
            ax.plot(x, func_log(x, *popt), 'pink', label = 'log fit')
  
            fit = np.poly1d(np.polyfit(x, y, 2))(x)
            ax.plot(x,fit,linestyle='-',color='cyan',label='2nd order fit')     

            fit = np.poly1d(np.polyfit(x, y, 1))(x)
            plt.plot(x,fit,linestyle='-',color='greenyellow',label='linreg')  
            
#            popt, pcov = curve_fit(func_exp, x, y)
#            ax.plot(x, func_exp(x, *popt), 'yellow', label = 'exp fit')
            

             
            handlesX, labelsX = ax.get_legend_handles_labels()
        
            ax.text(0.5,1.05,
                    list(models_name[model])[0],
                    color='grey',
                    horizontalalignment='center',
                    verticalalignment='center',
                    transform=ax.transAxes,
                    fontsize=cexx+2,
                    bbox={'facecolor':'white', 'alpha':0.7, 'pad':5})
            
            ax.text(-0.05,1.05,['(a)','(b)','(c)','(d)','(e)','(f)','(g)','(h)','(i)'][p],
                    color='k',
                    horizontalalignment='left',
                    verticalalignment='center',
                    transform=ax.transAxes,
                    fontsize=cexx,
                    bbox={'facecolor':'white', 'alpha':0.7, 'pad':5})
        
            
            ax.set_yticks(yticks, minor=False)                
            plt.yticks(fontsize=cexx) 
            
            if var[vv] == 'tas':
                ix = 5
            else: 
                ix = 2
                
            for index, label in enumerate(ax.yaxis.get_ticklabels()):
                if index % ix != 0: # show only every 5th label
                    label.set_visible(False)
                    
            ax.set_xticks(xticks, minor=False) 
            plt.xticks(fontsize=cexx)
            ax.tick_params(axis='x', labelsize=cexx)
            
            for label in [ax.xaxis.get_ticklabels()[0]]:
                    label.set_visible(False)    
            
            
            if p in range(0,6):
                dummy = [item.get_text() for item in ax.get_xticklabels()]
                ax.set_xticklabels(['']*len(dummy))
                ax.set_xlabel('',fontsize=cexx)
            else:
                for index, label in enumerate(ax.xaxis.get_ticklabels()):
                    if index % 2 != 0: # show only every nth label
                        label.set_visible(False)
                
                ax.set_xlabel('$\Delta$F [%]',fontsize=cexx)
        
            if p in [1,2,4,5,7,8]:
                dummy = [item.get_text() for item in ax.get_yticklabels()]
                ax.set_yticklabels(['']*len(dummy))
                ax.set_ylabel('',fontsize=cexx)
 
            if i in [0,3,6]:
                string = varinfo.loc[var[vv]]['name_short']+' ['+unit+']'
                ax.set_ylabel(string,fontsize=cexx2)
            
            ## legend
            if p == 6:
                leg = ax.legend(handlesX,
                              labelsX,
                              ncol=1,
                              fontsize=cexx-2,
                              prop={'size':cexx-2},
                              title='',
                              loc='center left', 
                              bbox_to_anchor=(-0.5, -0.5), borderaxespad=0)
                for line in leg.get_lines():
                    line.set_linewidth(2.0)    
            
            
            
            
            i = i + 1
    
            if p in [2, 5]:
                i = 0
                j = j + 1
            p = p + 1     
            
            
            
        ### --- Colorbar ----
        def myround(x, base=5):
            return int(base * round(float(x)/base))
        
        if bb != 'global':
            tcks = np.arange(myround(vmin)-5, myround(vmax)+2*5, 5)
        else:
            nt = 20
            tcks = np.arange(myround(vmin)-nt, myround(vmax)+nt/2*nt, nt)   
        
        axes = plt.subplot(gs1[3, :])
        axes.set_visible(False)
    #    cbar_ax = fig.add_axes([0.12, 0.1, 0.8, 0.03]) ## x0, y0, width, height
        cbar_ax = fig.add_axes([0.22, 0.05, 0.6, 0.02]) ## x0, y0, width, height 
        
        
        cbar = mpl.colorbar.ColorbarBase(cbar_ax, cmap=cmap,
                                         ticks=tcks, boundaries=tcks,
                           orientation='horizontal',extend='both')
    
#        for label in bar.ax.xaxis.get_ticklabels()[::2]:
#            label.set_visible(False)
#        
        cbar.ax.tick_params(labelsize=10,length=2, width=1)
        cbar.outline.set_visible(False)   
        cbar.set_label('Latitude [deg]',fontsize=10,labelpad=-40)  
    
        fig.show()
        fig.tight_layout()
        pngfile='/Users/lenaboysen/Google Drive/LUMIP/nonlinear/'+tag+'_Deffrac_nonlinear_'+bb+'_ALL.png' 
        fig.savefig(pngfile,dpi=300, bbox_inches='tight')
        plt.close()               
##  

#%% DTas against final deffrac; MEAN over regions (one plot per region with all models) --> manuscript
var = ['tas','cLand']
base = 'deffrac' #'deffrac' or 'init'


for vv in [0,1]:
     
    if var[vv] == 'tas':
        tag  = 'Clim'
        unit = '$^{\circ}$C'
    else:
        tag = 'Carbon'
        unit = 'kg m$^{-2}$'
       
    for stat in ['mean']: #'lin','poly','log']:#'mean',
            
        fig_scale = 2
        fig = plt.figure(figsize=(4*fig_scale,3*fig_scale))
        gs1 = plt.GridSpec(2,3, height_ratios=[1,1],width_ratios=[1,1,1])
    #    gs1.update(wspace=0.1, hspace=0.4) 
        gs1.update(wspace=0.5, hspace=0.5) 
        
        i = 0; j = 0;  p = 0
        
    #   #    
        biomes = ['tropics','temperate','boreal']   
        for bb in np.arange(0,3):
            print '*** ' + biomes[bb] + ' ***'
            if biomes[bb] == 'tropics':
                rrix = [0] #[1,2,3]
                latmin = -23
                latmax = 23   
                col = 'red'
            elif biomes[bb] == 'temperate':
                rrix = [0] #[1,2,3]
                latmin = 23
                latmax = 50  
                col = 'green'
            elif biomes[bb] == 'boreal':
                rrix = [0] #[1,2,3]
                latmin = 50
                latmax = 90   
                col = 'blue'
            elif biomes[bb] == 'global':
                rrix = [0] #[1,2,3]
                latmin = -50
                latmax = 80  
                col = 'grey'
        
            ax = plt.subplot(gs1[j,i])
            plt.axhline(y=0, color='k', linestyle='-',linewidth= 1)   
                    
            sns.set_context("notebook", font_scale=1.2, rc={"lines.linewidth": 1.2})
            sns.set_style("whitegrid")
            custom_style = {
                    'grid.color': '0.8',
                    'grid.linestyle': '--',
                    'grid.linewidth': 0.5,
                    }
            sns.set_style(custom_style)  
            
            if var[vv] == 'tas':
                ax.set_ylim(-3,1.5)        
                yticks = np.arange(-3,2,0.5) 
            else:
                ax.set_ylim(-15, 2)        
                yticks = np.arange(-15,2,2.5) 
            
            if base == 'deffrac':
                xticks = np.arange(0,100,10)   
            else:
                xticks = np.arange(0,110,10) 
            ax.set_xlim(xticks[1],xticks[len(xticks)-1]) 
                               
            plt.axvline(x=0, color='k', linestyle='-',linewidth= 1) 
            
        
            for model in models:
                if (model == 'Can') and (var[vv] == 'cLand'):
                    continue
                if (model == 'BCC') and (var[vv] == 'cLand'):
                    continue                
                print model
    #            print 'i = ' + str(i) + ' j = ' + str(j)
      
                if mistral == True:
                    savepath = "/Users/lenaboysen/mistral/LUMIP/" 
                    path="/Users/lenaboysen/mistral/LUMIP/"+model+"/"
                else:
                    savepath = "/Users/lenaboysen/Documents/Crescendo_LUMIP/LUMIP/DATA/"
                    path = "/Users/lenaboysen/Documents/Crescendo_LUMIP/LUMIP/DATA/"+model+"/"
                
                file_out = savepath + var[vv] + '_' + model +'_map.obj'
                if os.path.isfile(file_out):
                    ds = pickle.load(open(file_out,'rb'))
            
    #   #   #   #               
                biome_GD = None; def_GD = None
                rr = 0
    #            print regions[rr]
                def_file = savepath + 'def_mask_regions.obj'
                if os.path.isfile(def_file):
    #                print 'loading def mask for ' + model
                    def_mask_regions = pickle.load(open(def_file,'rb'))
                    def_mask = def_mask_regions[model][model+'_defmask'+regions[rr]]
                    deffrac = def_mask_regions[model][model+'_deffrac'+regions[rr]]/100.*(-1)
                    deffrac = deffrac.where(deffrac > 0, 0)
                    cellarea = def_mask_regions[model][model+'_cellarea']
                    reg = def_mask_regions[model][model+regions[rr]]
                    init = def_mask_regions[model][model+'_init_forest']/100.
                    ## no weighting needed since dTas / dF
                    
#                plt.pcolormesh(init)
#                plt.colorbar()
#                plt.show()    
                    
                lon = deffrac.lon
                lat = deffrac.lat   
                latdiff = np.append(np.diff(lat),np.diff(lat)[0])   
                
                deffrac = deffrac.where(deffrac > 0.1, 0)  
                
                ds_GD = np.ma.masked_array(ds['var_masked'].copy(),xr.where(deffrac > 0, 0, 1)) 
                ds_GD = xr.Dataset({'var_masked':(('lon','lat'),ds_GD)},
                                            {'lon': lon, 'lat': lat}) 

                if base == 'deffrac': 
                    deffrac = deffrac.to_dataset(name='deffrac').where(deffrac>0, np.nan)*100
                elif base == 'init':
                    init = init.where(deffrac > 0.1, 0) 
                    deffrac = init.to_dataset(name='deffrac').where(deffrac>0, np.nan)*100
                
                
                if biome_GD is None:
                    biome_GD = ds_GD
                    def_GD = deffrac
                else: 
                    biome_GD = xr.merge([biome_GD, ds_GD], join='outer')
                    def_GD = xr.merge([def_GD, deffrac])
    #   #   #   #         
                work_org = xr.merge([biome_GD,def_GD])    
            
                ### 
                if stat == 'mean':
                    work = work_org.sel(lat=slice(latmin,latmax))
                    
                    label = biomes[bb] 
                    if biomes[bb] == 'temperate':
                        work2 = work_org.sel(lat=slice(-latmax,-latmin))
                        work = xr.merge([work,work2])
                        label = model
                        
                    work = work.where(~work['deffrac'].isnull(), drop=True)                    
                    
                    # for every biome, bin into deffrac groups, mean // color could be lat
                    work_bin = work.groupby_bins('deffrac',np.arange(10,100,10)).mean() #.groups # .fillna(0.0)
                    
                    print model + ' Min = ' + str(round(work_bin['var_masked'].min(skipna= True),2))
                    print model + ' Max = ' + str(round(work_bin['var_masked'].max(skipna= True),2))
                    print model + ' Mean =' + str(round(work_bin['var_masked'].mean(skipna= True),2))
                    print '==================='
                    
                    cmap = 'gnuplot'
                    plot = work_bin.to_dataframe().plot.scatter(
                            x = 'deffrac', y = 'var_masked',ax=ax,c=cols[model][1],
                            alpha=0.7,label = list(models_name[model])[0],s=10), 
                    work_bin.to_dataframe().plot.line(x='deffrac', y='var_masked', 
                                         ax=ax, color = cols[model][1],
                                         label = list(models_name[model])[0])
                    ax.get_legend().remove()
                    
                else:
                    work = work_org.sel(lat=slice(latmin,latmax))
                    
                    work = work.where(~work['deffrac'].isnull(), drop=True)
                    
                    ## ==== sort deffrac increasing, get according tas ===
                    print 'Sort *** '
                    ## turn into 1d array
                    work1d = work.stack(z=('lat', 'lon'))
                    work_sort = work1d.sortby('deffrac') ## = deffrac
                    
                    df_work = work_sort.to_dataframe().dropna().reset_index()
    
                    x = df_work['deffrac']
                    y = df_work['var_masked']
        
                    if stat == 'log':
                        popt, pcov = curve_fit(func_log, x, y)
                        ax.plot(x, func_log(x, *popt), cols[model][1], 
                                label = list(models_name[model])[0])
                    elif stat == 'poly':
                        fit = np.poly1d(np.polyfit(x, y, 2))(x)
                        ax.plot(x,fit,linestyle='-',color=cols[model][1],
                                label=list(models_name[model])[0]) 
                    elif stat == 'lin':
                        fit = np.poly1d(np.polyfit(x, y, 1))(x)
                        plt.plot(x,fit,linestyle='-',color=cols[model][1],
                                 label = list(models_name[model])[0]) 
                
                
            ax.text(0.5,1.05,
                    ['tropics','temperate','boreal'][bb],
                    color='grey',
                    horizontalalignment='center',
                    verticalalignment='center',
                    transform=ax.transAxes,
                    fontsize=cexx+2,
                    bbox={'facecolor':'white', 'alpha':0.7, 'pad':5})
            
            ax.text(-0.05,1.05,['(a)','(b)','(c)'][p],
                    color='k',
                    horizontalalignment='left',
                    verticalalignment='center',
                    transform=ax.transAxes,
                    fontsize=cexx,
                    bbox={'facecolor':'white', 'alpha':0.7, 'pad':5})
            
            if p == 2:
                handles, labels = ax.get_legend_handles_labels()
                if var[vv] == 'tas':
                    indices = np.arange(0,9,1)
                else: # remove Can & BCC
                    indices = np.arange(0,7,1)
                leg = ax.legend(np.take(handles,indices),
                              np.take(labels,indices),
                              ncol=1,
                              fontsize=cexx2-2,
                              prop={'size':8},
                              title='',
                              loc='center left', 
                              bbox_to_anchor=(1.1, 0.5), borderaxespad=0)
                for line in leg.get_lines():
                    line.set_linewidth(2.0)
                
                
            ax.set_yticks(yticks, minor=False)                
            plt.yticks(fontsize=cexx) 
            
            for index, label in enumerate(ax.yaxis.get_ticklabels()):
                if index % 2 != 0: # show only every 5th label
                    label.set_visible(False)
                    
            ax.set_xticks(xticks, minor=False) 
            plt.xticks(fontsize=cexx)
            ax.tick_params(axis='x', labelsize=cexx)
            
            for label in [ax.xaxis.get_ticklabels()[0]]:
                    label.set_visible(False)    
            
           
            for index, label in enumerate(ax.xaxis.get_ticklabels()):
                if index % 2 != 0: # show only every nth label
                    label.set_visible(False)
                    
            if base == 'deffrac':
                ax.set_xlabel('$\Delta$F [%]',fontsize=cexx)
            else:
                ax.set_xlabel('Initital forest fraction [%]',fontsize=cexx)
        
            if p in [1,2]:
                dummy = [item.get_text() for item in ax.get_yticklabels()]
                ax.set_yticklabels(['']*len(dummy))
                ax.set_ylabel('',fontsize=cexx)
     
            if i in [0]:
                string = varinfo.loc[var[vv]]['name_short']+' ['+unit+']'
                ax.set_ylabel(string,fontsize=cexx2)
            
            
            i = i + 1
    
            if p in [2, 5]:
                i = 0
                j = j + 1
            p = p + 1     
            
    
    #    plt.subplots_adjust(left=0.11, right=0.95, top=0.8, bottom=0.1)
        
        fig.show()
        fig.tight_layout()
        
        pngfile='/Users/lenaboysen/Google Drive/LUMIP/nonlinear/'+tag+'_'+base+'_nonlinear_'+stat+'_ALL2.png' 
        fig.savefig(pngfile,dpi=300, bbox_inches='tight')
        plt.close()               
##  




#%% DTas against initail frac
var = ['tas']
for vv in [0]:
    
    biomes = ['tropics','temperate','boreal']
#   #   
    for bb in ['tropics','temperate','boreal']: #biomes:
        if bb == 'tropics':
            rrix = [0] #[1,2,3]
            latmin = -23
            latmax = 23   
        elif bb == 'temperate': 
            rrix = [0] #[1,2,3]
            latmin = 23
            latmax = 50  
        elif bb == 'boreal':
            rrix = [0] #[1,2,3]
            latmin = 50
            latmax = 90              

        fig_scale = 2
        fig = plt.figure(figsize=(3*fig_scale,4*fig_scale))
        gs1 = plt.GridSpec(4,3, height_ratios=[1,1,1,0.05],width_ratios=[1,1,1])
        
        gs1.update(wspace=0.1, hspace=0.2) 
        i = 0; j = 0;  p = 0
        
        zero_lat = pd.DataFrame({'model':models})
        zero_GD_lat = pd.DataFrame({'model':models})

        for model in models:
            print model
            print 'i = ' + str(i) + ' j = ' + str(j)
            
            ax = plt.subplot(gs1[j,i])
            plt.axhline(y=0, color='k', linestyle='-',linewidth= 1)   
                    
            sns.set_context("notebook", font_scale=1.2, rc={"lines.linewidth": 1.2})
            sns.set_style("whitegrid")
            custom_style = {
                    'grid.color': '0.8',
                    'grid.linestyle': '--',
                    'grid.linewidth': 0.5,
                    }
            sns.set_style(custom_style)  
            ax.set_ylim(-4, 4)        
            yticks = np.arange(-4,4.5,0.5) 
            
            
            xticks = np.arange(0,100,10)   
            ax.set_xlim(xticks[1],xticks[len(xticks)-1])                                
            plt.axvline(x=0, color='k', linestyle='-',linewidth= 1) 
        
            
            if mistral == True:
                savepath = "/Users/lenaboysen/mistral/LUMIP/" 
                path="/Users/lenaboysen/mistral/LUMIP/"+model+"/"
            else:
                savepath = "/Users/lenaboysen/Documents/Crescendo_LUMIP/LUMIP/DATA/"
                path = "/Users/lenaboysen/Documents/Crescendo_LUMIP/LUMIP/DATA/"+model+"/"
            
            file_out = savepath + var[0] + '_' + model +'_map.obj'
            if os.path.isfile(file_out):
                ds = pickle.load(open(file_out,'rb'))
        
#   #   #   #                
            biome_GD = None; def_GD = None
            for rr in rrix: #range(0,8): 
                print regions[rr]
                def_file = savepath + 'def_mask_regions.obj'
                if os.path.isfile(def_file):
                    print 'loading def mask for ' + model
                    def_mask_regions = pickle.load(open(def_file,'rb'))
                    def_mask = def_mask_regions[model][model+'_defmask'+regions[rr]]
                    deffrac = def_mask_regions[model][model+'_deffrac'+regions[rr]]/100.*(-1)
                    deffrac = deffrac.where(deffrac > 0, 0)
                    cellarea = def_mask_regions[model][model+'_cellarea']
                    reg = def_mask_regions[model][model+'_init_forest']
                    
#                plt.pcolormesh(reg.T)
#                plt.colorbar()
#                plt.show()    
                    
                lon = deffrac.lon
                lat = deffrac.lat   
                latdiff = np.append(np.diff(lat),np.diff(lat)[0])   
                
                reg = reg.where(deffrac > 0.1, 0)  
                
                ds_GD = np.ma.masked_array(ds['var_masked'].copy(),xr.where(deffrac > 0, 0, 1)) 
                ds_GD = xr.Dataset({'var_masked':(('lon','lat'),ds_GD)},
                                            {'lon': lon, 'lat': lat}) 
                
        #                plt.pcolormesh(ds_GD['var_masked'].T)
        #                plt.colorbar()
        #                plt.show()                 
                
                reg = reg.to_dataset(name='def').where(deffrac>0, np.nan)
#                plt.pcolormesh(reg['reg'].T)
#                plt.colorbar()
#                plt.show()                 
                
                if biome_GD is None:
                    biome_GD = ds_GD
                    def_GD = reg
                else: 
                    biome_GD = xr.merge([biome_GD, ds_GD], join='outer')
                    def_GD = xr.merge([def_GD, reg])
#   #   #   #                
            work = xr.merge([biome_GD,def_GD])    
            
            ### merge all tropical 
    #        plt.pcolormesh(work['var_masked'].T)
    #        plt.colorbar()
    #        plt.show()  
    #        plt.pcolormesh(work['reg'].T)
    #        plt.colorbar()
    #        plt.show()      
        
            work = work.sel(lat=slice(latmin,latmax))
            work = work.where(~work['def'].isnull(), drop=True)
        
            ## ==== sort deffrac increasing, get according tas ===
            print 'Sort *** '
            ## turn into 1d array
            work1d = work.stack(z=('lat', 'lon'))
            work_sort = work1d.sortby('def') ## = deffrac
            
            vmax = latmax
            vmin = latmin
            cmap = 'gnuplot'
            sc = work_sort.to_dataframe().reset_index().plot.scatter(
                    x = 'def', y = 'var_masked',
                    alpha=0.4,c='lat',edgecolors='none',colormap=cmap,
                    vmax=vmax,vmin=vmin,ax=ax,colorbar=False) 

            
            ax.text(0.5,1.05,
                    list(models_name[model])[0],
                    color='grey',
                    horizontalalignment='center',
                    verticalalignment='center',
                    transform=ax.transAxes,
                    fontsize=cexx+2,
                    bbox={'facecolor':'white', 'alpha':0.7, 'pad':5})
            
            ax.text(-0.05,1.05,['(a)','(b)','(c)','(d)','(e)','(f)','(g)','(h)','(i)'][p],
                    color='k',
                    horizontalalignment='left',
                    verticalalignment='center',
                    transform=ax.transAxes,
                    fontsize=cexx,
                    bbox={'facecolor':'white', 'alpha':0.7, 'pad':5})
        
            
            ax.set_yticks(yticks, minor=False)                
            plt.yticks(fontsize=cexx) 
            
            for index, label in enumerate(ax.yaxis.get_ticklabels()):
                if index % 5 != 0: # show only every 5th label
                    label.set_visible(False)
                    
            ax.set_xticks(xticks, minor=False) 
            plt.xticks(fontsize=cexx)
            ax.tick_params(axis='x', labelsize=cexx)
            
            for label in [ax.xaxis.get_ticklabels()[0]]:
                    label.set_visible(False)    
            
            
            if p in range(0,6):
                dummy = [item.get_text() for item in ax.get_xticklabels()]
                ax.set_xticklabels(['']*len(dummy))
                ax.set_xlabel('',fontsize=cexx)
            else:
                for index, label in enumerate(ax.xaxis.get_ticklabels()):
                    if index % 2 != 0: # show only every nth label
                        label.set_visible(False)
                
                ax.set_xlabel('init. F [%]',fontsize=cexx)
        
        
        
            if p in [1,2,4,5,7,8]:
                dummy = [item.get_text() for item in ax.get_yticklabels()]
                ax.set_yticklabels(['']*len(dummy))
                ax.set_ylabel('',fontsize=cexx)
 
            if i in [0,3,6]:
                ax.set_ylabel('$\Delta$Tas [$^\circ$C]',fontsize=cexx2)
            
            
            i = i + 1
    
            if p in [2, 5]:
                i = 0
                j = j + 1
            p = p + 1     
            
            
            
        ### --- Colorbar ----
        def myround(x, base=5):
            return int(base * round(float(x)/base))
        
        tcks = np.arange(myround(vmin)-5, myround(vmax)+2*5, 5)
        
        axes = plt.subplot(gs1[3, :])
        axes.set_visible(False)
    #    cbar_ax = fig.add_axes([0.12, 0.1, 0.8, 0.03]) ## x0, y0, width, height
        cbar_ax = fig.add_axes([0.22, 0.05, 0.6, 0.02]) ## x0, y0, width, height 
        
        
        cbar = mpl.colorbar.ColorbarBase(cbar_ax, cmap=cmap,
                                         ticks=tcks, boundaries=tcks,
                           orientation='horizontal',extend='both')
    
#        for label in bar.ax.xaxis.get_ticklabels()[::2]:
#            label.set_visible(False)
#        
        cbar.ax.tick_params(labelsize=10,length=2, width=1)
        cbar.outline.set_visible(False)   
        cbar.set_label('Latitude [deg]',fontsize=10,labelpad=-40)  
    
        fig.show()
        fig.tight_layout()
        pngfile='/Users/lenaboysen/Google Drive/LUMIP/nonlinear/Clim_Frac_nonlinear_'+bb+'_ALL.png' 
        fig.savefig(pngfile,dpi=300, bbox_inches='tight')
        plt.close()  
        
        
        
#%% relative change in dF = (init-def) / init *100

var = ['tas','cLand']
for vv in [0]:
    
    if var[vv] == 'tas':
        tag  = 'Clim'
        unit = '$^{\circ}$C'
    else:
        tag = 'Carbon'
        unit = 'kg m$^{-2}$'
    
    
    biomes = ['tropics'] #,'temperate','boreal','global']
#   #   
    for bb in biomes:
        if bb == 'tropics':
            rrix = [0] #[1,2,3]
            latmin = -23
            latmax = 23   
        elif bb == 'temperate':
            rrix = [0] #[1,2,3]
            latmin = 23
            latmax = 50  
        elif bb == 'boreal':
            rrix = [0] #[1,2,3]
            latmin = 50
            latmax = 70              
        elif bb == 'global':
            rrix = [0] #[1,2,3]
            latmin = -50
            latmax = 70  
            
            
        fig_scale = 2
        fig = plt.figure(figsize=(3*fig_scale,4*fig_scale))
        gs1 = plt.GridSpec(4,3, height_ratios=[1,1,1,0.05],width_ratios=[1,1,1])
        
        gs1.update(wspace=0.1, hspace=0.2) 
        i = 0; j = 0;  p = 0
        
        zero_lat = pd.DataFrame({'model':models})
        zero_GD_lat = pd.DataFrame({'model':models})

        for model in models:
            print model
            print 'i = ' + str(i) + ' j = ' + str(j)
            
            ax = plt.subplot(gs1[j,i])
            plt.axhline(y=0, color='k', linestyle='-',linewidth= 1)   
                    
            sns.set_context("notebook", font_scale=1.2, rc={"lines.linewidth": 1.2})
            sns.set_style("whitegrid")
            custom_style = {
                    'grid.color': '0.8',
                    'grid.linestyle': '--',
                    'grid.linewidth': 0.5,
                    }
            sns.set_style(custom_style)  
            if var[vv] == 'tas':
                ax.set_ylim(-4, 4)        
                yticks = np.arange(-4,4.5,0.5) 
            else:
                ax.set_ylim(-20, 4)        
                yticks = np.arange(-20,4.5,2.5) 

            
            
            xticks = np.arange(0,100,10)   
            ax.set_xlim(xticks[1],xticks[len(xticks)-1])                                
            plt.axvline(x=0, color='k', linestyle='-',linewidth= 1) 
        
            
            if mistral == True:
                savepath = "/Users/lenaboysen/mistral/LUMIP/" 
                path="/Users/lenaboysen/mistral/LUMIP/"+model+"/"
            else:
                savepath = "/Users/lenaboysen/Documents/Crescendo_LUMIP/LUMIP/DATA/"
                path = "/Users/lenaboysen/Documents/Crescendo_LUMIP/LUMIP/DATA/"+model+"/"
            
            file_out = savepath + var[vv] + '_' + model +'_map.obj'
            if os.path.isfile(file_out):
                ds = pickle.load(open(file_out,'rb'))
        
#   #   #   #                
            biome_GD = None; def_GD = None
            for rr in rrix: #range(0,8): 
                print regions[rr]
                def_file = savepath + 'def_mask_regions.obj'
                if os.path.isfile(def_file):
                    print 'loading def mask for ' + model
                    def_mask_regions = pickle.load(open(def_file,'rb'))
                    def_mask = def_mask_regions[model][model+'_defmask'+regions[rr]]
                    deffrac = def_mask_regions[model][model+'_deffrac'+regions[rr]]/100.*(-1)
                    deffrac = deffrac.where(deffrac > 0, 0)
                    cellarea = def_mask_regions[model][model+'_cellarea']
                    reg = def_mask_regions[model][model+regions[rr]]
                    init = def_mask_regions[model][model+'_init_forest']
                    
                lon = deffrac.lon
                lat = deffrac.lat   
                latdiff = np.append(np.diff(lat),np.diff(lat)[0])   
                
                deffrac = deffrac.where(deffrac > 0.1, 0)  
                
                dFrel = (init-deffrac*100)/init*100
                
                plt.pcolormesh(init,vmin=0,vmax=100)
                plt.colorbar()
                plt.show()    
                plt.pcolormesh(deffrac*100,vmin=0,vmax=100)
                plt.colorbar()
                plt.show()                    
#                
#                
                plt.pcolormesh(dFrel) #,vmin=35,vmax=45)
                plt.colorbar()
                plt.show()                   
                
#                plt.pcolormesh(mask)
#                plt.colorbar()
#                plt.show()                   
                                
                ds_GD = np.ma.masked_array(ds['var_masked'].copy(),xr.where(deffrac > 0, 0, 1)) 
                ds_GD = xr.Dataset({'var_masked':(('lon','lat'),ds_GD)},
                                            {'lon': lon, 'lat': lat}) 
                
#                dummy = init.copy()
#                dummy = dummy.where((dummy.lat > 50) & (dummy.lat < latmax),2)
#  
#                plt.pcolormesh(dummy.T)
#                plt.colorbar()
#                plt.show()    
                                 
                deffrac = deffrac.to_dataset(name='deffrac').where(deffrac>0, np.nan)*100
                
                if biome_GD is None:
                    biome_GD = ds_GD
                    def_GD = deffrac
                else: 
                    biome_GD = xr.merge([biome_GD, ds_GD], join='outer')
                    def_GD = xr.merge([def_GD, deffrac])
#   #   #   #                
            work = xr.merge([biome_GD,def_GD])    
            
            ### merge all tropical 
    #        plt.pcolormesh(work['var_masked'].T)
    #        plt.colorbar()
    #        plt.show()  
    #        plt.pcolormesh(work['reg'].T)
    #        plt.colorbar()
    #        plt.show()      
        
            work = work.sel(lat=slice(latmin,latmax))
            work = work.where(~work['deffrac'].isnull(), drop=True)
        
            ## ==== sort deffrac increasing, get according tas ===
            print 'Sort *** '
            ## turn into 1d array
            work1d = work.stack(z=('lat', 'lon'))
            work_sort = work1d.sortby('deffrac') ## = deffrac
            
            df_work = work_sort.to_dataframe().dropna().reset_index()
            
            vmax = latmax
            vmin = latmin
            cmap = 'gnuplot'
            sc = df_work.plot.scatter(
                    x = 'deffrac', y = 'var_masked',
                    alpha=0.2,c='lat',edgecolors='none',colormap=cmap,
                    vmax=vmax,vmin=vmin,colorbar=False, ax=ax)
            
            
            
            
            x = df_work['deffrac']
            y = df_work['var_masked']


            popt, pcov = curve_fit(func_log, x, y)
            ax.plot(x, func_log(x, *popt), 'pink', label = 'log fit')
  
            fit = np.poly1d(np.polyfit(x, y, 2))(x)
            ax.plot(x,fit,linestyle='-',color='cyan',label='2nd order fit')     

            fit = np.poly1d(np.polyfit(x, y, 1))(x)
            plt.plot(x,fit,linestyle='-',color='greenyellow',label='linreg')  
            
#            popt, pcov = curve_fit(func_exp, x, y)
#            ax.plot(x, func_exp(x, *popt), 'yellow', label = 'exp fit')
            

             
            handlesX, labelsX = ax.get_legend_handles_labels()
        
            ax.text(0.5,1.05,
                    list(models_name[model])[0],
                    color='grey',
                    horizontalalignment='center',
                    verticalalignment='center',
                    transform=ax.transAxes,
                    fontsize=cexx+2,
                    bbox={'facecolor':'white', 'alpha':0.7, 'pad':5})
            
            ax.text(-0.05,1.05,['(a)','(b)','(c)','(d)','(e)','(f)','(g)','(h)','(i)'][p],
                    color='k',
                    horizontalalignment='left',
                    verticalalignment='center',
                    transform=ax.transAxes,
                    fontsize=cexx,
                    bbox={'facecolor':'white', 'alpha':0.7, 'pad':5})
        
            
            ax.set_yticks(yticks, minor=False)                
            plt.yticks(fontsize=cexx) 
            
            if var[vv] == 'tas':
                ix = 5
            else: 
                ix = 2
                
            for index, label in enumerate(ax.yaxis.get_ticklabels()):
                if index % ix != 0: # show only every 5th label
                    label.set_visible(False)
                    
            ax.set_xticks(xticks, minor=False) 
            plt.xticks(fontsize=cexx)
            ax.tick_params(axis='x', labelsize=cexx)
            
            for label in [ax.xaxis.get_ticklabels()[0]]:
                    label.set_visible(False)    
            
            
            if p in range(0,6):
                dummy = [item.get_text() for item in ax.get_xticklabels()]
                ax.set_xticklabels(['']*len(dummy))
                ax.set_xlabel('',fontsize=cexx)
            else:
                for index, label in enumerate(ax.xaxis.get_ticklabels()):
                    if index % 2 != 0: # show only every nth label
                        label.set_visible(False)
                
                ax.set_xlabel('$\Delta$F [%]',fontsize=cexx)
        
            if p in [1,2,4,5,7,8]:
                dummy = [item.get_text() for item in ax.get_yticklabels()]
                ax.set_yticklabels(['']*len(dummy))
                ax.set_ylabel('',fontsize=cexx)
 
            if i in [0,3,6]:
                string = varinfo.loc[var[vv]]['name_short']+' ['+unit+']'
                ax.set_ylabel(string,fontsize=cexx2)
            
            ## legend
            if p == 6:
                leg = ax.legend(handlesX,
                              labelsX,
                              ncol=1,
                              fontsize=cexx-2,
                              prop={'size':cexx-2},
                              title='',
                              loc='center left', 
                              bbox_to_anchor=(-0.5, -0.5), borderaxespad=0)
                for line in leg.get_lines():
                    line.set_linewidth(2.0)    
            
            
            
            
            i = i + 1
    
            if p in [2, 5]:
                i = 0
                j = j + 1
            p = p + 1     
            
            
            
        ### --- Colorbar ----
        def myround(x, base=5):
            return int(base * round(float(x)/base))
        
        if bb != 'global':
            tcks = np.arange(myround(vmin)-5, myround(vmax)+2*5, 5)
        else:
            nt = 20
            tcks = np.arange(myround(vmin)-nt, myround(vmax)+nt/2*nt, nt)   
        
        axes = plt.subplot(gs1[3, :])
        axes.set_visible(False)
    #    cbar_ax = fig.add_axes([0.12, 0.1, 0.8, 0.03]) ## x0, y0, width, height
        cbar_ax = fig.add_axes([0.22, 0.05, 0.6, 0.02]) ## x0, y0, width, height 
        
        
        cbar = mpl.colorbar.ColorbarBase(cbar_ax, cmap=cmap,
                                         ticks=tcks, boundaries=tcks,
                           orientation='horizontal',extend='both')
    
#        for label in bar.ax.xaxis.get_ticklabels()[::2]:
#            label.set_visible(False)
#        
        cbar.ax.tick_params(labelsize=10,length=2, width=1)
        cbar.outline.set_visible(False)   
        cbar.set_label('Latitude [deg]',fontsize=10,labelpad=-40)  
    
        fig.show()
        fig.tight_layout()
        pngfile='/Users/lenaboysen/Google Drive/LUMIP/nonlinear/'+tag+'_DeffracREL_nonlinear_'+bb+'_ALL2.png' 
        fig.savefig(pngfile,dpi=300, bbox_inches='tight')
        plt.close()               
##          