#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Sat Oct 12 15:36:05 2019

@author: lenaboysen


Analysis of dTas against dPr over latitudes
Scatter plot and regression

Data needed: LUMIP_MAP_read_save_data.py


"""


import os
import numpy as np
np.set_printoptions(threshold=100) # 'nan'
#import os.path
import sys as sys
import pickle

#import cdo
#cdo = cdo.Cdo()
#cdo.setCdo('/opt/local/bin/cdo')
import xarray as xr
#from matplotlib import gridspec
#from datetime import datetime, date, time, timedelta

#plot
import matplotlib.pyplot as plt
import matplotlib as mpl
import matplotlib.colors as mcolors
import seaborn as sns; sns.set(color_codes=True)

#import plotly.plotly as py
#import plotly.graph_objs as go
import pandas as pd


new_path = '/Users/lenaboysen/Google Drive/python/'
if new_path not in sys.path:
    sys.path.append(new_path)
    

## Constants
kg_C_per_kg_CO2 = 0.272918
kg_C_perMol = 0.012011
C_to_K = 273.15
MPa_to_mm = 1022.8

#%% FUNCTIONS
            
def func_log(x, a, b, c):
  #return a * np.exp(-b * x) + c
  return a * np.log(b * x) + c  
def func_exp(x, a, b, c):
    #Return values from a general exponential function.
    return a * np.exp(b * x) + c



#%%

rr = 0
timstat = 'year'
timstr = 'yr'
typ = '_'

varinfo = pd.DataFrame({'var':['tas',
                               'pr',
                               'gpp',
                               'cLand',
                               'cVeg','cSoil','cLitter',
                               'cSoilFast','cSoilSlow','cSoilMedium',
                               'evspsbl','npp','rh','fFire'],
                     'name':['$\Delta$Near Surface Temperature',
                             '$\Delta$Precipitation',
                             '$\Delta$GPP',
                             '$\Delta$Total Land Carbon',
                             '$\Delta$Vegetation Carbon',
                             '$\Delta$Soil Carbon',
                             '$\Delta$Litter Carbon',
                             '$\Delta$Soil Carbon (fast)',
                             '$\Delta$Soil Carbon (slow)',
                             '$\Delta$Soil Carbon (medium)',
                             '$\Delta$Evapotranspiration',
                             '$\Delta$NPP',
                             '$\Delta$Het. respiration',
                             '$\Delta$Fire Emissions'] ,
                     'name_short': ['$\Delta$Tas',
                             '$\Delta$Pr',
                             '$\Delta$GPP',
                             '$\Delta$cLand',
                             '$\Delta$cVeg',
                             '$\Delta$cSoil',
                             '$\Delta$cLitter',
                             '$\Delta$cSoil (fast)',
                             '$\Delta$cSoil (slow)',
                             '$\Delta$cSoil (medium)',
                             '$\Delta$ET',
                             '$\Delta$NPP',
                             '$\Delta$rh',
                             '$\Delta$fFire'] ,
                     'unit': ['$^\circ$C',
                              'mm '+timstr+'$^{-1}$',
                              'GtC '+timstr+'$^{-1}$',
                              'GtC','GtC','GtC','GtC','GtC','GtC','GtC',
                              'mm '+timstr+'$^{-1}$',
                              'GtC '+timstr+'$^{-1}$','GtC '+timstr+'$^{-1}$','GtC '+timstr+'$^{-1}$'],
                     'rm':[30,30,10,10,10,10,10,10,10,10,
                           30,10,10,10],
                    'minmax':[2.5,500,1.0,20.0,
                              15,5,2,0.6,0.6,0.6,
                              400,0.5,0.5,0.25],
                    'minmax_scaled':[5.0,500,1.0,40.0,
                                     20,10,5,0.6,0.6,0.6,
                                     400,0.5,0.5,0.25]\
                               }).set_index('var')    


mistral = False          
regions = ['_glob','_soam','_africa','_sea_aust','_noam_temp','_noam_bor','_eura_bor','_eura_temp']
regions_names = ['Global','South America','Africa','SE Asia & Aust.','North Am. temp.','North Am. bor.','Eurasia bor.','Eurasia temp.']

models = ['MPI','IPSL','CESM2','Can','CNRM','BCC','MIROC','UKESM','EC']
models_name = pd.DataFrame({'IPSL': ['IPSL-CM6A-LR'],
                            'MPI': ['MPI-ESM-1.2.0'],
                             'CESM2': ['CESM2'],
                             'Can': ['CanESM5'],
                             'CNRM': ['CNRM-ESM2-1'],
                             'BCC': ['BCC-CSM2-MR'],
                             'MIROC': ['MIROC-ES2L'],
                             'UKESM':['UKESM1-0-LL'],
                             'EC':['EC-Earth3-Veg']})


timstat = 'year'

meta = pd.DataFrame({'model':['MPI','IPSL','CESM2',\
                              'Can','CNRM','BCC','MIROC','UKESM','EC'],
                     'imax':[151, 80,  80, 90, 80, 81,150, 80, 80],
                     'imaxctl': [90, 90, 90, 90, 90, 90,90,90, 90],
                     'nruns':[7,3, 3, 2, 1, 1, 1, 1, 1],
                     'runs': [['r1i1p1f1','r2i1p1f1','r3i1p1f1','r4i1p1f1',\
                               'r5i1p1f1','r6i1p1f1','r7i1p1f1'],\
                              ['r1i1p1f1','r2i1p1f1','r3i1p1f1'],\
                              ['r1i1p1f1','r2i1p1f1','r3i1p1f1'],\
                              ['r1i1p2f1'],\
                              ['r1i1p1f2'],\
                              ['r1i1p1f1'],\
                              ['r1i1p1f1'],\
                              ['r1i1p1f2'],\
                              ['r1i1p1f1']]\
                               }).set_index('model')
    
metaCTL = pd.DataFrame({'model':['MPI','IPSL','CESM2',\
                                 'Can','CNRM','BCC','MIROC','UKESM','EC'],
                     'imax':[151, 80,  80, 90, 80, 81, 90, 90, 80],
                     'imaxctl': [90, 90, 90, 90, 90, 90, 90, 90, 90],
                     'nruns':[ 7, 3, 1, 2, 1, 1, 1, 1, 1],
                     'runs': [['piControl'],
                              ['piControl'],
                              ['piControl'],\
                              ['piControl'],\
                              ['piControl'],
                              ['piControl'],
                              ['piControl'],
                              ['piControl'],
                              ['piControl']]}).set_index('model')

## index = rownames; select with meta.loc['IPSL']    
cols = pd.DataFrame({'IPSL': ['orchid','purple'],
                             'CESM': ['orchid','orchid'],
                              'MPI': ['indianred','indianred'],
                              'CESM2':['dodgerblue','dodgerblue'],
                              'Can': ['orange','orange'],
                              'BCC': ['mediumblue','mediumblue'],
                              'CNRM' : ['seagreen','seagreen'],
                              'MIROC' : ['gray','gray'],
                              'UKESM': ['pink','pink'],
                              'EC':['turquoise','turquoise']})      
cexx = 10
cexx2 = 12   


#%% DTas against final deffrac


biomes = ['global'] #,'tropics','temperate','boreal']
#   #   

rrix = [0] #[1,2,3]
latmin = -70
latmax = 70  
    
    
fig_scale = 2
fig = plt.figure(figsize=(3*fig_scale,4*fig_scale))
gs1 = plt.GridSpec(4,3, height_ratios=[1,1,1,0.05],width_ratios=[1,1,1])

gs1.update(wspace=0.1, hspace=0.3) 
i = 0; j = 0;  p = 0

zero_lat = pd.DataFrame({'model':models})
zero_GD_lat = pd.DataFrame({'model':models})

for model in models:
    print model
    print 'i = ' + str(i) + ' j = ' + str(j)

    if mistral == True:
        savepath = "/Users/lenaboysen/mistral/LUMIP/" 
        path="/Users/lenaboysen/mistral/LUMIP/"+model+"/"
    else:
        savepath = "/Users/lenaboysen/Documents/Crescendo_LUMIP/LUMIP/DATA/"
        path = "/Users/lenaboysen/Documents/Crescendo_LUMIP/LUMIP/DATA/"+model+"/"

#   #   #   #                
    biome_GD = None; def_GD = None
    rr == rrix
    print regions[rr]
    def_file = savepath + 'def_mask_regions.obj'
    if os.path.isfile(def_file):
        print 'loading def mask for ' + model
        def_mask_regions = pickle.load(open(def_file,'rb'))
        def_mask = def_mask_regions[model][model+'_defmask'+regions[rr]]
        deffrac = def_mask_regions[model][model+'_deffrac'+regions[rr]]/100.*(-1)
        deffrac = deffrac.where(deffrac > 0, 0)
        cellarea = def_mask_regions[model][model+'_cellarea']
        reg = def_mask_regions[model][model+regions[rr]]
        init = def_mask_regions[model][model+'_init_forest']
        
    lon = deffrac.lon
    lat = deffrac.lat   
    latdiff = np.append(np.diff(lat),np.diff(lat)[0])   
    
    deffrac = deffrac.where(deffrac > 0.1, 0)  
        
    ####  ------- 
    ## A: 1. get DEF and CTL; 2. build rel changes; mask 3. sign areas
    file_out = savepath + 'tas' + '_' + model +'_map.obj'
    ds = pickle.load(open(file_out,'rb'))
    tasrel = (ds[model + '_DEF_tas'] - ds[model + '_CTL_tas']) / ds[model + '_CTL_tas'] *100
    tasrelmask = tasrel.where(~np.isnan(ds['var_masked']))
    tasrelmask = tasrelmask.where(deffrac > 0)

    file_out = savepath + 'pr' + '_' + model +'_map.obj'
    ds = pickle.load(open(file_out,'rb'))    
    prrel = (ds[model + '_DEF_pr'] - ds[model + '_CTL_pr']) / ds[model + '_CTL_pr'] *100
    prrelmask = prrel.where(~np.isnan(ds['var_masked']))
    prrelmask = prrelmask.where(deffrac > 0)
    
#        plt.pcolormesh(prrelmask)
#        plt.colorbar()
#        plt.show() 
    
    ## B: identify significant pixels that apprear in both variables (dataframe?)
    merge = xr.merge([tasrelmask.to_dataset(name='tas'),
                      prrelmask.to_dataset(name='pr')])
    ## mask DF
    merge = merge.where(deffrac > 0.1, 0) 

    ## C: sort for corr
    
    ## -- global
    merge = merge.sel(lat=slice(-70,70))
    work1d = merge.stack(z=('lat', 'lon'))
    work_sort = work1d.sortby('pr') ## = deffrac
    df = work_sort.to_dataframe().dropna(axis=0).reset_index()
    
    ## -- tropical 
    mergetrop = merge.sel(lat=slice(-23,23))
    work1dtrop = mergetrop.stack(z=('lat', 'lon'))
    worktrop_sort = work1dtrop.sortby('pr') ## = deffrac
    dftrop = worktrop_sort.to_dataframe().dropna(axis=0).reset_index()        
    
    ## -- temperate
    mergetemp1 = merge.sel(lat=slice(23,50))
    mergetemp2 = merge.sel(lat=slice(-23,-50))
    mergetemp = xr.merge([mergetemp1,mergetemp2])
    work1dtemp = mergetemp.stack(z=('lat', 'lon'))
    worktemp_sort = work1dtemp.sortby('pr') ## = deffrac
    dftemp = worktemp_sort.to_dataframe().dropna(axis=0).reset_index()          
    
    ## -- boreal 
    mergebor = merge.sel(lat=slice(50,70))
    work1dbor = mergebor.stack(z=('lat', 'lon'))
    workbor_sort = work1dbor.sortby('pr') ## = deffrac
    dfbor = workbor_sort.to_dataframe().dropna(axis=0).reset_index()        
    

    ## D: do plotting -------------------------------------
    
    if model == 'MIROC':
        i = i+1
        p = p+1
        continue
    ax = plt.subplot(gs1[j,i])
    plt.axhline(y=0, color='k', linestyle='-',linewidth= 1)   
            
    sns.set_context("notebook", font_scale=1.2, rc={"lines.linewidth": 1.2})
    sns.set_style("whitegrid")
    custom_style = {
            'grid.color': '0.8',
            'grid.linestyle': '--',
            'grid.linewidth': 0.5,
            }
    sns.set_style(custom_style)  


## if dtas is not in percent
#    ax.set_ylim(-5, 5)
#    ax.set_xlim(-50, 50)         
#    yticks = np.arange(-5,5.5,1) 
#    xticks = np.arange(-50,60,10) 
    
    ax.set_ylim(-1.5, 2)
    ax.set_xlim(-50, 50)         
    yticks = np.arange(-1.5,2.5,0.5) 
    xticks = np.arange(-50,60,10)   
           
    plt.axvline(x=0, color='k', linestyle='-',linewidth= 1) 
    plt.axhline(y=0, color='k', linestyle='-',linewidth= 1) 
            
    vmax = latmax
    vmin = latmin
    cmap = 'terrain_r'#'gist_rainbow'
    
#    import matplotlib.colors as mcolors
    c1 = plt.cm.Greens_r(np.linspace(0., 1, 10))[[2,4,6,7,8],:]
    c2 = plt.cm.RdPu(np.linspace(0, 1, 10))[[6,9],:]
    c2b = plt.cm.autumn(np.linspace(0, 1, 10))[[3,5],:]
    c3 = plt.cm.summer_r(np.linspace(0., 1, 10))[[1,4,8],:] 
    c4 = plt.cm.GnBu(np.linspace(0., 1, 10))[[7,9],:]
    c4 = [[0./255., 191./255., 225./255.,1.],
          [0./255., 0./255., 11./255.,1.]] #deepskyblue1, blue	
    colors = np.vstack((c1, c2, c2b, c3, c4))
    cmap = mcolors.LinearSegmentedColormap.from_list('my_colormap', colors)

    
    
    sc = df.plot.scatter(
            x = 'pr', y = 'tas',
            alpha=0.2,c='lat',edgecolors='none',colormap=cmap,
            vmax=vmax,vmin=vmin,colorbar=False, ax=ax)

#    p1 = plt.plot([-10, 10],[-1, 1], color= 'grey', linestyle= '--',linewidth= 0.5)    
#    p2 = plt.plot([-10, 10],[1, -1], color= 'grey', linestyle= '--',linewidth= 0.5)       
    
    x = dftrop['pr']
    y = dftrop['tas']

#        popt, pcov = curve_fit(func_log, x, y)
#        ax.plot(x, func_log(x, *popt), 'pink', label = 'log fit')
    
    fit = np.poly1d(np.polyfit(x, y, 1))(x)
    plt.plot(x,fit,linestyle='-',color='red',label='tropics')  

    ### 2nd order fit
#    fit = np.poly1d(np.polyfit(x, y, 2))(x)
#    ax.plot(x,fit,linestyle='--',color='red',label='tropics')
        
        
    x = dftemp['pr']
    y = dftemp['tas']
   
    fit = np.poly1d(np.polyfit(x, y, 1))(x)
    plt.plot(x,fit,linestyle='-',color='greenyellow',label='temperate')         
   
### 2nd order fit
#    fit = np.poly1d(np.polyfit(x, y, 2))(x)
#    ax.plot(x,fit,linestyle='--',color='greenyellow',label='temperate')
        
    x = dfbor['pr']
    y = dfbor['tas']
  
    fit = np.poly1d(np.polyfit(x, y, 1))(x)
    plt.plot(x,fit,linestyle='-',color='blue',label='boreal') 
  
### 2nd order fit
#    fit = np.poly1d(np.polyfit(x, y, 2))(x)
#    ax.plot(x,fit,linestyle='--',color='blue',label='boreal')        
    
#            popt, pcov = curve_fit(func_exp, x, y)
#            ax.plot(x, func_exp(x, *popt), 'yellow', label = 'exp fit')
    
    handlesX, labelsX = ax.get_legend_handles_labels()

    ax.text(0.5,1.05,
            list(models_name[model])[0],
            color='grey',
            horizontalalignment='center',
            verticalalignment='center',
            transform=ax.transAxes,
            fontsize=cexx+2,
            bbox={'facecolor':'white', 'alpha':0.7, 'pad':5})
    
    ax.text(-0.05,1.05,['(a)','(b)','(c)','(d)','(e)','(f)','(g)','(h)','(i)'][p],
            color='k',
            horizontalalignment='left',
            verticalalignment='center',
            transform=ax.transAxes,
            fontsize=cexx,
            bbox={'facecolor':'white', 'alpha':0.7, 'pad':5})

    
    ax.set_yticks(yticks, minor=False)                
    plt.yticks(fontsize=cexx) 
    
#        ax.autoscale()

    ix = 2     
    for index, label in enumerate(ax.yaxis.get_ticklabels()):
        if index % ix != 0: # show only every 5th label
            label.set_visible(False)
            
    ax.set_xticks(xticks, minor=False) 
    plt.xticks(fontsize=cexx)
    ax.tick_params(axis='x', labelsize=cexx)
    
    for label in [ax.xaxis.get_ticklabels()[0]]:
            label.set_visible(False)    
    
    
    if p in [0,1,2,4,5]: #range(0,6):
        dummy = [item.get_text() for item in ax.get_xticklabels()]
        ax.set_xticklabels(['']*len(dummy))
        ax.set_xlabel('',fontsize=cexx)
    else:
        for index, label in enumerate(ax.xaxis.get_ticklabels()):
            if index % 2 != 0: # show only every nth label
                label.set_visible(False)
        
        ax.set_xlabel('$\Delta$Pr [%]',fontsize=cexx)


    if p in [0,3,7]:
#        ax.set_ylabel('$\Delta$Tas [$\circ$C]',fontsize=cexx2)
        ax.set_ylabel('$\Delta$Tas [%]',fontsize=cexx2)
    if p in [1,2,4,5,6,8]:
        dummy = [item.get_text() for item in ax.get_yticklabels()]
        ax.set_yticklabels(['']*len(dummy))
        ax.set_ylabel('',fontsize=cexx)
 

    
    # legend
    if p == 3:
        leg = ax.legend(handlesX, #np.take(handlesX,[0,2,4]),
                      labelsX, #np.take(labelsX,[0,2,4]),
                      ncol=1,
                      fontsize=cexx-2,
                      prop={'size':cexx-2},
                      title='',
                      loc='center left', 
                      bbox_to_anchor=(-0.1, -1.0), borderaxespad=0)
        for line in leg.get_lines():
            line.set_linewidth(2.0)    
   
    
    i = i + 1

    if p in [2, 5]:
        i = 0
        j = j + 1
    p = p + 1     
    
    
### --- Colorbar ----
def myround(x, base=5):
    return int(base * round(float(x)/base))


nt = 10
#tcks = np.arange(myround(vmin), myround(vmax), nt)    
tcks = [-70,-60,-50,-40,-30,-23,-10,0,10,23,30,40,50,60,70 ]

axes = plt.subplot(gs1[3, :])
axes.set_visible(False)
#    cbar_ax = fig.add_axes([0.12, 0.1, 0.8, 0.03]) ## x0, y0, width, height
cbar_ax = fig.add_axes([0.12, 0.05, 0.7, 0.02]) ## x0, y0, width, height 
cbar = mpl.colorbar.ColorbarBase(cbar_ax, cmap=cmap,
                                 ticks=tcks, boundaries=tcks,
                   orientation='horizontal',extend='both')

#        for label in bar.ax.xaxis.get_ticklabels()[::2]:
#            label.set_visible(False)
#        
cbar.ax.tick_params(labelsize=10,length=2, width=1)
cbar.outline.set_visible(False)   
cbar.set_label('Latitude [deg]',fontsize=10,labelpad=-40)  

fig.show()
fig.tight_layout()
pngfile='/Users/lenaboysen/Google Drive/LUMIP/nonlinear/'+'dPdT_nonlinear_global_ALL2_GD_percent.png' 
fig.savefig(pngfile,dpi=300, bbox_inches='tight')
plt.close()               
##  

