#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Sat Oct 12 15:36:05 2019

@author: lenaboysen


Plot zonal means of dTas and scaled with dF; plus dF per latitude

Zero latitude where the sign siwtches from minus to plus in temperate region

Data needed: LUMIP_MAP_read_save_data.py


"""


import os
import numpy as np
np.set_printoptions(threshold=200) 
import sys as sys
import pickle

import xarray as xr
import pandas as pd
import matplotlib.pyplot as plt
import seaborn as sns; sns.set(color_codes=True)


new_path = '/Users/lenaboysen/Google Drive/python/'
if new_path not in sys.path:
    sys.path.append(new_path)
    
 
## Constants
kg_C_per_kg_CO2 = 0.272918
kg_C_perMol = 0.012011
C_to_K = 273.15
MPa_to_mm = 1022.8

#%%
def average_da(self, dim=None, weights=None):
#    self = DEF_reg #_IPSL
#    weights = weights #_IPSL
#    dim=['lon','lat']
#    
    if weights is None:
        return self.mean(dim)
    else:
        if not isinstance(weights, xr.DataArray):
            raise ValueError("weights must be a DataArray")

        # if NaNs are present, we need individual weights
        if self.notnull().any():
            total_weights = weights.where(self.notnull()).sum(dim=dim)
        else:
            total_weights = weights.sum(dim)

        average_da = (self * weights).sum(dim) / total_weights
        return average_da

#%%
savepath = "/Users/lenaboysen/Google Drive/LUMIP/for_VB/"

rr = 0
timstat = 'year'
timstr = 'yr'
typ = '_'

varinfo = pd.DataFrame({'var':['tas',
                               'pr',
                               'gpp',
                               'cLand',
                               'cVeg','cSoil','cLitter',
                               'cSoilFast','cSoilSlow','cSoilMedium',
                               'evspsbl','npp','rh','fFire'],
                     'name':['$\Delta$Near Surface Temperature',
                             '$\Delta$Precipitation',
                             '$\Delta$GPP',
                             '$\Delta$Total Land Carbon',
                             '$\Delta$Vegetation Carbon',
                             '$\Delta$Soil Carbon',
                             '$\Delta$Litter Carbon',
                             '$\Delta$Soil Carbon (fast)',
                             '$\Delta$Soil Carbon (slow)',
                             '$\Delta$Soil Carbon (medium)',
                             '$\Delta$Evapotranspiration',
                             '$\Delta$NPP',
                             '$\Delta$Het. respiration',
                             '$\Delta$Fire Emissions'] ,
                     'name_short': ['$\Delta$Tas',
                             '$\Delta$Pr',
                             '$\Delta$GPP',
                             '$\Delta$cLand',
                             '$\Delta$cVeg',
                             '$\Delta$cSoil',
                             '$\Delta$cLitter',
                             '$\Delta$cSoil (fast)',
                             '$\Delta$cSoil (slow)',
                             '$\Delta$cSoil (medium)',
                             '$\Delta$ET',
                             '$\Delta$NPP',
                             '$\Delta$rh',
                             '$\Delta$fFire'] ,
                     'unit': ['$^\circ$C',
                              'mm '+timstr+'$^{-1}$',
                              'GtC '+timstr+'$^{-1}$',
                              'GtC','GtC','GtC','GtC','GtC','GtC','GtC',
                              'mm '+timstr+'$^{-1}$',
                              'GtC '+timstr+'$^{-1}$','GtC '+timstr+'$^{-1}$','GtC '+timstr+'$^{-1}$'],
                     'rm':[30,30,10,10,10,10,10,10,10,10,
                           30,10,10,10],
                    'minmax':[2.5,500,1.0,20.0,
                              15,5,2,0.6,0.6,0.6,
                              400,0.5,0.5,0.25],
                    'minmax_scaled':[5.0,500,1.0,40.0,
                                     20,10,5,0.6,0.6,0.6,
                                     400,0.5,0.5,0.25]\
                               }).set_index('var')    


mistral = False          
regions = ['_glob','_soam','_africa','_sea_aust','_noam_temp','_noam_bor','_eura_bor','_eura_temp']
regions_names = ['Global','South America','Africa','SE Asia & Aust.','North Am. temp.','North Am. bor.','Eurasia bor.','Eurasia temp.']

models = ['MPI','IPSL','CESM2','Can','CNRM','BCC','MIROC','UKESM','EC']
models_name = pd.DataFrame({'IPSL': ['IPSL-CM6A-LR'],
                            'MPI': ['MPI-ESM-1.2.0'],
                             'CESM2': ['CESM2'],
                             'Can': ['CanESM5'],
                             'CNRM': ['CNRM-ESM2-1'],
                             'BCC': ['BCC-CSM2-MR'],
                             'MIROC': ['MIROC-ES2L'],
                             'UKESM':['UKESM1-0-LL'],
                             'EC':['EC-Earth3-Veg']})


timstat = 'year'

meta = pd.DataFrame({'model':['MPI','IPSL','CESM2',\
                              'Can','CNRM','BCC','MIROC','UKESM','EC'],
                     'imax':[151, 80,  80, 90, 80, 81,150, 80, 80],
                     'imaxctl': [90, 90, 90, 90, 90, 90,90,90, 90],
                     'nruns':[7,3, 3, 2, 1, 1, 1, 1, 1],
                     'runs': [['r1i1p1f1','r2i1p1f1','r3i1p1f1','r4i1p1f1',\
                               'r5i1p1f1','r6i1p1f1','r7i1p1f1'],\
                              ['r1i1p1f1','r2i1p1f1','r3i1p1f1'],\
                              ['r1i1p1f1','r2i1p1f1','r3i1p1f1'],\
                              ['r1i1p2f1'],\
                              ['r1i1p1f2'],\
                              ['r1i1p1f1'],\
                              ['r1i1p1f1'],\
                              ['r1i1p1f2'],\
                              ['r1i1p1f1']]\
                               }).set_index('model')
    
metaCTL = pd.DataFrame({'model':['MPI','IPSL','CESM2',\
                                 'Can','CNRM','BCC','MIROC','UKESM','EC'],
                     'imax':[151, 80,  80, 90, 80, 81, 90, 90, 80],
                     'imaxctl': [90, 90, 90, 90, 90, 90, 90, 90, 90],
                     'nruns':[ 7, 3, 1, 2, 1, 1, 1, 1, 1],
                     'runs': [['piControl'],
                              ['piControl'],
                              ['piControl'],\
                              ['piControl'],\
                              ['piControl'],
                              ['piControl'],
                              ['piControl'],
                              ['piControl'],
                              ['piControl']]}).set_index('model')

## index = rownames; select with meta.loc['IPSL']    
cols = pd.DataFrame({'IPSL': ['orchid','purple'],
                             'CESM': ['orchid','orchid'],
                              'MPI': ['indianred','indianred'],
                              'CESM2':['dodgerblue','dodgerblue'],
                              'Can': ['orange','orange'],
                              'BCC': ['mediumblue','mediumblue'],
                              'CNRM' : ['seagreen','seagreen'],
                              'MIROC' : ['gray','gray'],
                              'UKESM': ['pink','pink'],
                              'EC':['turquoise','turquoise']})      
cexx = 10
cexx2 = 12   


#%% without cLand 

## use all data or only stat sign (=maksed)
masked = True

## input format
nc = False

## running window
rw_base = 10.



fig_scale = 2
fig = plt.figure(figsize=(4*fig_scale,3*fig_scale))
gs1 = plt.GridSpec(2,4, height_ratios=[1,1],width_ratios=[1,1,1,0.75])

gs1.update(wspace=0.1, hspace=0.4) 
i = 0; j = 0; p = 0    

zero_lat = pd.DataFrame({'model':models})
zero_GD_lat = pd.DataFrame({'model':models})


for var in ['tas']:
    for tt in ['global','GD','scaled']:
            
        ax = plt.subplot(gs1[j,i])
        plt.axhline(y=0, color='k', linestyle='-',linewidth= 1)   
        
        sns.set_context("notebook", font_scale=1.2, rc={"lines.linewidth": 1.2})
        sns.set_style("whitegrid")
        custom_style = {
                'grid.color': '0.8',
                'grid.linestyle': '--',
                'grid.linewidth': 0.5,
                }
        sns.set_style(custom_style)  
        ax.set_ylim(-90, 90)
        if tt in ['GD','global'] and var == 'tas':
            xticks = np.arange(-3,2,1)
        elif tt == 'scaled' and var == 'tas':
            xticks = np.arange(-20,20,5)
          
        elif tt in ['GD','global'] and var == 'cLand':
            xticks = np.arange(-600,20,200)
        elif tt == 'scaled' and var == 'cLand':
            xticks = np.arange(-1000,20,200)
            
        ax.set_xlim(xticks[0], 
                    xticks[len(xticks)-1]+(xticks[len(xticks)-1]-xticks[len(xticks)-2])/2.)                 
        plt.axvline(x=0, color='k', linestyle='-',linewidth= 1) 
    
    
        for model in models:
            
            ### === READ in data (either nc or obj) =======
            if nc == False:
                savepath = "/Users/lenaboysen/Documents/Crescendo_LUMIP/LUMIP/DATA/"
                path = "/Users/lenaboysen/Documents/Crescendo_LUMIP/LUMIP/DATA/"+model+"/"
                file_out = savepath + var + '_' + model +'_map.obj'
            
                if os.path.isfile(file_out):
                    ds = pickle.load(open(file_out,'rb'))
                    
                def_file = savepath + 'def_mask_regions.obj'
                if os.path.isfile(def_file):
                    print 'loading def mask for ' + model
                    def_mask_regions = pickle.load(open(def_file,'rb'))
                    def_mask = def_mask_regions[model][model+'_defmask_glob']
                    deffrac = def_mask_regions[model][model+'_deffrac_glob']/100.*(-1)
                    deffrac = deffrac.where(deffrac > 0, 0)
                    cellarea = def_mask_regions[model][model+'_cellarea']
                    weights = cellarea.copy(deep=True)/np.sum(cellarea)
                    
            else:
                file_out = savepath + var + '_' + model +'_map.nc'
                if os.path.isfile(file_out):
                    ds = xr.open_dataset(file_out, cache=False, decode_times=False)
                    ds.close()  
                
                def_file = savepath + model + '_def_mask_regions.nc'
                if os.path.isfile(def_file):
                    deffrac = xr.open_dataset(def_file,
                                         cache=False, 
                                         decode_times=False)/100.*(-1)
                    deffrac.close()
                    deffrac = deffrac.where(deffrac > 0, 0)
                    deffrac = deffrac[model +'_deffrac_glob'] #/100.
                
        
                
                ca_file = savepath + model + '_cellarea.nc'
                if os.path.isfile(ca_file):                
                    cellarea = xr.open_dataset(ca_file,
                                         cache=False, 
                                         decode_times=False)
                    cellarea.close()
                    cellarea = cellarea[model+'_cellarea']
                    
                    weights = cellarea.copy(deep=True)/np.sum(cellarea)
#            plt.pcolormesh(deffrac) #ds_GD['var_masked'])
#            plt.colorbar()
#            plt.show()  
                
            ### =====================================================
            

            lon = deffrac.lon
            lat = deffrac.lat
            latdiff = np.append(np.diff(lat),np.diff(lat)[0])
            
            deffrac = deffrac.where(deffrac > 0.1, 0)
            defarea = deffrac * cellarea / 10**12  ##Mkm2
            
            if tt == 'global':
                print model + ' dF = '+str(round(defarea.sum(),2)) + ' Mkm2'

            ### chose whether to take all cells or only stat sign. ones
            if masked == True:
                signal = ds['var_masked']
            else:
                signal = ds[model+'_DEF_'+var] - ds[model+'_CTL_'+var]


            ds_GD = np.ma.masked_array(signal.copy(),xr.where(deffrac > 0, 0, 1)) 
            ds_GD = xr.Dataset({'signal':(('lon','lat'),ds_GD)},
                                        {'lon': lon, 'lat': lat}) 
            ds_glob = xr.Dataset({'signal':(('lon','lat'),signal)},
                                        {'lon': lon, 'lat': lat})

            ### zonal analysis
            if var == 'tas':          
                ds_GD_mean = average_da(ds_GD,dim=(['lon']),
                                        weights=weights)
                
                ds_mean = average_da(ds_glob,dim=(['lon']),
                                        weights=weights)
            elif var == 'cLand':
                ds_GD_mean = ds_GD.sum(['lon'])
                ds_mean = ds_glob.copy().sum(['lon']) 
    
### CHECKED WITH: 
#cdo output -zonmean -sub -timmean -selyear,1900/1929 -yearmean r1i1p2f1/tas_Amon_CanESM5_deforest-globe_r1i1p2f1_gn_185001-194012.nc -timmean -yearmean -selyear,6001/6090 piControl/tas_Amon_CanESM5_piControl_r1i1p1f1_gn_600101-620012.nc
            
              
            if model == 'Can':
                globmean = average_da(ds_glob['signal'], 
                                      dim=['lon','lat'], 
                                      weights=weights)
#                print 'Global tas mean ='
#                print globmean
#                
#                print 'Global tas ='
#                print ds_mean['signal'].values
#                print 'dTas over dF = '
#                print ds_GD_mean['signal'].values
#            
#            
            
            
            ### get value for every latitude
            latdiffx = round(latdiff.mean(),1)            
            if (latdiffx > rw_base ):
                rw =  1
            else:
                rw = int(rw_base/latdiffx)
            if tt == 'global':
                print 'rw = ' + str(rw) 
                print 'rw*latdiff = ' + str(rw*latdiffx)                
                
                
            ds_GD_mean = (ds_GD_mean).rolling(lat = int(rw), center=True).mean()
            ds_mean = (ds_mean).rolling(lat = int(rw), center=True).mean()
#            print model + ' latdiffx = '+str(latdiffx) + ' rw = ' +str(int(rw/latdiffx))
            lat_rw = ds_mean.lat
            
            if tt == 'GD':
                for ll in range(0,len(lat_rw)): 
                    if ds_mean['signal'][::-1][ll] > 0:
                        zero_lat[model] = lat_rw[::-1][ll].values
                        break
                for ll in range(0,len(lat_rw)):  
                    if not ds_GD_mean['signal'][::-1][ll].isnull() and \
                                        ds_GD_mean['signal'][::-1][ll] > 0:
                        zero_GD_lat[model] = lat_rw[::-1][ll].values
                        print model +' reached  at lat = ' + str(round(lat_rw[::-1][ll].values,2)) 
                        break
                    else:
                        zero_GD_lat[model] = np.nan

                        
            if tt == 'scaled':
                ds_plot = ds_mean['signal'] / defarea.sum(['lon']) / latdiff
                ds_plot = ds_plot.rolling(lat = int(rw), center=True).mean() 
                
#               ds_GD_plot = ds_GD_mean['signal'] / defarea.sum(['lon'])/ latdiff
                ds_GD_plot = ds_GD_mean['signal'] / (defarea.sum(['lon']) / latdiff * -1.)
                ds_GD_plot = ds_GD_plot.rolling(lat = int(rw), center=True).mean() 
                
            else:
                ds_plot = ds_mean['signal']
                ds_GD_plot = ds_GD_mean['signal']



    
            if tt == 'global' and var == 'tas':
                plt.plot(ds_plot, lat,color=cols[model][0],\
                       linestyle='-', linewidth=1,alpha=1,
                       label=list(models_name[model])[0])
                
                ax.text(0.4,1,
                    'global',
                    color='grey',
                    horizontalalignment='left',
                    verticalalignment='center',
                    transform=ax.transAxes,
                    fontsize=cexx+2,
                    bbox={'facecolor':'white', 'alpha':0.7, 'pad':5})
                
            elif tt in ['GD','scaled']:
                plt.plot(ds_GD_plot, lat,color=cols[model][0],\
                       linestyle='-', linewidth=1,alpha=1)
                ax.text(0.4,1,
                    'over $\Delta$F',
                    color='grey',
                    horizontalalignment='left',
                    verticalalignment='center',
                    transform=ax.transAxes,
                    fontsize=cexx+2,
                    bbox={'facecolor':'white', 'alpha':0.7, 'pad':5})
                if tt == 'GD' and var == 'tas':
                    plt.plot(xticks[len(xticks)-1], zero_GD_lat[model][0],
                             color=cols[model][0],\
                             marker="+",linestyle='-', 
                             fillstyle='none',
                             linewidth=1,alpha=1 )

#            ax.autoscale()
        yticks = np.arange(-90,90,20) 
        ax.set_yticks(yticks, minor=False)                
        plt.yticks(fontsize=cexx) 
        if tt in ['GD','scaled']:
            ax.set_yticklabels(['']*len(yticks))

        ax.set_xticks(xticks, minor=False) 
        plt.xticks(fontsize=cexx)
        ax.tick_params(axis='x', labelsize=cexx) 
        if tt == 'scaled':
            for label in ax.get_xticklabels()[::2]:
                label.set_visible(False)
            
        if tt == 'scaled':
            ax.set_xlabel('mean '+varinfo.loc[var]['name_short']+'/$\Delta$F'\
                          +'\n[' +varinfo.loc[var]['unit'] \
                          +' Mkm$^{-2}$ lat$^{-1}$]',fontsize=cexx)
        else:    
            ax.set_xlabel('mean '+varinfo.loc[var]['name_short']+\
                             ' ['+varinfo.loc[var]['unit']+']',fontsize=cexx)
        if i in [0]:
            ax.set_ylabel('Latitude',fontsize=cexx2)
            
            
        if tt == 'global' and var == 'tas':
            handlesX, labelsX = ax.get_legend_handles_labels()
            ax = plt.subplot(gs1[j,i])
            
            leg = ax.legend(handlesX,
                          labelsX,
                          ncol=3,
                          fontsize=cexx,
                          prop={'size':cexx},
                          title='',
                          loc='center left', 
                          bbox_to_anchor=(0.3, 1.3), borderaxespad=0)
            
            for line in leg.get_lines():
                line.set_linewidth(2.0) 
            
            
        ax.text(0,1,['(a)','(b)','(c)','(d)'][i],
                    color='k',
                    horizontalalignment='left',
                    verticalalignment='center',
                    transform=ax.transAxes,
                    fontsize=cexx,
                    bbox={'facecolor':'white', 'alpha':0.7, 'pad':5})   
        i = i+1
            
        if i == 3 and var == 'tas':
            i = 0
            j = j+1
            
            
             
## ===============  plot zonal deffrac    =======================
for j in [0]:
    ax = plt.subplot(gs1[j,3])
    plt.axhline(y=0, color='k', linestyle='-',linewidth= 1)   
    
    sns.set_context("notebook", font_scale=1.2, rc={"lines.linewidth": 1.2})
    sns.set_style("whitegrid")
    custom_style = {
            'grid.color': '0.8',
            'grid.linestyle': '--',
            'grid.linewidth': 0.5,
            }
    sns.set_style(custom_style)  
    ax.set_ylim(-90, 90)
    xticks = np.arange(-0.6,0,0.2)
    ax.set_xlim(xticks[0]-0.05, 0.02)    
    
    for model in models:
        def_file = savepath + model + '_def_mask_regions.nc'
        if os.path.isfile(def_file):
            deffrac = xr.open_dataset(def_file,
                                 cache=False, 
                                 decode_times=False)
            deffrac.close()
            deffrac = deffrac.where(deffrac < 0, 0)
            deffrac = deffrac[model +'_deffrac_glob']/100.*(-1)
            
        ca_file = savepath + model + '_cellarea.nc'
        if os.path.isfile(ca_file):                
            cellarea = xr.open_dataset(ca_file,
                                 cache=False, 
                                 decode_times=False)
            cellarea.close()
            cellarea = cellarea[model + '_cellarea']

#        plt.pcolormesh(cellarea) #ds_GD['var_masked'])
#        plt.colorbar()
#        plt.show() 

        
        lat = deffrac.lat
        latdiff = np.append(np.diff(lat),np.diff(lat)[0])
        
        
        ### get value for every latitude
        latdiffx = round(latdiff.mean(),1)            
        if (latdiffx > rw_base ):
            rw =  1
        else:
            rw = int(rw_base/latdiffx)
#        print 'rw = ' + str(rw) 
#        print 'rw*latdiff = ' + str(rw*latdiffx)                
                
        defarea = deffrac.where(deffrac > 0.1, 0) * cellarea / 10**12
  
        weights = cellarea.copy(deep=True)/np.sum(cellarea)
        weights = cellarea.sum('lon') / cellarea.sum(['lon','lat'])

        defarea_sum = defarea.sum(['lon'])/latdiff*(-1)
        print model + ' dF = '+str(round(defarea.sum(),2)) + ' Mkm2'
        
        plt.plot(defarea_sum.rolling(lat = rw, center=True).mean(), 
                 lat,color=cols[model][0],\
                 linestyle='-', linewidth=1,alpha=1 )        
        
    yticks = np.arange(-90,90,20) 
    ax.set_yticks(yticks, minor=False)                
    plt.yticks(fontsize=cexx) 
    ax.set_yticklabels(['']*len(yticks))
    
    ax.set_xticks(xticks, minor=False) 
    plt.xticks(fontsize=cexx)
    ax.tick_params(axis='x', labelsize=cexx)
    ax.set_xlabel('$\Delta$F [Mkm$^2$ lat$^{-1}$]',fontsize=cexx)
#    for label in ax.get_xticklabels()[::2]:
    label = ax.get_xticklabels()[1]
    label.set_visible(False)    
    ax.text(0,1,'d)',
            color='k',
            horizontalalignment='left',
            verticalalignment='center',
            transform=ax.transAxes,
            fontsize=cexx,
            bbox={'facecolor':'white', 'alpha':0.7, 'pad':5}) 
        
        
        

fig.show()
fig.tight_layout()
if nc ==  True:
    if masked == True:
        pngfile=savepath+'Zonal_clim_'+str(rw)+'yr_ALL_VB_masked.png' 
    else:
        pngfile=savepath+'Zonal_clim_'+str(rw)+'yr_ALL_VB_allvalues.png'
    
else:
    if masked == True:
        pngfile='/Users/lenaboysen/Google Drive/LUMIP/Time_series/Zonal_clim_'+str(rw_base)+'yr_ALL_VB_masked.png' 
    else:
        pngfile='/Users/lenaboysen/Google Drive/LUMIP/Time_series/Zonal_clim_'+str(rw_base)+'yr_ALL_VB_allvalues.png'
        
fig.savefig(pngfile,dpi=300, bbox_inches='tight')
plt.close()               
               
