#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Fri Mar 16 15:50:15 2018

@author: lenaboysen


CALC and PLOT Surface Energy Balance (SEB) Decomposition

Data needed: LUMIP_MAP_read_save_data.py



model-wise:
based on Rnet
based on albedo and incoming SW

component-wise

Fluxes alone

Cloud cover and rlds vs rldscs

Bowen and QA/QT partitioning

"""


import os
import numpy as np
np.set_printoptions(threshold=100) # 'nan'
#import os.path
import sys as sys
import pickle

#import cdo
#cdo = cdo.Cdo()
#cdo.setCdo('/opt/local/bin/cdo')
import xarray as xr
#from matplotlib import gridspec
#from datetime import datetime, date, time, timedelta

#plot
import matplotlib.pyplot as plt
import seaborn as sns; sns.set(color_codes=True)

#import plotly.plotly as py
#import plotly.graph_objs as go
import pandas as pd



new_path = '/Users/lenaboysen/Google Drive/python/'
if new_path not in sys.path:
    sys.path.append(new_path)
 
## Constants
kg_C_per_kg_CO2 = 0.272918
kg_C_perMol = 0.012011
C_to_K = 273.15
MPa_to_mm = 1022.8


#%% Functions
def cutregions(data_var, var, model, def_mask_regions, nregs=range(0,8)):
#    data_var = DEF.copy()
#    var = ['tas']
#    nregs=range(0,7)
#    runs = meta.loc[model]['runs']
#    def_mask_regions = def_mask_regions
#    model = model
    lat = data_var.lat.values
    lon = data_var.lon.values
    
    data_out = None
    for rr in nregs:
        data = data_var.copy(deep=True)   

        def_regmask = def_mask_regions[model][
                    model+'_defmask'+regions[rr]].copy(deep=True)
        def_regmask2 = xr.where(def_regmask > 0, 0, 1)               
                             
        regions_def_t = def_regmask2 
        

        data_regdef = np.ma.masked_array(
                    data.copy(deep=True),regions_def_t)

        ds1 =  xr.Dataset({model+'_'+var+'_'+'GD':
                                    (('lon','lat'),data_regdef)},
                                {'lon': lon, 'lat': lat})                                
        if data_out is None:
            data_out = ds1
        else:
            old = data_out
            data_out = xr.merge([old, ds1])
    return(data_out)
    
def average_da(self, dim=None, weights=None):
#    self = DEF_reg #_IPSL
#    weights = weights #_IPSL
#    dim=['lon','lat']
#    
    if weights is None:
        return self.mean(dim)
    else:
        if not isinstance(weights, xr.DataArray):
            raise ValueError("weights must be a DataArray")

        # if NaNs are present, we need individual weights
        if self.notnull().any():
            total_weights = weights.where(self.notnull()).sum(dim=dim)
        else:
            total_weights = weights.sum(dim)

        average_da = (self * weights).sum(dim) / total_weights
        return average_da

def mmm(t, rw, latmax, lat_default= None):
    t = t.rolling(lat = rw, center=True).mean()
    latmax = latmax
    latnew = np.linspace(t.lat[0], t.lat[-1], len(t.lat) * latmax/len(t.lat))
    t_new = t.interp(lat=latnew).to_dataset(name=model)
    if not lat_default is None:
        t_new['lat'] = lat_default
    return (t_new)

    
#%%  DEFINE VARIABLES
mistral = False        

regions = ['_glob','_soam','_africa','_sea_aust','_noam_temp','_noam_bor','_eura_bor','_eura_temp']
regions_names = ['Global','South America','Africa','SE Asia & Aust.','North Am. temp.','North Am. bor.','Eurasia bor.','Eurasia temp.']

models = ['MPI','IPSL','CESM2','Can','CNRM','BCC','MIROC','UKESM','EC']
models_name = pd.DataFrame({'IPSL': ['IPSL-CM6A-LR'],
                            'MPI': ['MPI-ESM-1.2.0'],
                             'CESM2': ['CESM2'],
                             'Can': ['CanESM5'],
                             'CNRM': ['CNRM-ESM2-1'],
                             'BCC': ['BCC-CSM2-MR'],
                             'MIROC': ['MIROC-ES2L'],
                             'UKESM':['UKESM1-0-LL'],
                             'EC':['EC-Earth3-Veg']})


timstat = 'year'

meta = pd.DataFrame({'model':['MPI','IPSL','CESM2',\
                              'Can','CNRM','BCC','MIROC','UKESM','EC'],
                     'imax':[151, 80,  80, 90, 80, 81,150, 80, 80],
                     'imaxctl': [90, 90, 90, 90, 90, 90,90,90, 90],
                     'nruns':[7,3, 3, 2, 1, 1, 1, 1, 1],
                     'runs': [['r1i1p1f1','r2i1p1f1','r3i1p1f1','r4i1p1f1',\
                               'r5i1p1f1','r6i1p1f1','r7i1p1f1'],\
                              ['r1i1p1f1','r2i1p1f1','r3i1p1f1'],\
                              ['r1i1p1f1','r2i1p1f1','r3i1p1f1'],\
                              ['r1i1p2f1'],\
                              ['r1i1p1f2'],\
                              ['r1i1p1f1'],\
                              ['r1i1p1f1'],\
                              ['r1i1p1f2'],\
                              ['r1i1p1f1']]\
                               }).set_index('model')
    
metaCTL = pd.DataFrame({'model':['MPI','IPSL','CESM2',\
                                 'Can','CNRM','BCC','MIROC','UKESM','EC'],
                     'imax':[151, 80,  80, 90, 80, 81, 90, 90, 80],
                     'imaxctl': [90, 90, 90, 90, 90, 90, 90, 90, 90],
                     'nruns':[ 7, 3, 1, 2, 1, 1, 1, 1, 1],
                     'runs': [['piControl'],
                              ['piControl'],
                              ['piControl'],\
                              ['piControl'],\
                              ['piControl'],
                              ['piControl'],
                              ['piControl'],
                              ['piControl'],
                              ['piControl']]}).set_index('model')

## index = rownames; select with meta.loc['IPSL']    
cols = pd.DataFrame({'IPSL': ['orchid','purple'],
                             'CESM': ['orchid','orchid'],
                              'MPI': ['indianred','indianred'],
                              'CESM2':['dodgerblue','dodgerblue'],
                              'Can': ['orange','orange'],
                              'BCC': ['mediumblue','mediumblue'],
                              'CNRM' : ['seagreen','seagreen'],
                              'MIROC' : ['gray','gray'],
                              'UKESM': ['pink','pink'],
                              'EC':['turquoise','turquoise']})     
cexx = 10
cexx2 = 12    
    


#%%
mistral = False

eps = 0.97
CtoK = 273.15
SBK = 5.67*10**(-8)

variables = ['ts','hfls','hfss','rlds','rsds','rsus',
             'rldscs','clt','tas','evspsbl']  

rr = 0
timstat = 'year'
typ = '_'

if type(variables) == str:
    length = 1
    var = [variables]
elif type(variables) == list:
    length = np.shape(variables)[0]
    var = variables    
    
    
if mistral == True:
    savepath = "/Users/lenaboysen/mistral/LUMIP/"  
else:
    savepath = "/Users/lenaboysen/Documents/Crescendo_LUMIP/LUMIP/DATA/"
           
file_out = savepath + 'def_mask_regions.obj'
if os.path.isfile(file_out):
    def_mask_regions = pickle.load(open(file_out,'rb'))
    

SEB = {}
T_Rn = {} ; T_Ld = {}; T_QH = {}; T_QS = {}; T_Sd = {}; T_Su = {}; T_albedo= {}
dT = {}; T_T = {}; T_Tas = {}
CLT = {}; Ld = {}; Ldcs = {}; Sd = {}; Su = {}; albedo = {}; 
Hfls = {}; Hfss ={}; ET = {}
R = {}; weights = {}
latitude = {}

for model in models:#['MPI']:#
    seb = None
    for vv in range(0, len(variables)):
        
#        if var[vv] in ['clt','rldscs'] and model in ['MPI']:
#            continue
        if model == 'CESM2'and var[vv] == 'rldscs':
            continue
        
        
        #### created with time series scripts (or ToE_ts)
        if mistral == True:
            savepath = "/Users/lenaboysen/mistral/LUMIP/" 
            path="/Users/lenaboysen/mistral/LUMIP/"+model+"/"
        else:
            savepath = "/Users/lenaboysen/Documents/Crescendo_LUMIP/LUMIP/DATA/"
            path = "/Users/lenaboysen/Documents/Crescendo_LUMIP/LUMIP/DATA/"+model+"/"
        
        file_out = savepath + var[vv] + '_' + model +'_map.obj'
        if os.path.isfile(file_out):
            ds = pickle.load(open(file_out,'rb'))
            
        def_file = savepath + 'def_mask_regions.obj'
        if os.path.isfile(def_file):
            print 'loading def mask for ' + model
            def_mask_regions = pickle.load(open(def_file,'rb'))
            cellarea = def_mask_regions[model][model+'_cellarea']
            areaweights = cellarea.copy(deep=True)/np.sum(cellarea)
            
        weights[model] = areaweights
        
#        print var[vv]
#        plt.pcolormesh(ds[model+'_DEF_'+var[vv]])
#        plt.colorbar()
#        plt.show()
#        plt.pcolormesh(ds[model+'_CTL_'+var[vv]])
#        plt.colorbar()
#        plt.show()
        
        if seb is None:
            seb = ds
        else:
            seb = xr.merge([seb,ds[model+'_DEF_'+var[vv]],ds[model+'_CTL_'+var[vv]]])
            
        seb = xr.merge([seb,cutregions(ds[model+'_DEF_'+var[vv]].copy(), 'DEF_'+var[vv], 
                             model, def_mask_regions, nregs=[0])])
        seb = xr.merge([seb,cutregions(ds[model+'_CTL_'+var[vv]].copy(), 'CTL_'+var[vv], 
                             model, def_mask_regions, nregs=[0])])
            
        latitude = seb[model+'_DEF_'+var[vv]].lat
     
        del ds
        
    SEB[model] = seb
    del seb

    for tt in ['','_GD']:       
        factor = 1/(SBK*eps*4*SEB[model][model+'_CTL_ts'+tt]**3)
        
        T_Rn[model+'_Rn'+tt] = (factor * ((SEB[model][model+'_DEF_rsds'+tt]-\
                                    SEB[model][model+'_DEF_rsus'+tt])-\
                        (SEB[model][model+'_CTL_rsds'+tt]-\
                         SEB[model][model+'_CTL_rsus'+tt])))
        
        T_Sd[model + '_rsds' + tt] =  factor *(SEB[model][model+'_DEF_rsds'+tt] -  \
            SEB[model][model+'_CTL_rsds'+tt])
        T_Su[model + '_rsus' + tt] =  -factor *(SEB[model][model+'_DEF_rsus'+tt] -  \
            SEB[model][model+'_CTL_rsus'+tt])
        
        T_albedo[model + '_albedo' + tt] = (factor * \
                ((SEB[model][model+'_DEF_rsus'+tt]/SEB[model][model+'_DEF_rsds'+tt]) -\
                 (SEB[model][model+'_CTL_rsus'+tt]/SEB[model][model+'_CTL_rsds'+tt]))) * \
                SEB[model][model+'_DEF_rsds'+tt] *(-1)

        T_Ld[model+'_rlds'+tt] = factor * (SEB[model][model+'_DEF_rlds'+tt]-\
            SEB[model][model+'_CTL_rlds'+tt])
        
        T_QH[model+'_hfls'+tt] = -factor * (SEB[model][model+'_DEF_hfls'+tt]-\
            SEB[model][model+'_CTL_hfls'+tt])
        
        T_QS[model+'_hfss'+tt] = -factor * (SEB[model][model+'_DEF_hfss'+tt]-\
            SEB[model][model+'_CTL_hfss'+tt]) 
        
        dT[model+'_ts'+tt]    = T_Rn[model+'_Rn'+tt] + T_Ld[model+'_rlds'+tt] \
                                + T_QH[model+'_hfls'+tt] + T_QS[model+'_hfss'+tt]
        T_T[model+'_ts'+tt] = (SEB[model][model+'_DEF_ts'+tt]-\
           SEB[model][model+'_CTL_ts'+tt]) 
        T_Tas[model+'_tas'+tt] = (SEB[model][model+'_DEF_tas'+tt]-\
           SEB[model][model+'_CTL_tas'+tt])         
        
        
        R[model+'_R'+tt] = -(T_T[model+'_ts'+tt] - dT[model+'_ts'+tt])
        
     
        if not model in ['CESM2']: #'MPI',
            Ldcs[model+'_rldscs'+tt] =  SEB[model][model+'_DEF_rldscs'+tt]-SEB[model][model+'_CTL_rldscs'+tt]
            CLT[model+'_clt'+tt] = SEB[model][model+'_DEF_clt'+tt]-SEB[model][model+'_CTL_clt'+tt]
        elif model == 'CESM2':
            CLT[model+'_clt'+tt] = SEB[model][model+'_DEF_clt'+tt]-SEB[model][model+'_CTL_clt'+tt]
        Ld[model+'_rlds'+tt] = SEB[model][model+'_DEF_rlds'+tt]-SEB[model][model+'_CTL_rlds'+tt]
        Sd[model+'_rsds'+tt] = SEB[model][model+'_DEF_rsds'+tt]-SEB[model][model+'_CTL_rsds'+tt]
        Su[model+'_rsus'+tt] = SEB[model][model+'_DEF_rsus'+tt]-SEB[model][model+'_CTL_rsus'+tt]
        albedo[model+'_albedo'+tt] = (SEB[model][model+'_DEF_rsus'+tt]/SEB[model][model+'_DEF_rsds'+tt])\
            - (SEB[model][model+'_CTL_rsus'+tt] - SEB[model][model+'_CTL_rsds'+tt])
        ET[model+'_et'+tt] = SEB[model][model+'_DEF_evspsbl'+tt]-SEB[model][model+'_CTL_evspsbl'+tt]
        
        Hfss[model+'_hfss'+tt] = SEB[model][model+'_DEF_hfss'+tt]-SEB[model][model+'_CTL_hfss'+tt]
        Hfls[model+'_hfls'+tt] = SEB[model][model+'_DEF_hfls'+tt]-SEB[model][model+'_CTL_hfls'+tt]
        
        
        print model + tt 
        plt.pcolormesh(T_Rn[model+'_Rn'+tt])
        plt.colorbar()
        plt.show()    

    
    #%% net shortwave
    
fig_scale = 2
fig = plt.figure(figsize=(3*fig_scale,4*fig_scale))
gs1 = plt.GridSpec(3,3, height_ratios=[1,1,1],width_ratios=[1,1,1])

gs1.update(wspace=0.1, hspace=0.2) 
i = 0; j = 0;  p = 0
            

for model in models:
    print model
    print 'i = ' + str(i) + ' j = ' + str(j)
    
    ax = plt.subplot(gs1[j,i])
    
    plt.axhline(y=0, color='k', linestyle='-',linewidth= 1)   
            
    sns.set_context("notebook", font_scale=1.2, rc={"lines.linewidth": 1.2})
    sns.set_style("whitegrid")
    custom_style = {
            'grid.color': '0.8',
            'grid.linestyle': '--',
            'grid.linewidth': 0.5,
            }
    sns.set_style(custom_style)  
    ax.set_ylim(-90, 90)        
    xticks = np.arange(-2,3,1)   
    ax.set_xlim(xticks[0],xticks[len(xticks)-1])                                
    plt.axvline(x=0, color='k', linestyle='-',linewidth= 1) 
    
    rw = 3
    ll = 1         
    for tt in ['','_GD']:            
        T_Rn_mean = average_da(T_Rn[model+'_Rn'+tt],dim=(['lon']),weights=weights[model])
        plt.plot(T_Rn_mean.rolling(lat = rw, center=True).mean(), 
                 T_Rn_mean.lat.rolling(lat = rw, center=True).mean(),color='red',\
                   linestyle=['-',':'][ll], linewidth=1,alpha=1,
                   label='net shortwave')
        T_Ld_mean = average_da(T_Ld[model+'_rlds'+tt],dim=(['lon']),weights=weights[model])
        plt.plot(T_Ld_mean.rolling(lat = rw, center=True).mean(),
                 T_Ld_mean.lat.rolling(lat = rw, center=True).mean(),color='orange',\
                   linestyle=['-',':'][ll], linewidth=1,alpha=1,
                   label='incoming longwave')
        T_QS_mean = average_da(T_QS[model+'_hfss'+tt],dim=(['lon']),weights=weights[model])
        plt.plot(T_QS_mean.rolling(lat = rw, center=True).mean(),
                 T_QS_mean.lat.rolling(lat = rw, center=True).mean(),color='cyan',\
                   linestyle=['-',':'][ll], linewidth=1,alpha=1,
                   label='sensible')
        T_QH_mean = average_da(T_QH[model+'_hfls'+tt],dim=(['lon']),weights=weights[model])
        plt.plot(T_QH_mean.rolling(lat = rw, center=True).mean(),
                 T_QH_mean.lat.rolling(lat = rw, center=True).mean(),color='blue',\
                   linestyle=['-',':'][ll], linewidth=1,alpha=1,
                   label='latent')
        dT_mean = average_da(dT[model+'_ts'+tt],dim=(['lon']),weights=weights[model])
        plt.plot(dT_mean.rolling(lat = rw, center=True).mean(),
                 dT_mean.lat.rolling(lat = rw, center=True).mean(),color='grey',\
                   linestyle=['-',':'][ll], linewidth=1,alpha=1,
                   label='T$_{surf}$')
        T_T_mean = average_da(T_T[model+'_ts'+tt],dim=(['lon']),weights=weights[model]) 
        plt.plot(T_T_mean.rolling(lat = rw, center=True).mean(),
                 T_T_mean.lat.rolling(lat = rw, center=True).mean(),color='black',\
                   linestyle=['-',':'][ll], linewidth=1,alpha=1,
                   label='T$_{surf}$$_{\_model}$')
#        plt.plot(R[model+'_R'+tt].mean('lon'), T_T[model+'_ts'+tt].lat,color='pink',\
#                   linestyle=['-',':'][ll], linewidth=1,alpha=1,
#                   label='residual')        
        ll = ll - 1
    
        
    ax.text(0.5,1.05,
            list(models_name[model])[0],
            color='grey',
            horizontalalignment='center',
            verticalalignment='center',
            transform=ax.transAxes,
            fontsize=cexx+2,
            bbox={'facecolor':'white', 'alpha':0.7, 'pad':5})
    
    ax.text(-0.05,1.05,['(a)','(b)','(c)','(d)','(e)','(f)','(g)','(h)','(i)'][p],
            color='k',
            horizontalalignment='left',
            verticalalignment='center',
            transform=ax.transAxes,
            fontsize=cexx,
            bbox={'facecolor':'white', 'alpha':0.7, 'pad':5})
    
    
    
    yticks = np.arange(-70,90,20) 
    ax.set_yticks(yticks, minor=False)                
    plt.yticks(fontsize=cexx) 
    
    
    ax.set_xticks(xticks, minor=False) 
    plt.xticks(fontsize=cexx)
    ax.tick_params(axis='x', labelsize=cexx)
    
    
#    ax.autoscale()
    

    if p in [1,2,4,5,7,8]:
        dummy = [item.get_text() for item in ax.get_yticklabels()]
        ax.set_yticklabels(['']*len(dummy))
        ax.set_ylabel('',fontsize=cexx)
    
    
    if i in [0,3,6]:
        ax.set_ylabel('Latitude',fontsize=cexx2)
    
    
    if p in range(0,6):
        dummy = [item.get_text() for item in ax.get_xticklabels()]
        ax.set_xticklabels(['']*len(dummy))
        ax.set_xlabel('',fontsize=cexx)
    else:
        ax.set_xlabel('$\Delta$T$_{surf}$ [K]',fontsize=cexx)




    if p==0:
        import matplotlib.lines as mlines
        x, y = np.array([[2.3,2.3], [0.9,0.9]])
        line = mlines.Line2D(x , y , 
                             lw=2, linestyle=':', 
                             alpha=1, color='black',
                             label = 'global',
                             transform=ax.transAxes)
        
        x, y = np.array([[2.3,2.1], [0.9,0.9]])
        line2 = mlines.Line2D(x , y , 
                             lw=2, linestyle='-', 
                             alpha=1,color='black',
                             label = 'over $\Delta$F',
                             transform=ax.transAxes)
        
        plt.legend(handles=[line,line2], ncol=1, loc='center left',
                                 fontsize=cexx-2,
                   bbox_to_anchor=(-0.0, 1.3), borderaxespad=0.)
        
    
    
    
    handlesX, labelsX = ax.get_legend_handles_labels()
    ##-------------------------------------------
    ## legend in empty plot
    if p==1:
#        
#        ax = plt.subplot(gs1[j,i])
        if len(labelsX) == 12:
            indices = np.arange(6,12,1) 
        elif len(labelsX) == 6:
            indices = np.arange(0,6,1) 
            
        leg = ax.legend(np.take(handlesX,indices),
                      np.take(labelsX,indices),
                      ncol=3,
                      fontsize=cexx-2,
                      prop={'size':cexx-2},
                      title='',
                      loc='center left', 
#                          bbox_to_anchor=(0.8, 0.5), borderaxespad=0)
                      bbox_to_anchor=(-0.5, 1.3), borderaxespad=0)
        
        for line in leg.get_lines():
            line.set_linewidth(2.0)    


    i = i + 1
    
    if p in [2, 5]:
        i = 0
        j = j + 1
    p = p + 1

fig.show()
fig.tight_layout()
pngfile='/Users/lenaboysen/Google Drive/LUMIP/Time_series/Zonal_SEB_ALL_new.png' 
fig.savefig(pngfile,dpi=300, bbox_inches='tight')
plt.close()  
    #%%  split incoming / outgoing shortwave
    
fig_scale = 2
fig = plt.figure(figsize=(3*fig_scale,4*fig_scale))
gs1 = plt.GridSpec(3,3, height_ratios=[1,1,1],width_ratios=[1,1,1])

gs1.update(wspace=0.1, hspace=0.2) 
i = 0; j = 0;  p = 0
            

for model in models:
    print model
    print 'i = ' + str(i) + ' j = ' + str(j)
    
    ax = plt.subplot(gs1[j,i])
    
    plt.axhline(y=0, color='k', linestyle='-',linewidth= 1)   
            
    sns.set_context("notebook", font_scale=1.2, rc={"lines.linewidth": 1.2})
    sns.set_style("whitegrid")
    custom_style = {
            'grid.color': '0.8',
            'grid.linestyle': '--',
            'grid.linewidth': 0.5,
            }
    sns.set_style(custom_style)  
    ax.set_ylim(-90, 90)        
    xticks = np.arange(-2,3,1)   
#    ax.set_xlim(xticks[0],xticks[len(xticks)-1])   
    ax.set_xlim(-3,3)         
    plt.axvline(x=0, color='k', linestyle='-',linewidth= 1) 
    
    rw_base = 10
    ll = 1
    for tt in ['','_GD']:            
#        plt.plot(T_Rn[model+'_Rn'+tt].mean('lon').rolling(lat = rw, center=True).mean(), 
#                 T_Rn[model+'_Rn'+tt].lat.rolling(lat = rw, center=True).mean(),color='red',\
#                   linestyle=['-',':'][ll], linewidth=1,alpha=1,
#                   label='net shortwave')

## Check oky; gives same result as with rsus        
#        plt.plot(T_albedo[model+'_albedo'+tt].mean('lon').rolling(lat = rw, center=True).mean(), 
#                 T_albedo[model+'_albedo'+tt].lat.rolling(lat = rw, center=True).mean(),color='red',\
#                   linestyle=['-',':'][ll], linewidth=1,alpha=1,
#                   label='albedo')  
        
        lat = T_Su[model+'_rsus'+tt].lat
        latdiff = np.append(np.diff(lat),np.diff(lat)[0])
        latdiffx = round(latdiff.mean(),1)
        
        
        if ll == 1:
            print 'latdifx = ' + str(latdiffx)
        if (latdiffx > rw_base ):
            rw =  1
        else:
            rw = int(rw_base/latdiffx)
        if ll == 1:
            print 'rw = ' + str(rw) 
            print 'outcome = ' + str(rw*latdiffx)
        T_Su_mean = average_da(T_Su[model+'_rsus'+tt],dim=(['lon']),weights=weights[model])
        plt.plot(T_Su_mean.rolling(lat = rw, center=True).mean(), 
                 T_Su_mean.lat.rolling(lat = rw, center=True).mean(),color='red',\
                   linestyle=['-',':'][ll], linewidth=1,alpha=1,
                   label='albedo') 
        
        T_Sd_mean = average_da(T_Sd[model+'_rsds'+tt],dim=(['lon']),weights=weights[model])
        plt.plot(T_Sd_mean.rolling(lat = rw, center=True).mean(), 
                 T_Sd_mean.lat.rolling(lat = rw, center=True).mean(),color='darkred',\
                   linestyle=['-',':'][ll], linewidth=1,alpha=1,
                   label='incoming shortwave')        
        
#        T_Rn_mean = average_da(T_Rn[model+'_Rn'+tt],dim=(['lon']),weights=weights)
#        plt.plot(T_Rn_mean.rolling(lat = rw, center=True).mean(), 
#                 T_Rn_mean.lat.rolling(lat = rw, center=True).mean(),color='red',\
#                   linestyle=['-',':'][ll], linewidth=1,alpha=1,
#                   label='net shortwave')
        
        T_Ld_mean = average_da(T_Ld[model+'_rlds'+tt],dim=(['lon']),weights=weights[model])
        plt.plot(T_Ld_mean.rolling(lat = rw, center=True).mean(),
                 T_Ld_mean.lat.rolling(lat = rw, center=True).mean(),color='orange',\
                   linestyle=['-',':'][ll], linewidth=1,alpha=1,
                   label='incoming longwave')
        T_QS_mean = average_da(T_QS[model+'_hfss'+tt],dim=(['lon']),weights=weights[model])
        plt.plot(T_QS_mean.rolling(lat = rw, center=True).mean(),
                 T_QS_mean.lat.rolling(lat = rw, center=True).mean(),color='cyan',\
                   linestyle=['-',':'][ll], linewidth=1,alpha=1,
                   label='sensible')
        T_QH_mean = average_da(T_QH[model+'_hfls'+tt],dim=(['lon']),weights=weights[model])
        plt.plot(T_QH_mean.rolling(lat = rw, center=True).mean(),
                 T_QH_mean.lat.rolling(lat = rw, center=True).mean(),color='blue',\
                   linestyle=['-',':'][ll], linewidth=1,alpha=1,
                   label='latent')
        dT_mean = average_da(dT[model+'_ts'+tt],dim=(['lon']),weights=weights[model])
        plt.plot(dT_mean.rolling(lat = rw, center=True).mean(),
                 dT_mean.lat.rolling(lat = rw, center=True).mean(),color='grey',\
                   linestyle=['-',':'][ll], linewidth=1,alpha=1,
                   label='T$_{surf}$')
        T_T_mean = average_da(T_T[model+'_ts'+tt],dim=(['lon']),weights=weights[model]) 
        plt.plot(T_T_mean.rolling(lat = rw, center=True).mean(),
                 T_T_mean.lat.rolling(lat = rw, center=True).mean(),color='black',\
                   linestyle=['-',':'][ll], linewidth=1,alpha=1,
                   label='T$_{surf}$$_{\_model}$')
        
        T_Tas_mean = average_da(T_Tas[model+'_tas'+tt],dim=(['lon']),weights=weights[model]) 
        plt.plot(T_Tas_mean.rolling(lat = rw, center=True).mean(),
                 T_Tas_mean.lat.rolling(lat = rw, center=True).mean(),color='black',\
                   linestyle=['--',':'][ll], linewidth=1,alpha=1,
                   label='TAS$_{\_model}$')        
        
#        plt.plot(R[model+'_R'+tt].mean('lon'), T_T[model+'_ts'+tt].lat,color='pink',\
#                   linestyle=['-',':'][ll], linewidth=1,alpha=1,
#                   label='residual')        
        ll = ll - 1
    
        
    ax.text(0.5,1.05,
            list(models_name[model])[0],
            color='grey',
            horizontalalignment='center',
            verticalalignment='center',
            transform=ax.transAxes,
            fontsize=cexx+2,
            bbox={'facecolor':'white', 'alpha':0.7, 'pad':5})
    
    ax.text(-0.05,1.05,['(a)','(b)','(c)','(d)','(e)','(f)','(g)','(h)','(i)'][p],
            color='k',
            horizontalalignment='left',
            verticalalignment='center',
            transform=ax.transAxes,
            fontsize=cexx,
            bbox={'facecolor':'white', 'alpha':0.7, 'pad':5})
    
    
    
    yticks = np.arange(-70,90,20) 
    ax.set_yticks(yticks, minor=False)                
    plt.yticks(fontsize=cexx) 
    
    
    ax.set_xticks(xticks, minor=False) 
    plt.xticks(fontsize=cexx)
    ax.tick_params(axis='x', labelsize=cexx)
    
    
#    ax.autoscale()
    

    if p in [1,2,4,5,7,8]:
        dummy = [item.get_text() for item in ax.get_yticklabels()]
        ax.set_yticklabels(['']*len(dummy))
        ax.set_ylabel('',fontsize=cexx)
    
    
    if i in [0,3,6]:
        ax.set_ylabel('Latitude',fontsize=cexx2)
    
    
    if p in range(0,6):
        dummy = [item.get_text() for item in ax.get_xticklabels()]
        ax.set_xticklabels(['']*len(dummy))
        ax.set_xlabel('',fontsize=cexx)
    else:
        ax.set_xlabel('$\Delta$T$_{surf}$ [K]',fontsize=cexx)




    if p==0:
        import matplotlib.lines as mlines
        x, y = np.array([[2.3,2.3], [0.9,0.9]])
        line = mlines.Line2D(x , y , 
                             lw=2, linestyle=':', 
                             alpha=1, color='black',
                             label = 'global',
                             transform=ax.transAxes)
        
        x, y = np.array([[2.3,2.1], [0.9,0.9]])
        line2 = mlines.Line2D(x , y , 
                             lw=2, linestyle='-', 
                             alpha=1,color='black',
                             label = 'over $\Delta$F',
                             transform=ax.transAxes)
        
        plt.legend(handles=[line,line2], ncol=1, loc='center left',
                                 fontsize=cexx-2,
                   bbox_to_anchor=(-0.0, 1.3), borderaxespad=0.)
        
    
    
    
    handlesX, labelsX = ax.get_legend_handles_labels()
    ##-------------------------------------------
    ## legend in empty plot
    if p==1:
#        
#        ax = plt.subplot(gs1[j,i])
        if len(labelsX) == 16:
            indices = np.arange(8,16,1) 
        elif len(labelsX) == 8:
            indices = np.arange(0,8,1) 
            
        leg = ax.legend(np.take(handlesX,indices),
                      np.take(labelsX,indices),
                      ncol=3,
                      fontsize=cexx-2,
                      prop={'size':cexx-2},
                      title='',
                      loc='center left', 
#                          bbox_to_anchor=(0.8, 0.5), borderaxespad=0)
                      bbox_to_anchor=(-0.5, 1.3), borderaxespad=0)
        
        for line in leg.get_lines():
            line.set_linewidth(2.0)    


    i = i + 1
    
    if p in [2, 5]:
        i = 0
        j = j + 1
    p = p + 1

fig.show()
fig.tight_layout()
pngfile='/Users/lenaboysen/Google Drive/LUMIP/Time_series/Zonal_SEB_SW_ALL.png' 
fig.savefig(pngfile,dpi=300, bbox_inches='tight')
plt.close()  


#%% same but one plot for each flux with all models
### 7 subplots

fig_scale = 2
fig = plt.figure(figsize=(4*fig_scale,3*fig_scale))
gs1 = plt.GridSpec(2,4, height_ratios=[1,1],width_ratios=[1,1,1,1])

gs1.update(wspace=0.1, hspace=0.2) 
i = 0; j = 0;  p = 0
            

for comps in range(0,7):
    print model
    print 'i = ' + str(i) + ' j = ' + str(j)
    
    ax = plt.subplot(gs1[j,i])
    
    plt.axhline(y=0, color='k', linestyle='-',linewidth= 1)   
            
    sns.set_context("notebook", font_scale=1.2, rc={"lines.linewidth": 1.2})
    sns.set_style("whitegrid")
    custom_style = {
            'grid.color': '0.8',
            'grid.linestyle': '--',
            'grid.linewidth': 0.5,
            }
    sns.set_style(custom_style)  
#    ax.set_ylim(-90, 90)        
#    xticks = np.arange(-3,3,1)   
#    ax.set_xlim(xticks[0],xticks[len(xticks)-1])  

    ax.set_ylim(-90, 90)        
    xticks = np.arange(-3,3,1)   
    ax.set_xlim(-3.3,2.3)  
                              
    plt.axvline(x=0, color='k', linestyle='-',linewidth= 1) 
    
    rw_base = 10
    multi = None
    for model in models:
        ll = 0
        if model == 'MIROC':
            continue
        
        for tt in ['_GD']: #['','_GD']
            
            lat = T_Su[model+'_rsus'+tt].lat
            latdiff = np.append(np.diff(lat),np.diff(lat)[0])
            
            latdiffx = round(latdiff.mean(),1)

            if ll == 1:
                print 'latdifx = ' + str(latdiffx)
            if (latdiffx > rw_base ):
                rw =  1
            else:
                rw = int(rw_base/latdiffx)
            if ll == 1:
                print 'rw = ' + str(rw) 
                print 'outcome = ' + str(rw*latdiffx)
                
            latmax = 256 ## EC-Earth
            lwm = 1.5 ## line width multi model mean
            ltm = '-'; lcm = 'black'
            
            if comps == 0:
                T_Su_mean = average_da(T_Su[model+'_rsus'+tt],dim=(['lon']),weights=weights[model])
                plt.plot(T_Su_mean.rolling(lat = rw, center=True).mean(), 
                         T_Su_mean.lat.rolling(lat = rw, center=True).mean(),color=cols[model][1] ,\
                           linestyle=['-',':'][ll], linewidth=1,alpha=1,
                           label=model)     
                
                ### ---- multi model mean ---- 
                if multi is None:
                    multi = mmm(T_Su_mean,rw, latmax) 
                    lat_default = multi.lat  
                else:
                    multim =  mmm(T_Su_mean,rw, latmax, lat_default = lat_default)  
                    multi = xr.merge([multi, multim])
                    del(multim)
                    
                if model == 'EC': 
                    MM = multi.to_array().mean('variable', skipna=False)
                    plt.plot(MM,MM.lat,color=lcm,\
                           linestyle=ltm, linewidth=lwm,alpha=1,
                           label='multi-model mean')   
                ### -------------------------------  
                
            elif comps == 1:    
                T_Sd_mean = average_da(T_Sd[model+'_rsds'+tt],dim=(['lon']),weights=weights[model])
                plt.plot(T_Sd_mean.rolling(lat = rw, center=True).mean(), 
                         T_Sd_mean.lat.rolling(lat = rw, center=True).mean(),color=cols[model][1],\
                           linestyle=['-',':'][ll], linewidth=1,alpha=1,
                           label=model)  
                
                ### ---- multi model mean ---- 
                if multi is None:
                    multi = mmm(T_Sd_mean,rw, latmax) 
                    lat_default = multi.lat  
                else:
                    multim =  mmm(T_Sd_mean,rw, latmax, lat_default = lat_default)  
                    multi = xr.merge([multi, multim])
                    del(multim)
                    
                if model == 'EC': 
                    MM = multi.to_array().mean('variable', skipna=False)
                    plt.plot(MM, MM.lat,color=lcm,\
                           linestyle=ltm, linewidth=lwm,alpha=1,
                           label='multi-model mean')   
                ### -------------------------------  
                
            elif comps == 2: 
                T_Ld_mean = average_da(T_Ld[model+'_rlds'+tt],dim=(['lon']),weights=weights[model])
                plt.plot(T_Ld_mean.rolling(lat = rw, center=True).mean(),
                         T_Ld_mean.lat.rolling(lat = rw, center=True).mean(),color=cols[model][1],\
                           linestyle=['-',':'][ll], linewidth=1,alpha=1,
                           label=model)
                ### ---- multi model mean ---- 
                if multi is None:
                    multi = mmm(T_Ld_mean,rw, latmax) 
                    lat_default = multi.lat  
                else:
                    multim =  mmm(T_Ld_mean,rw, latmax, lat_default = lat_default)  
                    multi = xr.merge([multi, multim])
                    del(multim)
                    
                if model == 'EC': 
                    MM = multi.to_array().mean('variable', skipna=False)
                    plt.plot(MM,MM.lat,color=lcm,\
                           linestyle=ltm, linewidth=lwm,alpha=1,
                           label='multi-model mean')   
                ### -------------------------------  
            elif comps == 3:
                T_QS_mean = average_da(T_QS[model+'_hfss'+tt],dim=(['lon']),weights=weights[model])
                plt.plot(T_QS_mean.rolling(lat = rw, center=True).mean(),
                         T_QS_mean.lat.rolling(lat = rw, center=True).mean(),color=cols[model][1],\
                           linestyle=['-',':'][ll], linewidth=1,alpha=1,
                           label=model)
                ### ---- multi model mean ---- 
                if multi is None:
                    multi = mmm(T_QS_mean,rw, latmax) 
                    lat_default = multi.lat  
                else:
                    multim =  mmm(T_QS_mean,rw, latmax, lat_default = lat_default)  
                    multi = xr.merge([multi, multim])
                    del(multim)
                    
                if model == 'EC': 
                    MM = multi.to_array().mean('variable', skipna=False)
                    plt.plot(MM,MM.lat,color=lcm,\
                           linestyle=ltm, linewidth=lwm,alpha=1,
                           label='multi-model mean')   
                ### -------------------------------  
            elif comps == 4:
                T_QH_mean = average_da(T_QH[model+'_hfls'+tt],dim=(['lon']),weights=weights[model])
                plt.plot(T_QH_mean.rolling(lat = rw, center=True).mean(),
                         T_QH_mean.lat.rolling(lat = rw, center=True).mean(),color=cols[model][1],\
                           linestyle=['-',':'][ll], linewidth=1,alpha=1,
                           label=model)
                ### ---- multi model mean ---- 
                if multi is None:
                    multi = mmm(T_QH_mean,rw, latmax) 
                    lat_default = multi.lat  
                else:
                    multim =  mmm(T_QH_mean,rw, latmax, lat_default = lat_default)  
                    multi = xr.merge([multi, multim])
                    del(multim)
                    
                if model == 'EC': 
                    MM = multi.to_array().mean('variable', skipna=False)
                    plt.plot(MM,MM.lat,color=lcm,\
                           linestyle=ltm, linewidth=lwm,alpha=1,
                           label='multi-model mean')   
                ### -------------------------------                 
            elif comps == 5:
                dT_mean = average_da(dT[model+'_ts'+tt],dim=(['lon']),weights=weights[model])
                plt.plot(dT_mean.rolling(lat = rw, center=True).mean(),
                         dT_mean.lat.rolling(lat = rw, center=True).mean(),color=cols[model][1],\
                           linestyle=['-',':'][ll], linewidth=1,alpha=1,
                           label=model)
                ### ---- multi model mean ---- 
                if multi is None:
                    multi = mmm(dT_mean,rw, latmax) 
                    lat_default = multi.lat  
                else:
                    multim =  mmm(dT_mean,rw, latmax, lat_default = lat_default)  
                    multi = xr.merge([multi, multim])
                    del(multim)
                    
                if model == 'EC': 
                    MM = multi.to_array().mean('variable', skipna=False)
                    plt.plot(MM, MM.lat,color=lcm,\
                           linestyle=ltm, linewidth=lwm,alpha=1,
                           label='multi-model mean')   
                ### -------------------------------                   
            elif comps == 6:
                T_T_mean = average_da(T_T[model+'_ts'+tt],dim=(['lon']),weights=weights[model])
                plt.plot(T_T_mean.rolling(lat = rw, center=True).mean(),
                         T_T_mean.lat.rolling(lat = rw, center=True).mean(),color=cols[model][1],\
                           linestyle=['-',':'][ll], linewidth=1,alpha=1,
                           label=model)
                
                ### ---- multi model mean ---- 
                if multi is None:
                    multi = mmm(T_T_mean,rw, latmax) 
                    lat_default = multi.lat  
                else:
                    multim =  mmm(T_T_mean,rw, latmax, lat_default = lat_default)  
                    multi = xr.merge([multi, multim])
                    del(multim)
                    
                if model == 'EC': 
                    MM = multi.to_array().mean('variable', skipna=False)
                    plt.plot(MM, MM.lat, color=lcm,\
                           linestyle=ltm, linewidth=lwm,alpha=1,
                           label='multi-model mean')   
                ### -------------------------------                  
            elif comps == 7:
                plt.plot(dT_mean.rolling(lat = rw, center=True).mean()-\
                         dT_mean.rolling(lat = rw, center=True).mean(),
                         dT_mean.lat.rolling(lat = rw, center=True).mean(),color=cols[model][1],\
                           linestyle=['-',':'][ll], linewidth=1,alpha=1,
                           label=model) 
                
                ### ---- multi model mean ---- 
                if multi is None:
                    multi = mmm(dT_mean,rw, latmax) 
                    lat_default = multi.lat  
                else:
                    multim =  mmm(dT_mean,rw, latmax, lat_default = lat_default)  
                    multi = xr.merge([multi, multim])
                    del(multim)
                    
                if model == 'EC': 
                    MM = multi.to_array().mean('variable', skipna=False)
                    plt.plot(MM,MM.lat,color=lcm,\
                           linestyle=ltm, linewidth=lwm,alpha=1,
                           label='multi-model mean')   
                ### -------------------------------                   
            ll = ll - 1
        
        
        
        
        
        
    ax.text(0.5,1.05,
            ['albedo','incoming shortwave','incoming longwave',\
             'sensible','latent',\
             'T$_{surf}$','T$_{surf}$$_{\_model}$','T$_{surf}$ - T$_{surf}$$_{\_model}$'][comps],
            color='black', #['red','darkred','orange','cyan','blue','grey','black','grey'][comps],
            horizontalalignment='center',
            verticalalignment='center',
            transform=ax.transAxes,
            fontsize=cexx,
            bbox={'facecolor':'white', 'alpha':0.7, 'pad':5})
    
    ax.text(-0.1,1.05,['(a)','(b)','(c)','(d)','(e)','(f)','(g)','(h)','(i)'][p],
            color='k',
            horizontalalignment='left',
            verticalalignment='center',
            transform=ax.transAxes,
            fontsize=cexx,
            bbox={'facecolor':'white', 'alpha':0.7, 'pad':5})
    
    
    
    yticks = np.arange(-70,90,20) 
    ax.set_yticks(yticks, minor=False)                
    plt.yticks(fontsize=cexx) 
    
    
    ax.set_xticks(xticks, minor=False) 
    plt.xticks(fontsize=cexx)
    ax.tick_params(axis='x', labelsize=cexx)
    
    
#    ax.autoscale()
    

    if p in [1,2,4,5,6,7]:
        dummy = [item.get_text() for item in ax.get_yticklabels()]
        ax.set_yticklabels(['']*len(dummy))
        ax.set_ylabel('',fontsize=cexx)
    
    
    if p in [0,3]:
        ax.set_ylabel('Latitude',fontsize=cexx2)
    
    
    if p in range(0,3):
        dummy = [item.get_text() for item in ax.get_xticklabels()]
        ax.set_xticklabels(['']*len(dummy))
        ax.set_xlabel('',fontsize=cexx)
    else:
        ax.set_xlabel('$\Delta$T$_{surf}$ [K]',fontsize=cexx)





#    if p==0:
#        import matplotlib.lines as mlines
#        x, y = np.array([[2.3,2.3], [0.9,0.9]])
#        line = mlines.Line2D(x , y , 
#                             lw=2, linestyle=':', 
#                             alpha=1, color='black',
#                             label = 'global',
#                             transform=ax.transAxes)
#        
#        x, y = np.array([[2.3,2.1], [0.9,0.9]])
#        line2 = mlines.Line2D(x , y , 
#                             lw=2, linestyle='-', 
#                             alpha=1,color='black',
#                             label = 'over $\Delta$F',
#                             transform=ax.transAxes)
#        
#        plt.legend(handles=[line2], ncol=1, loc='center left',
#                                 fontsize=cexx-2,
#                   bbox_to_anchor=(-0.0, 1.3), borderaxespad=0.)

    
    handlesX, labelsX = ax.get_legend_handles_labels()
    ##-------------------------------------------
    ## legend in empty plot
    if p==2:
#        
#        ax = plt.subplot(gs1[j,i])
        if len(labelsX) == 18:
            indices = np.arange(1,19,2) 
        else:
            indices = np.arange(0,9,1) 
            
        leg = ax.legend(np.take(handlesX,indices),
                      np.take(labelsX,indices),
                      ncol=1,
                      fontsize=cexx-2,
                      prop={'size':cexx-2},
                      title='',
                      loc='center left', 
#                          bbox_to_anchor=(0.8, 0.5), borderaxespad=0)
#                      bbox_to_anchor=(-0.5, 1.3), borderaxespad=0)
                      bbox_to_anchor=(1.15, 0.5), borderaxespad=0)
        for line in leg.get_lines():
            line.set_linewidth(2.0)    


    i = i + 1
    
    if p in [2, 7]:
        i = 0
        j = j + 1
    p = p + 1

fig.show()
fig.tight_layout()
pngfile='/Users/lenaboysen/Google Drive/LUMIP/Time_series/Zonal_SEB_components_ALL_GD.png' 
fig.savefig(pngfile,dpi=300, bbox_inches='tight')
plt.close()  



#%% Cloud cover and rlds vs rldscs


for tt in ['','_GD']:
    fig_scale = 2
    fig = plt.figure(figsize=(4*fig_scale,5*fig_scale))
    gs1 = plt.GridSpec(3,3, height_ratios=[1,1,1],width_ratios=[1,1,1])
    
    gs1.update(wspace=0.2, hspace=0.5) 
    i = 0; j = 0;  p = 0
                
    
    for comps in ['turb','avail','rnet',\
                  'latent','sensible','clt',\
                  'sw down','lw down','lw-cs']: #range(0,10):
        
        print model
        print 'i = ' + str(i) + ' j = ' + str(j)
        
        ax = plt.subplot(gs1[j,i])
        
        plt.axhline(y=0, color='k', linestyle='-',linewidth= 1)   
                
        sns.set_context("notebook", font_scale=1.2, rc={"lines.linewidth": 1.2})
        sns.set_style("whitegrid")
        custom_style = {
                'grid.color': '0.8',
                'grid.linestyle': '--',
                'grid.linewidth': 0.5,
                }
        sns.set_style(custom_style)  
        ax.set_ylim(-90, 90)    
    
    
        if comps in ['lw down']:    
            xticks = np.arange(-9,6,3) 
        elif comps in ['latent','sensible','rnet']:
            xticks = np.arange(-9,9,3)              
        elif comps in ['lw-cs']:
            xticks = np.arange(-2,3,1)        
        elif comps in ['sw down']:
            xticks = np.arange(-2,8,2) 
        elif comps in ['albedo']:
            xticks = np.arange(125,250,25)
        elif comps in ['clt']:
            xticks = np.arange(-3,3,1)   
        elif comps in ['turb','avail']:
            xticks = np.arange(-20,10,5) 
        elif comps in ['Ts']:
            xticks = np.arange(-3,3,1) 
            
        ax.set_xlim(xticks[0],xticks[len(xticks)-1])  
    
                                  
        plt.axvline(x=0, color='k', linestyle='-',linewidth= 1) 
        
        rw_base = 10
    
        for model in models:
            ll = 0
            
            lat = T_Su[model+'_rsus'+tt].lat
            latdiff = np.append(np.diff(lat),np.diff(lat)[0])
            
            latdiffx = round(latdiff.mean(),1)

            if ll == 1:
                print 'latdifx = ' + str(latdiffx)
            if (latdiffx > rw_base ):
                rw =  1
            else:
                rw = int(rw_base/latdiffx)
            if ll == 1:
                print 'rw = ' + str(rw) 
                print 'outcome = ' + str(rw*latdiffx)
                
                
            if comps == 'lw down':
                Ld_mean = average_da(Ld[model+'_rlds'+tt],dim=(['lon']),weights=weights[model])
                plt.plot(Ld_mean.rolling(lat = rw, center=True).mean(), 
                         Ld_mean.lat.rolling(lat = rw, center=True).mean(),color=cols[model][1] ,\
                           linestyle=['-',':'][ll], linewidth=1,alpha=1,
                           label=model)     
                    
                    
            elif comps == 'lw-cs':
                if model in ['CESM2']: #'MPI',
                   continue
                else:
                    Ldcs_mean = average_da(Ldcs[model+'_rldscs'+tt],dim=(['lon']),weights=weights[model])
                    Ld_mean = average_da(Ld[model+'_rlds'+tt],dim=(['lon']),weights=weights[model])
                    
                    plt.plot((Ld_mean-Ldcs_mean).rolling(lat = rw, center=True).mean(), 
                         Ldcs_mean.lat.rolling(lat = rw, center=True).mean(),color=cols[model][1],\
                           linestyle=['-',':'][ll], linewidth=1,alpha=1,
                           label=model)
                    
                    
            elif comps == 'clt':
#                if model in ['MPI']:
#                    continue
#                else:
                CLT_mean = average_da(CLT[model+'_clt'+tt],dim=(['lon']),weights=weights[model])
                plt.plot(CLT_mean.rolling(lat = rw, center=True).mean(),
                         CLT_mean.lat.rolling(lat = rw, center=True).mean(),color=cols[model][1],\
                           linestyle=['-',':'][ll], linewidth=1,alpha=1,
                               label=model)                     
                    
            elif comps == 'sw down':
                Sd_mean = average_da(Sd[model+'_rsds'+tt],dim=(['lon']),weights=weights[model])
                plt.plot(Sd_mean.rolling(lat = rw, center=True).mean(),
                         Sd_mean.lat.rolling(lat = rw, center=True).mean(),color=cols[model][1],\
                           linestyle=['-',':'][ll], linewidth=1,alpha=1,
                           label=model)
                
            elif comps == 'rnet':
                Sd_mean = average_da(Sd[model+'_rsds'+tt],dim=(['lon']),weights=weights[model])
                Su_mean = average_da(Su[model+'_rsus'+tt],dim=(['lon']),weights=weights[model])                
                plt.plot((Sd_mean-Su_mean).rolling(lat = rw, center=True).mean(),
                         Sd_mean.lat.rolling(lat = rw, center=True).mean(),color=cols[model][1],\
                           linestyle=['-',':'][ll], linewidth=1,alpha=1,
                           label=model)                
                
            elif comps == 'albedo':
                albedo_mean = average_da(albedo[model+'_albedo'+tt],dim=(['lon']),weights=weights[model])
                plt.plot(albedo_mean.rolling(lat = rw, center=True).mean(),
                         albedo_mean.lat.rolling(lat = rw, center=True).mean(),color=cols[model][1],\
                           linestyle=['-',':'][ll], linewidth=1,alpha=1,
                           label=model)
                
            elif comps == 'sensible':
                Hfss_mean = average_da(Hfss[model+'_hfss'+tt],dim=(['lon']),weights=weights[model])
                plt.plot(Hfss_mean.rolling(lat = rw, center=True).mean(),
                         Hfss_mean.lat.rolling(lat = rw, center=True).mean(),color=cols[model][1],\
                           linestyle=['-',':'][ll], linewidth=1,alpha=1,
                           label=model)
                
            elif comps == 'latent':
                Hfls_mean = average_da(Hfls[model+'_hfls'+tt],dim=(['lon']),weights=weights[model])
                plt.plot(Hfls_mean.rolling(lat = rw, center=True).mean(),
                         Hfls_mean.lat.rolling(lat = rw, center=True).mean(),color=cols[model][1],\
                           linestyle=['-',':'][ll], linewidth=1,alpha=1,
                           label=model)
                
            elif comps == 'turb':
#                ET_mean = average_da(ET[model+'_et'+tt],dim=(['lon']),weights=weights[model])
#                
#                plt.plot(ET_mean.rolling(lat = rw, center=True).mean(),
#                         ET_mean.lat.rolling(lat = rw, center=True).mean(),color=cols[model][1],\
#                           linestyle=['-',':'][ll], linewidth=1,alpha=1,
#                           label=model)
                Hfls_mean = average_da(Hfls[model+'_hfls'+tt],dim=(['lon']),weights=weights[model])
                Hfss_mean = average_da(Hfss[model+'_hfss'+tt],dim=(['lon']),weights=weights[model])
                Turb = Hfls_mean + Hfss_mean
                plt.plot(Turb.rolling(lat = rw, center=True).mean(),
                         Turb.lat.rolling(lat = rw, center=True).mean(),color=cols[model][1],\
                           linestyle=['-',':'][ll], linewidth=1,alpha=1,
                           label=model)                
                
            elif comps == 'avail':
#                T_T_mean = average_da(T_T[model+'_ts'+tt],dim=(['lon']),weights=weights[model])
#                plt.plot(T_T_mean.rolling(lat = rw, center=True).mean(),
#                         T_T_mean.lat.rolling(lat = rw, center=True).mean(),color=cols[model][1],\
#                           linestyle=['-',':'][ll], linewidth=1,alpha=1,
#                           label=model) 
                
                Su_mean = average_da(Su[model+'_rsus'+tt],dim=(['lon']),weights=weights[model])
                Sd_mean = average_da(Sd[model+'_rsds'+tt],dim=(['lon']),weights=weights[model])
                Ld_mean = average_da(Ld[model+'_rlds'+tt],dim=(['lon']),weights=weights[model])
                avail = Sd_mean - Su_mean + Ld_mean 
                plt.plot(avail.rolling(lat = rw, center=True).mean(),
                         avail.lat.rolling(lat = rw, center=True).mean(),color=cols[model][1],\
                           linestyle=['-',':'][ll], linewidth=1,alpha=1,
                           label=model)      
                
            elif comps == 'Ts':
                T_T_mean = average_da(T_T[model+'_ts'+tt],dim=(['lon']),weights=weights[model])
                plt.plot(T_T_mean.rolling(lat = rw, center=True).mean(),
                         T_T_mean.lat.rolling(lat = rw, center=True).mean(),color=cols[model][1],\
                           linestyle=['-',':'][ll], linewidth=1,alpha=1,
                           label=model) 
                                    
                
            ll = ll - 1
        
        
        names = ['turbulent','available','rnet',\
                 'latent','sensible','clouds',\
                 'SWd',  'LWd','LWd - \nLWd_clear_sky']
        
        
        units = ['W m$^{-2}$','W m$^{-2}$','W m$^{-2}$',\
                 'W m$^{-2}$','W m$^{-2}$','%',\
                 'W m$^{-2}$','W m$^{-2}$','W m$^{-2}$']
        
    #    ax.text(0.5,1.05,
    #            names[comps],
    #            color='black', #['red','darkred','orange','cyan','blue','grey','black','grey'][comps],
    #            horizontalalignment='center',
    #            verticalalignment='center',
    #            transform=ax.transAxes,
    #            fontsize=cexx,
    #            bbox={'facecolor':'white', 'alpha':0.7, 'pad':5})
    #    
        ax.text(-0.1,1.05,['(a)','(b)','(c)','(d)','(e)','(f)','(g)','(h)','(i)','(j)'][p],
                color='k',
                horizontalalignment='left',
                verticalalignment='center',
                transform=ax.transAxes,
                fontsize=cexx,
                bbox={'facecolor':'white', 'alpha':0.7, 'pad':5})
        
        
        
        yticks = np.arange(-70,90,20) 
        ax.set_yticks(yticks, minor=False)                
        plt.yticks(fontsize=cexx) 
        
    #    ax.autoscale()    
        ax.set_xticks(xticks, minor=False) 
        plt.xticks(fontsize=cexx)
        ax.tick_params(axis='x', labelsize=cexx)
        
    
        
    
        if p in [1,2,4,5,7,8,9]:
            dummy = [item.get_text() for item in ax.get_yticklabels()]
            ax.set_yticklabels(['']*len(dummy))
            ax.set_ylabel('',fontsize=cexx)
        
        
        if p in [0,3,6]:
            ax.set_ylabel('Latitude',fontsize=cexx2)
        
        
    #    if p in range(0,4):
    #        dummy = [item.get_text() for item in ax.get_xticklabels()]
    #        ax.set_xticklabels(['']*len(dummy))
    #        ax.set_xlabel('',fontsize=cexx)
    #    else:
        ax.set_xlabel(names[p]+' [' +units[p]+']',fontsize=cexx)
    
    
    
    
    
        if p==0:
            import matplotlib.lines as mlines
            x, y = np.array([[2.3,2.3], [0.9,0.9]])
            line = mlines.Line2D(x , y , 
                                 lw=2, linestyle='-', 
                                 alpha=1, color='black',
                                 label = 'global',
                                 transform=ax.transAxes)
            
            x, y = np.array([[2.3,2.1], [0.9,0.9]])
            line2 = mlines.Line2D(x , y , 
                                 lw=2, linestyle='-', 
                                 alpha=1,color='black',
                                 label = 'over $\Delta$F',
                                 transform=ax.transAxes)
            if tt == '':
                plt.legend(handles=[line], ncol=1, loc='center left',
                                     fontsize=cexx-2,
                       bbox_to_anchor=(-0.0, 1.3), borderaxespad=0.)
            else:    
                plt.legend(handles=[line2], ncol=1, loc='center left',
                                     fontsize=cexx-2,
                       bbox_to_anchor=(-0.0, 1.3), borderaxespad=0.)
    
            handlesX, labelsX = ax.get_legend_handles_labels()
        
        ##-------------------------------------------
        ## legend in empty plot
        if p==1:
    #        ax = plt.subplot(gs1[j,i])
            if len(labelsX) == 20:
                indices = np.arange(1,20,2) 
            else:
                indices = np.arange(0,9,1) 
                
            leg = ax.legend(np.take(handlesX,indices),
                          np.take(labelsX,indices),
                          ncol=3,
                          fontsize=cexx-2,
                          prop={'size':cexx-2},
                          title='',
                          loc='center left', 
    #                          bbox_to_anchor=(0.8, 0.5), borderaxespad=0)
                          bbox_to_anchor=(-0.1, 1.3), borderaxespad=0)
            
            for line in leg.get_lines():
                line.set_linewidth(2.0)    
    
    
        i = i + 1
        
        if p in [2,5]:
            i = 0
            j = j + 1
        p = p + 1
    
    fig.show()
    fig.tight_layout()
    if tt == '':
        tag = 'global'
    else:
        tag = 'GD'
    pngfile='/Users/lenaboysen/Google Drive/LUMIP/Time_series/Zonal_fluxes_ALL_'+tag+'.png' 
    fig.savefig(pngfile,dpi=300, bbox_inches='tight')
    plt.close()  





#%% Cloud cover and rlds vs rldscs

fig_scale = 2
fig = plt.figure(figsize=(4*fig_scale,3*fig_scale))
gs1 = plt.GridSpec(1,2, height_ratios=[1],width_ratios=[1,1])

gs1.update(wspace=0.1, hspace=0.5) 
i = 0; j = 0;  p = 0
            

for comps in range(0,2):
    print model
    print 'i = ' + str(i) + ' j = ' + str(j)
    
    ax = plt.subplot(gs1[j,i])
    
    plt.axhline(y=0, color='k', linestyle='-',linewidth= 1)   
            
    sns.set_context("notebook", font_scale=1.2, rc={"lines.linewidth": 1.2})
    sns.set_style("whitegrid")
    custom_style = {
            'grid.color': '0.8',
            'grid.linestyle': '--',
            'grid.linewidth': 0.5,
            }
    sns.set_style(custom_style)  
    ax.set_ylim(-90, 90)    


    if comps in [0]:    
        xticks = np.arange(-2,3,1) 
    elif comps in [1]:
        xticks = np.arange(-3,4,1) 
    
    ax.set_xlim(xticks[0],xticks[len(xticks)-1])  

                            
    plt.axvline(x=0, color='k', linestyle='-',linewidth= 1) 
    
    rw_base = 10

    for model in models:
        ll = 1      
            
        for tt in ['','_GD']:            
            
            lat = T_Su[model+'_rsus'+tt].lat
            latdiff = np.append(np.diff(lat),np.diff(lat)[0])
            
#            latdiffx = round(latdiff.mean(),1)

#            if ll == 1:
#                print 'latdifx = ' + str(latdiffx)
            if (latdiffx > rw_base ):
                rw =  1
            else:
                rw = int(rw_base/latdiffx)
#            if ll == 1:
#                print 'rw = ' + str(rw) 
#                print 'outcome = ' + str(rw*latdiffx)
                    
                
            if comps == 0:    
                if model in ['CESM2']:#'MPI','IPSL',
                    print 'nothing to plot'
                else:
                    Ldcs_mean = average_da(Ldcs[model+'_rldscs'+tt],dim=(['lon']),weights=weights[model]) -\
                                average_da(Ld[model+'_rlds'+tt],dim=(['lon']),weights=weights[model])
                                
                    plt.plot(Ldcs_mean.rolling(lat = rw, center=True).mean(), 
                         Ldcs_mean.lat.rolling(lat = rw, center=True).mean(),color=cols[model][1],\
                           linestyle=['-',':'][ll], linewidth=1.5,alpha=1,
                           label=model)        

            elif comps == 1:
#                if model in ['MPI','IPSL']:
#                    print 'nothing to plot'
#                else:
                CLT_mean = average_da(CLT[model+'_clt'+tt],dim=(['lon']),weights=weights[model])
                
                plt.plot(CLT_mean.rolling(lat = rw, center=True).mean(),
                         CLT_mean.lat.rolling(lat = rw, center=True).mean(),color=cols[model][1],\
                           linestyle=['-',':'][ll], linewidth=1.5,alpha=1,
                           label=model)                
                      
            ll = ll - 1
        
    names = ['LW clear sky - LW cloudy','cloud cover fraction']
    units = ['W m$^{-2}$','%']
    
#    
    ax.text(-0.1,1.05,['(a)','(b)','(c)','(d)','(e)','(f)','(g)','(h)','(i)'][p],
            color='k',
            horizontalalignment='left',
            verticalalignment='center',
            transform=ax.transAxes,
            fontsize=cexx,
            bbox={'facecolor':'white', 'alpha':0.7, 'pad':5})
    
    
    yticks = np.arange(-70,90,20) 
    ax.set_yticks(yticks, minor=False)                
    plt.yticks(fontsize=cexx) 
    
#    ax.autoscale()    
    ax.set_xticks(xticks, minor=False) 
    plt.xticks(fontsize=cexx)
    ax.tick_params(axis='x', labelsize=cexx)
    

    if p in [1,2,3,5,6,7]:
        dummy = [item.get_text() for item in ax.get_yticklabels()]
        ax.set_yticklabels(['']*len(dummy))
        ax.set_ylabel('',fontsize=cexx)
    
    
    if p in [0,4]:
        ax.set_ylabel('Latitude',fontsize=cexx2)
    
    ax.set_xlabel(names[p]+' [' +units[p]+']',fontsize=cexx)

    if p==0:
        import matplotlib.lines as mlines
        x, y = np.array([[2.3,2.3], [0.9,0.9]])
        line = mlines.Line2D(x , y , 
                             lw=2, linestyle=':', 
                             alpha=1, color='black',
                             label = 'global',
                             transform=ax.transAxes)
        
        x, y = np.array([[2.3,2.1], [0.9,0.9]])
        line2 = mlines.Line2D(x , y , 
                             lw=2, linestyle='-', 
                             alpha=1,color='black',
                             label = 'over $\Delta$F',
                             transform=ax.transAxes)
        
        plt.legend(handles=[line,line2], ncol=1, loc='center left',
                                 fontsize=cexx-2,
                   bbox_to_anchor=(-0.0, 1.1), borderaxespad=0.)

    
    handlesX, labelsX = ax.get_legend_handles_labels()
    ##-------------------------------------------
    ## legend in empty plot
    if p==1:
#        ax = plt.subplot(gs1[j,i])
        if len(labelsX) == 14:
            indices = np.arange(1,14,2) 
        else:
            indices = np.arange(0,5,1) 
            
        leg = ax.legend(np.take(handlesX,indices),
                      np.take(labelsX,indices),
                      ncol=3,
                      fontsize=cexx-2,
                      prop={'size':cexx-2},
                      title='',
                      loc='center left', 
#                          bbox_to_anchor=(0.8, 0.5), borderaxespad=0)
                      bbox_to_anchor=(-0, 1.1), borderaxespad=0)
        
        for line in leg.get_lines():
            line.set_linewidth(2.0)    


    i = i + 1
    
    if p in [3, 7]:
        i = 0
        j = j + 1
    p = p + 1

fig.show()
fig.tight_layout()
pngfile='/Users/lenaboysen/Google Drive/LUMIP/Time_series/Zonal_clouds_ALL.png' 
fig.savefig(pngfile,dpi=300, bbox_inches='tight')
plt.close()  





#%% Bowen and Partitioning

fig_scale = 2
fig = plt.figure(figsize=(3*fig_scale,4*fig_scale))
gs1 = plt.GridSpec(2,2, height_ratios=[1,1],width_ratios=[1,1])

gs1.update(wspace=0.2, hspace=0.25) 
i = 0; j = 0;  p = 0
            


rw = 10; ll = 0 ; tt =  '_GD'; m = 0 
labelsX = ['']*len(models) ; handelsX = ['']*len(models)

for model in models:
    
    QT_CTL = SEB[model][model+'_CTL_hfss'+tt] + SEB[model][model+'_CTL_hfls'+tt]
    QT_DEF = SEB[model][model+'_DEF_hfss'+tt] + SEB[model][model+'_DEF_hfls'+tt]
    QA_CTL = SEB[model][model+'_CTL_rsds'+tt] - SEB[model][model+'_CTL_rsus'+tt] \
            + SEB[model][model+'_CTL_rlds'+tt]
    QA_DEF = SEB[model][model+'_DEF_rsds'+tt] - SEB[model][model+'_DEF_rsus'+tt] \
            + SEB[model][model+'_DEF_rlds'+tt]    
     
    QT_QA_CTL =  QT_CTL / QA_CTL * 100
    QT_QA_DEF =  QT_DEF / QA_DEF * 100    
    
    dQT_QA = QT_QA_CTL - QT_QA_DEF

    
    
    
    ## ====== Partitioning QT/QA CTL ==============
    ax = plt.subplot(gs1[0,0])
    plt.axhline(y=0, color='k', linestyle='-',linewidth= 1)   
            
    sns.set_context("notebook", font_scale=1.2, rc={"lines.linewidth": 1.2})
    sns.set_style("whitegrid")
    custom_style = {
            'grid.color': '0.8',
            'grid.linestyle': '--',
            'grid.linewidth': 0.5,
            }
    sns.set_style(custom_style)  
    ax.set_ylim(-90, 90)        
    xticks = np.arange(0,35,5)   
    ax.set_xlim(xticks[0],xticks[len(xticks)-1])                                
    plt.axvline(x=0, color='k', linestyle='-',linewidth= 1) 
    
    plt.plot(QT_QA_CTL.mean('lon').rolling(lat = rw, center=True).mean(), 
                 QT_QA_CTL.lat.rolling(lat = rw, center=True).mean(),color=cols[model][0],\
                   linestyle=['-',':'][ll], linewidth=1,alpha=1,
                   label=list(models_name[model])[0])
    
    
    ax.text(-0.13,1.0,['(a)','(b)','(c)','(d)'][0],
            color='k',
            horizontalalignment='left',
            verticalalignment='center',
            transform=ax.transAxes,
            fontsize=cexx,
            bbox={'facecolor':'white', 'alpha':0.7, 'pad':5})
    
    yticks = np.arange(-70,90,20) 
    ax.set_yticks(yticks, minor=False)                
    plt.yticks(fontsize=cexx) 

    ax.set_xticks(xticks, minor=False) 
    plt.xticks(fontsize=cexx)
    ax.tick_params(axis='x', labelsize=cexx)

    ax.set_ylabel('Latitude',fontsize=cexx2)
    ax.set_xlabel('Partitioning of energy $Q_T$/$Q_A$ [%]',fontsize=cexx)


    ## ====== Change in Partitioning QT/QA_CTL - QT/QA_DEF  ==============
    ax2 = plt.subplot(gs1[0,1])
    plt.axhline(y=0, color='k', linestyle='-',linewidth= 1)   
            
    sns.set_context("notebook", font_scale=1.2, rc={"lines.linewidth": 1.2})
    sns.set_style("whitegrid")
    custom_style = {
            'grid.color': '0.8',
            'grid.linestyle': '--',
            'grid.linewidth': 0.5,
            }
    sns.set_style(custom_style)  
    ax2.set_ylim(-90, 90)        
    xticks = np.arange(-3,2,1)   
    ax2.set_xlim(xticks[0],xticks[len(xticks)-1])                                
    plt.axvline(x=0, color='k', linestyle='-',linewidth= 1) 
    
    plt.plot((QT_QA_DEF-QT_QA_CTL).mean('lon').rolling(lat = rw, center=True).mean(), 
                 QT_QA_CTL.lat.rolling(lat = rw, center=True).mean(),color=cols[model][0],\
                   linestyle=['-',':'][ll], linewidth=1,alpha=1)
    
    
    ax2.text(-0.13,1.0,['(a)','(b)','(c)','(d)'][1],
            color='k',
            horizontalalignment='left',
            verticalalignment='center',
            transform=ax2.transAxes,
            fontsize=cexx,
            bbox={'facecolor':'white', 'alpha':0.7, 'pad':5})
    
    yticks = np.arange(-70,90,20) 
    ax2.set_yticks(yticks, minor=False)                
    plt.yticks(fontsize=cexx) 
    ax2.set_yticklabels(['']*len(yticks))
    ax2.set_xticks(xticks, minor=False) 
    plt.xticks(fontsize=cexx)
    ax2.tick_params(axis='x', labelsize=cexx)

    ax2.set_ylabel('',fontsize=cexx2)
    ax2.set_xlabel('$\Delta$ $Q_T$/$Q_A$ [%]',fontsize=cexx)
    
    ### ============= legend =============================
    if m == 0:
        x, y = np.array([[2.3,2.3], [0.9,0.9]])
        line = mlines.Line2D(x , y , 
                             lw=2, linestyle=':', 
                             alpha=1, color='black',
                             label = 'global',
                             transform=ax2.transAxes)
        
        x, y = np.array([[2.3,2.1], [0.9,0.9]])
        line2 = mlines.Line2D(x , y , 
                             lw=2, linestyle='-', 
                             alpha=1,color='black',
                             label = 'over $\Delta$F',
                             transform=ax2.transAxes)
        
        plt.legend(handles=[line,line2], ncol=1, loc='center left',
                                 fontsize=cexx-2,
                   bbox_to_anchor=(-1.2, 1.2), borderaxespad=0.)
        
    
    ## ====== Partitioning QSens/QLatent CTL  = QH/QE ==============
    QH_CTL = SEB[model][model+'_CTL_hfls'+tt]  
    QH_DEF = SEB[model][model+'_DEF_hfls'+tt]
    QS_CTL = SEB[model][model+'_CTL_hfss'+tt] 
    QS_DEF = SEB[model][model+'_DEF_hfss'+tt]    
    
    B = QS_CTL/QH_CTL * 100
    dB = QS_DEF/QH_DEF * 100 - QS_CTL/QH_CTL * 100
    
    ax3 = plt.subplot(gs1[1,0])
    plt.axhline(y=0, color='k', linestyle='-',linewidth= 1)   
            
    sns.set_context("notebook", font_scale=1.2, rc={"lines.linewidth": 1.2})
    sns.set_style("whitegrid")
    custom_style = {
            'grid.color': '0.8',
            'grid.linestyle': '--',
            'grid.linewidth': 0.5,
            }
    sns.set_style(custom_style)  
    ax3.set_ylim(-90, 110)        
    xticks = np.arange(0,110,20)   
    ax3.set_xlim(xticks[0],xticks[len(xticks)-1])                                
    plt.axvline(x=0, color='k', linestyle='-',linewidth= 1) 
    
    plt.plot(B.mean('lon').rolling(lat = rw, center=True).mean(), 
                 B.lat.rolling(lat = rw, center=True).mean(),color=cols[model][0],\
                   linestyle=['-',':'][ll], linewidth=1,alpha=1)
    
    
    ax3.text(-0.13,1.0,['(a)','(b)','(c)','(d)'][2],
            color='k',
            horizontalalignment='left',
            verticalalignment='center',
            transform=ax3.transAxes,
            fontsize=cexx,
            bbox={'facecolor':'white', 'alpha':0.7, 'pad':5})
    
    yticks = np.arange(-70,90,20) 
    ax3.set_yticks(yticks, minor=False)                
    plt.yticks(fontsize=cexx) 
    ax3.set_xticks(xticks, minor=False) 
    plt.xticks(fontsize=cexx)
    ax3.tick_params(axis='x', labelsize=cexx)

    ax3.set_ylabel('Latitude',fontsize=cexx2)
    ax3.set_xlabel('Bowen Ratio $Q_H$/$Q_E$ [%]',fontsize=cexx)    
    
    
    
    
    ## ====== Change in partitioning of QSens/QLatent = QH/QE==============    
    ax4 = plt.subplot(gs1[1,1])
    plt.axhline(y=0, color='k', linestyle='-',linewidth= 1)   
            
    sns.set_context("notebook", font_scale=1.2, rc={"lines.linewidth": 1.2})
    sns.set_style("whitegrid")
    custom_style = {
            'grid.color': '0.8',
            'grid.linestyle': '--',
            'grid.linewidth': 0.5,
            }
    sns.set_style(custom_style)  
    ax4.set_ylim(-90, 90)        
    xticks = np.arange(-20,20,5)   
    ax4.set_xlim(xticks[0],xticks[len(xticks)-1])                                
    plt.axvline(x=0, color='k', linestyle='-',linewidth= 1) 
    
    plt.plot(dB.mean('lon').rolling(lat = rw, center=True).mean(), 
                 B.lat.rolling(lat = rw, center=True).mean(),color=cols[model][0],\
                   linestyle=['-',':'][ll], linewidth=1,alpha=1)
    
    
    ax4.text(-0.13,1.0,['(a)','(b)','(c)','(d)'][3],
            color='k',
            horizontalalignment='left',
            verticalalignment='center',
            transform=ax4.transAxes,
            fontsize=cexx,
            bbox={'facecolor':'white', 'alpha':0.7, 'pad':5})
    
    yticks = np.arange(-70,90,20) 
    ax4.set_yticks(yticks, minor=False)                
    plt.yticks(fontsize=cexx) 
    ax4.set_yticklabels(['']*len(yticks))
    ax4.set_xticks(xticks, minor=False) 
    plt.xticks(fontsize=cexx)
    ax4.tick_params(axis='x', labelsize=cexx)

    ax4.set_ylabel('',fontsize=cexx2)
    ax4.set_xlabel('$\Delta$ $Q_H$/$Q_E$ [%]',fontsize=cexx)    
        
    
    m = m + 1


### ============= legend =============================
handlesX, labelsX = ax.get_legend_handles_labels()

leg = ax.legend(handlesX,
              labelsX,
              ncol=3,
              fontsize=cexx-2,
              prop={'size':cexx-2},
              title='',
              loc='center left', 
              bbox_to_anchor=(0.4 , 1.2), borderaxespad=0)
for line in leg.get_lines():
    line.set_linewidth(2.0) 



    
fig.show()
fig.tight_layout()
pngfile='/Users/lenaboysen/Google Drive/LUMIP/Time_series/Zonal_Bowen_Avail_ALL.png' 
fig.savefig(pngfile,dpi=300, bbox_inches='tight')
plt.close()  

#%% Bowen and Partitioning

fig_scale = 2
fig = plt.figure(figsize=(4*fig_scale,3*fig_scale))
gs1 = plt.GridSpec(2,4, height_ratios=[1,1],width_ratios=[1,1,1,1])

gs1.update(wspace=0.2, hspace=0.28) 
i = 0; j = 0;  p = 0
            


rw = 10; ll = 0 ; tt =  '_GD'; m = 0 
labelsX = ['']*len(models) ; handelsX = ['']*len(models)

for model in models:
    
    QT_CTL = SEB[model][model+'_CTL_hfss'+tt] + SEB[model][model+'_CTL_hfls'+tt]
    QT_DEF = SEB[model][model+'_DEF_hfss'+tt] + SEB[model][model+'_DEF_hfls'+tt]
    QA_CTL = SEB[model][model+'_CTL_rsds'+tt] - SEB[model][model+'_CTL_rsus'+tt] \
            + SEB[model][model+'_CTL_rlds'+tt]
    QA_DEF = SEB[model][model+'_DEF_rsds'+tt] - SEB[model][model+'_DEF_rsus'+tt] \
            + SEB[model][model+'_DEF_rlds'+tt]    
     
    QT_QA_CTL =  QT_CTL / QA_CTL * 100
    QT_QA_DEF =  QT_DEF / QA_DEF * 100    
    
    dQT_QA =  QT_QA_DEF - QT_QA_CTL
    dQA = QA_DEF - QA_CTL
    dQT = QT_DEF - QT_CTL
    dQH = SEB[model][model+'_DEF_hfss'+tt] - SEB[model][model+'_CTL_hfss'+tt]
    dQE = SEB[model][model+'_DEF_hfls'+tt] - SEB[model][model+'_CTL_hfls'+tt]
    
    ## ========= Change in QA =====================
    axA = plt.subplot(gs1[0,0])
    plt.axhline(y=0, color='k', linestyle='-',linewidth= 1)   
            
    sns.set_context("notebook", font_scale=1.2, rc={"lines.linewidth": 1.2})
    sns.set_style("whitegrid")
    custom_style = {
            'grid.color': '0.8',
            'grid.linestyle': '--',
            'grid.linewidth': 0.5,
            }
    sns.set_style(custom_style)  
    axA.set_ylim(-90, 90)        
    xticks = np.arange(-20,15,5)   
    axA.set_xlim(xticks[0],xticks[len(xticks)-1])                                
    plt.axvline(x=0, color='k', linestyle='-',linewidth= 1) 
    
#    print dQA.mean('lon').rolling(lat = rw, center=True).mean()
    plt.plot(dQA.mean('lon').rolling(lat = rw, center=True).mean(), 
                 dQA.lat.rolling(lat = rw, center=True).mean(),color=cols[model][0],\
                   linestyle=['-',':'][ll], linewidth=1,alpha=1,
                   label=list(models_name[model])[0])
    
    
    axA.text(-0.18,1.0,['(a)','(b)','(c)','(d)','(e)','(f)','(g)','(h)'][0],
            color='k',
            horizontalalignment='left',
            verticalalignment='center',
            transform=axA.transAxes,
            fontsize=cexx,
            bbox={'facecolor':'white', 'alpha':0.7, 'pad':5})
    
    yticks = np.arange(-70,90,20) 
    axA.set_yticks(yticks, minor=False)                
    plt.yticks(fontsize=cexx) 

    axA.set_xticks(xticks, minor=False) 
    for index, label in enumerate(axA.xaxis.get_ticklabels()):
                if index % 2 != 0: # show only every 5th label
                    label.set_visible(False)
    plt.xticks(fontsize=cexx)
    axA.tick_params(axis='x', labelsize=cexx)

    axA.set_ylabel('Latitude',fontsize=cexx2)
    axA.set_xlabel('$\Delta Q_A$ [W m$^{-2}$]',fontsize=cexx)



    ## ========= Change in QT =====================
    axT = plt.subplot(gs1[1,0])
    plt.axhline(y=0, color='k', linestyle='-',linewidth= 1)   
            
    sns.set_context("notebook", font_scale=1.2, rc={"lines.linewidth": 1.2})
    sns.set_style("whitegrid")
    custom_style = {
            'grid.color': '0.8',
            'grid.linestyle': '--',
            'grid.linewidth': 0.5,
            }
    sns.set_style(custom_style)  
    axT.set_ylim(-90, 90)        
    xticks = np.arange(-20,15,5)   
    ax.set_xlim(xticks[0],2)                                
    plt.axvline(x=0, color='k', linestyle='-',linewidth= 1) 
    
    plt.plot(dQT.mean('lon').rolling(lat = rw, center=True).mean(), 
                 dQT.lat.rolling(lat = rw, center=True).mean(),color=cols[model][0],\
                   linestyle=['-',':'][ll], linewidth=1,alpha=1,
                   label=list(models_name[model])[0])
    
    
    axT.text(-0.18,1.0,['(a)','(b)','(c)','(d)','(e)','(f)','(g)','(h)'][4],
            color='k',
            horizontalalignment='left',
            verticalalignment='center',
            transform=axT.transAxes,
            fontsize=cexx,
            bbox={'facecolor':'white', 'alpha':0.7, 'pad':5})
    
    yticks = np.arange(-70,90,20) 
    axT.set_yticks(yticks, minor=False)                
    plt.yticks(fontsize=cexx) 

    axT.set_xticks(xticks, minor=False) 
    for index, label in enumerate(axT.xaxis.get_ticklabels()):
            if index % 2 != 0: # show only every 5th label
                label.set_visible(False)
    plt.xticks(fontsize=cexx)
    axT.tick_params(axis='x', labelsize=cexx)

    axT.set_ylabel('Latitude',fontsize=cexx2)
    axT.set_xlabel('$\Delta Q_T$ [W m$^{-2}$]',fontsize=cexx)
#
    
    
    
    ## ========= Change in QH =====================
    axA = plt.subplot(gs1[0,1])
    plt.axhline(y=0, color='k', linestyle='-',linewidth= 1)   
            
    sns.set_context("notebook", font_scale=1.2, rc={"lines.linewidth": 1.2})
    sns.set_style("whitegrid")
    custom_style = {
            'grid.color': '0.8',
            'grid.linestyle': '--',
            'grid.linewidth': 0.5,
            }
    sns.set_style(custom_style)  
    axA.set_ylim(-90, 90)        
    xticks = np.arange(-15,15,5)   
    axA.set_xlim(xticks[0],xticks[len(xticks)-1])                                
    plt.axvline(x=0, color='k', linestyle='-',linewidth= 1) 
    
#    print dQA.mean('lon').rolling(lat = rw, center=True).mean()
    plt.plot(dQH.mean('lon').rolling(lat = rw, center=True).mean(), 
                 dQA.lat.rolling(lat = rw, center=True).mean(),color=cols[model][0],\
                   linestyle=['-',':'][ll], linewidth=1,alpha=1,
                   label=list(models_name[model])[0])
    
    
    axA.text(-0.18,1.0,['(a)','(b)','(c)','(d)','(e)','(f)','(g)','(h)'][1],
            color='k',
            horizontalalignment='left',
            verticalalignment='center',
            transform=axA.transAxes,
            fontsize=cexx,
            bbox={'facecolor':'white', 'alpha':0.7, 'pad':5})
    
    yticks = np.arange(-70,90,20) 
    axA.set_yticks(yticks, minor=False)                
    plt.yticks(fontsize=cexx) 
    axA.set_yticklabels(['']*len(yticks))
    axA.set_xticks(xticks, minor=False) 
    for index, label in enumerate(axA.xaxis.get_ticklabels()):
                if index % 2 != 0: # show only every 5th label
                    label.set_visible(False)
    plt.xticks(fontsize=cexx)
    axA.tick_params(axis='x', labelsize=cexx)

    axA.set_ylabel('',fontsize=cexx2)
    axA.set_xlabel('$\Delta Q_H$ [W m$^{-2}$]',fontsize=cexx)



    ## ========= Change in QE =====================
    axT = plt.subplot(gs1[1,1])
    plt.axhline(y=0, color='k', linestyle='-',linewidth= 1)   
            
    sns.set_context("notebook", font_scale=1.2, rc={"lines.linewidth": 1.2})
    sns.set_style("whitegrid")
    custom_style = {
            'grid.color': '0.8',
            'grid.linestyle': '--',
            'grid.linewidth': 0.5,
            }
    sns.set_style(custom_style)  
    axT.set_ylim(-90, 90)        
    xticks = np.arange(-15,15,5)   
    ax.set_xlim(xticks[0],2)                                
    plt.axvline(x=0, color='k', linestyle='-',linewidth= 1) 
    
    plt.plot(dQE.mean('lon').rolling(lat = rw, center=True).mean(), 
                 dQT.lat.rolling(lat = rw, center=True).mean(),color=cols[model][0],\
                   linestyle=['-',':'][ll], linewidth=1,alpha=1,
                   label=list(models_name[model])[0])
    
    
    axT.text(-0.18,1.0,['(a)','(b)','(c)','(d)','(e)','(f)','(g)','(h)'][5],
            color='k',
            horizontalalignment='left',
            verticalalignment='center',
            transform=axT.transAxes,
            fontsize=cexx,
            bbox={'facecolor':'white', 'alpha':0.7, 'pad':5})
    
    yticks = np.arange(-70,90,20) 
    axT.set_yticks(yticks, minor=False)                
    plt.yticks(fontsize=cexx) 
    axT.set_yticklabels(['']*len(yticks))
    axT.set_xticks(xticks, minor=False) 
    for index, label in enumerate(axT.xaxis.get_ticklabels()):
            if index % 2 != 0: # show only every 5th label
                label.set_visible(False)
    plt.xticks(fontsize=cexx)
    axT.tick_params(axis='x', labelsize=cexx)

    axT.set_ylabel('',fontsize=cexx2)
    axT.set_xlabel('$\Delta Q_E$ [W m$^{-2}$]',fontsize=cexx)


    
    ## ====== Partitioning QT/QA CTL ==============
    ax = plt.subplot(gs1[0,2])
    plt.axhline(y=0, color='k', linestyle='-',linewidth= 1)   
            
    sns.set_context("notebook", font_scale=1.2, rc={"lines.linewidth": 1.2})
    sns.set_style("whitegrid")
    custom_style = {
            'grid.color': '0.8',
            'grid.linestyle': '--',
            'grid.linewidth': 0.5,
            }
    sns.set_style(custom_style)  
    ax.set_ylim(-90, 90)        
    xticks = np.arange(0,35,5)   
    ax.set_xlim(xticks[0],xticks[len(xticks)-1])                                
    plt.axvline(x=0, color='k', linestyle='-',linewidth= 1) 
    
    plt.plot(QT_QA_CTL.mean('lon').rolling(lat = rw, center=True).mean(), 
                 QT_QA_CTL.lat.rolling(lat = rw, center=True).mean(),color=cols[model][0],\
                   linestyle=['-',':'][ll], linewidth=1,alpha=1,
                   label=list(models_name[model])[0])
    
    
    ax.text(-0.13,1.0,['(a)','(b)','(c)','(d)','(e)','(f)','(g)','(h)'][2],
            color='k',
            horizontalalignment='left',
            verticalalignment='center',
            transform=ax.transAxes,
            fontsize=cexx,
            bbox={'facecolor':'white', 'alpha':0.7, 'pad':5})
    
    yticks = np.arange(-70,90,20) 
    ax.set_yticks(yticks, minor=False)                
    plt.yticks(fontsize=cexx) 
    ax.set_yticklabels(['']*len(yticks))
    ax.set_xticks(xticks, minor=False) 
    plt.xticks(fontsize=cexx)
    ax.tick_params(axis='x', labelsize=cexx)

    ax.set_ylabel('',fontsize=cexx2)
    ax.set_xlabel('$Q_T$/$Q_A$ [%]',fontsize=cexx)


    ## ====== Change in Partitioning QT/QA_CTL - QT/QA_DEF  ==============
    ax2 = plt.subplot(gs1[0,3])
    plt.axhline(y=0, color='k', linestyle='-',linewidth= 1)   
            
    sns.set_context("notebook", font_scale=1.2, rc={"lines.linewidth": 1.2})
    sns.set_style("whitegrid")
    custom_style = {
            'grid.color': '0.8',
            'grid.linestyle': '--',
            'grid.linewidth': 0.5,
            }
    sns.set_style(custom_style)  
    ax2.set_ylim(-90, 90)        
    xticks = np.arange(-3,2,1)   
    ax2.set_xlim(xticks[0],xticks[len(xticks)-1])                                
    plt.axvline(x=0, color='k', linestyle='-',linewidth= 1) 
    
    plt.plot((QT_QA_DEF-QT_QA_CTL).mean('lon').rolling(lat = rw, center=True).mean(), 
                 QT_QA_CTL.lat.rolling(lat = rw, center=True).mean(),color=cols[model][0],\
                   linestyle=['-',':'][ll], linewidth=1,alpha=1)
    
    
    ax2.text(-0.13,1.0,['(a)','(b)','(c)','(d)','(e)','(f)','(g)','(h)'][3],
            color='k',
            horizontalalignment='left',
            verticalalignment='center',
            transform=ax2.transAxes,
            fontsize=cexx,
            bbox={'facecolor':'white', 'alpha':0.7, 'pad':5})
    
    yticks = np.arange(-70,90,20) 
    ax2.set_yticks(yticks, minor=False)                
    plt.yticks(fontsize=cexx) 
    ax2.set_yticklabels(['']*len(yticks))
    ax2.set_xticks(xticks, minor=False) 
    plt.xticks(fontsize=cexx)
    ax2.tick_params(axis='x', labelsize=cexx)

    ax2.set_ylabel('',fontsize=cexx2)
    ax2.set_xlabel('$\Delta$ $Q_T$/$Q_A$ [%]',fontsize=cexx)
    
    ### ============= legend =============================
    if m == 0:
        x, y = np.array([[2.3,2.3], [0.9,0.9]])
        line = mlines.Line2D(x , y , 
                             lw=2, linestyle=':', 
                             alpha=1, color='black',
                             label = 'global',
                             transform=ax2.transAxes)
        
        x, y = np.array([[2.3,2.1], [0.9,0.9]])
        line2 = mlines.Line2D(x , y , 
                             lw=2, linestyle='-', 
                             alpha=1,color='black',
                             label = 'over $\Delta$F',
                             transform=ax2.transAxes)
        
        plt.legend(handles=[line,line2], ncol=1, loc='center left',
                                 fontsize=cexx-2,
                   bbox_to_anchor=(-3.2, 1.2), borderaxespad=0.)
        
    
    
    
    
    
    
    
    ## ====== Partitioning QSens/QLatent CTL  = QH/QE ==============
    QH_CTL = SEB[model][model+'_CTL_hfls'+tt]  
    QH_DEF = SEB[model][model+'_DEF_hfls'+tt]
    QS_CTL = SEB[model][model+'_CTL_hfss'+tt] 
    QS_DEF = SEB[model][model+'_DEF_hfss'+tt]    
    
    B = QS_CTL/QH_CTL * 100
    dB = QS_DEF/QH_DEF * 100 - QS_CTL/QH_CTL * 100
    
    ax3 = plt.subplot(gs1[1,2])
    plt.axhline(y=0, color='k', linestyle='-',linewidth= 1)   
            
    sns.set_context("notebook", font_scale=1.2, rc={"lines.linewidth": 1.2})
    sns.set_style("whitegrid")
    custom_style = {
            'grid.color': '0.8',
            'grid.linestyle': '--',
            'grid.linewidth': 0.5,
            }
    sns.set_style(custom_style)  
    ax3.set_ylim(-90, 110)        
    xticks = np.arange(0,110,20)   
    ax3.set_xlim(xticks[0],xticks[len(xticks)-1])                                
    plt.axvline(x=0, color='k', linestyle='-',linewidth= 1) 
    
    plt.plot(B.mean('lon').rolling(lat = rw, center=True).mean(), 
                 B.lat.rolling(lat = rw, center=True).mean(),color=cols[model][0],\
                   linestyle=['-',':'][ll], linewidth=1,alpha=1)
    
    
    ax3.text(-0.13,1.0,['(a)','(b)','(c)','(d)','(e)','(f)','(g)','(h)'][6],
            color='k',
            horizontalalignment='left',
            verticalalignment='center',
            transform=ax3.transAxes,
            fontsize=cexx,
            bbox={'facecolor':'white', 'alpha':0.7, 'pad':5})
    
    yticks = np.arange(-70,90,20) 
    ax3.set_yticks(yticks, minor=False)                
    plt.yticks(fontsize=cexx) 
    ax3.set_yticklabels(['']*len(yticks))
    ax3.set_xticks(xticks, minor=False) 
    plt.xticks(fontsize=cexx)
    ax3.tick_params(axis='x', labelsize=cexx)

    ax3.set_ylabel('',fontsize=cexx2)
    ax3.set_xlabel('Bowen $Q_H$/$Q_E$ [%]',fontsize=cexx)    
    
    
    
    
    ## ====== Change in partitioning of QSens/QLatent = QH/QE==============    
    ax4 = plt.subplot(gs1[1,3])
    plt.axhline(y=0, color='k', linestyle='-',linewidth= 1)   
            
    sns.set_context("notebook", font_scale=1.2, rc={"lines.linewidth": 1.2})
    sns.set_style("whitegrid")
    custom_style = {
            'grid.color': '0.8',
            'grid.linestyle': '--',
            'grid.linewidth': 0.5,
            }
    sns.set_style(custom_style)  
    ax4.set_ylim(-90, 90)        
    xticks = np.arange(-20,20,5)   
    ax4.set_xlim(xticks[0],xticks[len(xticks)-1])                                
    plt.axvline(x=0, color='k', linestyle='-',linewidth= 1) 
    
    plt.plot(dB.mean('lon').rolling(lat = rw, center=True).mean(), 
                 B.lat.rolling(lat = rw, center=True).mean(),color=cols[model][0],\
                   linestyle=['-',':'][ll], linewidth=1,alpha=1)
    
    
    ax4.text(-0.13,1.0,['(a)','(b)','(c)','(d)','(e)','(f)','(g)','(h)'][7],
            color='k',
            horizontalalignment='left',
            verticalalignment='center',
            transform=ax4.transAxes,
            fontsize=cexx,
            bbox={'facecolor':'white', 'alpha':0.7, 'pad':5})
    
    yticks = np.arange(-70,90,20) 
    ax4.set_yticks(yticks, minor=False)                
    plt.yticks(fontsize=cexx) 
    ax4.set_yticklabels(['']*len(yticks))
    ax4.set_xticks(xticks, minor=False) 
    plt.xticks(fontsize=cexx)
    ax4.tick_params(axis='x', labelsize=cexx)

    ax4.set_ylabel('',fontsize=cexx2)
    ax4.set_xlabel('$\Delta$ Bowen ($\Delta$ $Q_H$/$Q_E$) [%]',fontsize=cexx)    
        
    
    m = m + 1


### ============= legend =============================
handlesX, labelsX = ax.get_legend_handles_labels()

leg = ax.legend(handlesX,
              labelsX,
              ncol=3,
              fontsize=cexx-2,
              prop={'size':cexx-2},
              title='',
              loc='center left', 
              bbox_to_anchor=(-1.4 , 1.2), borderaxespad=0)
for line in leg.get_lines():
    line.set_linewidth(2.0) 



    
fig.show()
fig.tight_layout()
pngfile='/Users/lenaboysen/Google Drive/LUMIP/Time_series/Zonal_BowenPlus_Avail_ALL.png' 
fig.savefig(pngfile,dpi=300, bbox_inches='tight')
plt.close()  


#%%

fig_scale = 2
fig = plt.figure(figsize=(4*fig_scale,3*fig_scale))
gs1 = plt.GridSpec(1,1, height_ratios=[1],width_ratios=[1])

gs1.update(wspace=0.2, hspace=0.28) 
i = 0; j = 0;  p = 0
            


rw = 10; ll = 0 ; tt =  '_GD'; m = 0 
labelsX = ['']*len(models) ; handelsX = ['']*len(models)

for model in models:
    
    QT_CTL = SEB[model][model+'_CTL_hfss'+tt] + SEB[model][model+'_CTL_hfls'+tt]
    QT_DEF = SEB[model][model+'_DEF_hfss'+tt] + SEB[model][model+'_DEF_hfls'+tt]
    QA_CTL = SEB[model][model+'_CTL_rsds'+tt] - SEB[model][model+'_CTL_rsus'+tt] \
            + SEB[model][model+'_CTL_rlds'+tt]
    QA_DEF = SEB[model][model+'_DEF_rsds'+tt] - SEB[model][model+'_DEF_rsus'+tt] \
            + SEB[model][model+'_DEF_rlds'+tt]    
     
    QT_QA_CTL =  QT_CTL / QA_CTL * 100
    QT_QA_DEF =  QT_DEF / QA_DEF * 100    
    
    dQT_QA =  QT_QA_DEF - QT_QA_CTL
    dQA = QA_DEF - QA_CTL
    dQT = QT_DEF - QT_CTL
    dQH = SEB[model][model+'_DEF_hfss'+tt] - SEB[model][model+'_CTL_hfss'+tt]
    dQE = SEB[model][model+'_DEF_hfls'+tt] - SEB[model][model+'_CTL_hfls'+tt]
    
    ## ========= Change in QA =====================
    axA = plt.subplot(gs1[0,0])
    plt.axhline(y=0, color='k', linestyle='-',linewidth= 1)   
            
    sns.set_context("notebook", font_scale=1.2, rc={"lines.linewidth": 1.2})
    sns.set_style("whitegrid")
    custom_style = {
            'grid.color': '0.8',
            'grid.linestyle': '--',
            'grid.linewidth': 0.5,
            }
    sns.set_style(custom_style)  
    axA.set_ylim(-90, 90)        
    xticks = np.arange(-25,10,5)   
    axA.set_xlim(xticks[0],xticks[len(xticks)-1])                                
    plt.axvline(x=0, color='k', linestyle='-',linewidth= 1) 
    
#    print dQA.mean('lon').rolling(lat = rw, center=True).mean()
    plt.plot((dQA+dQT).mean('lon').rolling(lat = rw, center=True).mean(), 
                 dQA.lat.rolling(lat = rw, center=True).mean(),color=cols[model][0],\
                   linestyle=['-',':'][ll], linewidth=1,alpha=1,
                   label=list(models_name[model])[0])
    
    
    axA.text(-0.18,1.0,['(a)','(b)','(c)','(d)','(e)','(f)','(g)','(h)'][0],
            color='k',
            horizontalalignment='left',
            verticalalignment='center',
            transform=axA.transAxes,
            fontsize=cexx,
            bbox={'facecolor':'white', 'alpha':0.7, 'pad':5})
    
    yticks = np.arange(-70,90,20) 
    axA.set_yticks(yticks, minor=False)                
    plt.yticks(fontsize=cexx) 

    axA.set_xticks(xticks, minor=False) 
    for index, label in enumerate(axA.xaxis.get_ticklabels()):
                if index % 2 != 0: # show only every 5th label
                    label.set_visible(False)
    plt.xticks(fontsize=cexx)
    axA.tick_params(axis='x', labelsize=cexx)

    axA.set_ylabel('Latitude',fontsize=cexx2)
    axA.set_xlabel('$\Delta Q_T + \Delta Q_A$ [W m$^{-2}$]',fontsize=cexx)


