#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Thu May 28 08:28:18 2020

@author: lenaboysen

MAP deforestation / remaining forest / initial forest

"""

import os
import numpy as np
np.set_printoptions(threshold=100) # 'nan'
#import os.path
import sys as sys
import pickle

#plot
import matplotlib.pyplot as plt
import seaborn as sns; sns.set(color_codes=True)
import cartopy.crs as ccrs

#import plotly.plotly as py
#import plotly.graph_objs as go
import pandas as pd



new_path = '/Users/lenaboysen/Google Drive/python/'
if new_path not in sys.path:
    sys.path.append(new_path)
    

## Constants
kg_C_per_kg_CO2 = 0.272918
kg_C_perMol = 0.012011
C_to_K = 273.15
MPa_to_mm = 1022.8


#%%  DEFINE VARIABLES
mistral = False        

regions = ['_glob','_soam','_africa','_sea_aust','_noam_temp','_noam_bor','_eura_bor','_eura_temp']
regions_names = ['Global','South America','Africa','SE Asia & Aust.','North Am. temp.','North Am. bor.','Eurasia bor.','Eurasia temp.']

models = ['MPI','IPSL','CESM2','Can','CNRM','BCC','MIROC','UKESM','EC']
models_name = pd.DataFrame({'IPSL': ['IPSL-CM6A-LR'],
                            'MPI': ['MPI-ESM-1.2.0'],
                             'CESM2': ['CESM2'],
                             'Can': ['CanESM5'],
                             'CNRM': ['CNRM-ESM2-1'],
                             'BCC': ['BCC-CSM2-MR'],
                             'MIROC': ['MIROC-ES2L'],
                             'UKESM':['UKESM1-0-LL'],
                             'EC':['EC-Earth3-Veg']})


timstat = 'year'

meta = pd.DataFrame({'model':['MPI','IPSL','CESM2',\
                              'Can','CNRM','BCC','MIROC','UKESM','EC'],
                     'imax':[150, 80,  80, 90, 80, 81,150, 80, 80],
                     'imaxctl': [90, 90, 90, 90, 90, 90,90,90, 90],
                     'nruns':[7,3, 3, 2, 1, 1, 1, 1, 1],
                     'runs': [['r1i1p1f1','r2i1p1f1','r3i1p1f1','r4i1p1f1',\
                               'r5i1p1f1','r6i1p1f1','r7i1p1f1'],\
                              ['r1i1p1f1','r2i1p1f1','r3i1p1f1'],\
                              ['r1i1p1f1','r2i1p1f1','r3i1p1f1'],\
                              ['r1i1p2f1'],\
                              ['r1i1p1f2'],\
                              ['r1i1p1f1'],\
                              ['r1i1p1f1'],\
                              ['r1i1p1f2'],\
                              ['r1i1p1f1']]\
                               }).set_index('model')
    
metaCTL = pd.DataFrame({'model':['MPI','IPSL','CESM2',\
                                 'Can','CNRM','BCC','MIROC','UKESM','EC'],
                     'imax':[150, 80,  80, 90, 80, 81, 90, 90, 80],
                     'imaxctl': [90, 90, 90, 90, 90, 90, 90, 90, 90],
                     'nruns':[ 7, 3, 1, 2, 1, 1, 1, 1, 1],
                     'runs': [['piControl'],
                              ['piControl'],
                              ['piControl'],\
                              ['piControl'],\
                              ['piControl'],
                              ['piControl'],
                              ['piControl'],
                              ['piControl'],
                              ['piControl']]}).set_index('model')
cexx = 10
cexx2 = 12    
    


#%% PLOT MAP with REGIONS
if mistral == True:
    savepath = "/Users/lenaboysen/mistral/LUMIP/"  
else:
    savepath = "/Users/lenaboysen/Documents/Crescendo_LUMIP/LUMIP/DATA/"
        
file_out = savepath + 'def_mask_regions.obj'
if os.path.isfile(file_out):
    def_mask_regions = pickle.load(open(file_out,'rb'))

model = 'IPSL'

dummy = def_mask_regions[model]['IPSL_cf_glob'].copy()   

for rr in range(1,8):
    dummy = dummy.where(np.isnan(def_mask_regions[model][model+'_defmask'+regions[rr]]), 100*rr)

#
#plt.pcolormesh(dummy.values,cmap='RdBu')
#plt.colorbar()
#plt.show()
#    
    
fig_scale = 2
fig = plt.figure(figsize=(4*fig_scale,3*fig_scale))
gs1 = plt.GridSpec(2, 1,height_ratios=[1, 0.05])
axes = plt.subplot(gs1[0,0], projection=ccrs.Robinson(central_longitude=0.0))

plot = dummy.T.plot.pcolormesh(transform = ccrs.PlateCarree(), 
                          ax=axes,cmap='rocket', add_colorbar=False)

axes.coastlines(color='grey')

plt.show()
fig.tight_layout()                
pngfile='/Users/lenaboysen/Google Drive/LUMIP/Maps/RegBoxes.png' 
fig.savefig(pngfile,dpi=300, bbox_inches='tight',width=400,height=350)
plt.close()   


#%% Read in mask of deforestation --> Overlay maps with contour
    
    ### All models in one xarray: 
    ### lat and lon don't match but xarray merges anyway! -
    ### -> can still be used for time series

for model in models:
  
    #### created with time series scripts (or ToE_ts)
    if mistral == True:
        savepath = "/Users/lenaboysen/mistral/LUMIP/" 
        path="/Users/lenaboysen/mistral/LUMIP/"+model+"/"
    else:
        savepath = "/Users/lenaboysen/Documents/Crescendo_LUMIP/LUMIP/DATA/"
        path = "/Users/lenaboysen/Documents/Crescendo_LUMIP/LUMIP/DATA/"+model+"/"
    
    file_out = savepath + 'def_mask_regions.obj'
    if os.path.isfile(file_out):
        print 'loading def mask for ' + model
        def_mask_regions = pickle.load(open(file_out,'rb'))
        def_mask = def_mask_regions[model][model+'_defmask_glob']
        diff = def_mask_regions[model][model+'_deffrac_glob']
    

    ###----------------------------------------------------
#    mycmap = plt.cm.get_cmap('YlOrBr_r')
#    mycmap.set_over('w')
#    
    cmap = plt.cm.get_cmap('YlOrBr_r')

    cmaplist = [cmap(i) for i in range(cmap.N)]
    cmap = cmap.from_list('Custom cmap', cmaplist, cmap.N)
    
    fig_scale = 2
    fig = plt.figure(figsize=(4*fig_scale,3*fig_scale))
    gs1 = plt.GridSpec(2, 1,height_ratios=[1, 0.05])
    name = 'Deforestation Fraction'
    unit = '%'
    axes = plt.subplot(gs1[0,0], projection=ccrs.Robinson(central_longitude=0.0))
    
    diffplot = diff.where(diff < -0.1)
    
    
    
    plot = diffplot.T.plot.pcolormesh(transform = ccrs.PlateCarree(), 
                              ax=axes,cmap=cmap,
                              vmin = -100, vmax= -0.1,
                              add_colorbar=False)
    
    axes.coastlines(color='grey')
    axes.coastlines(color='grey')
    axes.outline_patch.set_linewidth(1)
    axes.set_title('',fontsize=14)
    axes.gridlines(color='grey',linewidth=1, alpha=0.5, linestyle='--')
    
    ### --- Colorbar ----
    axes = plt.subplot(gs1[1, :])
    axes.set_visible(False)
    cbar_ax = fig.add_axes([0.05, 0.1, 0.9, 0.03]) ## x0, y0, width, height    

    tcks = np.arange(-100,10,10)
    bar = plt.colorbar(plot, cax=cbar_ax, 
                       ticks=tcks, boundaries=tcks,
                       orientation='horizontal',extend='both')
    
#    for label in bar.ax.xaxis.get_ticklabels()[::2]:
#        label.set_visible(False)
    
    bar.ax.tick_params(labelsize=14)
    bar.outline.set_visible(False)
    bar.set_label(name + ' ['+unit+']',fontsize=14,labelpad=-50)

    plt.show()
    
    fig.tight_layout()                
    pngfile='/Users/lenaboysen/Google Drive/LUMIP/Maps/Deforestation_'+model+'.png' 
    fig.savefig(pngfile,dpi=300, bbox_inches='tight',width=400,height=350)
    plt.close()  
    ##----------------------------------------------------




#%% All in one plot (2 rows, 3 columns)
    
cmap = plt.cm.get_cmap('YlOrBr_r')
cmaplist = [cmap(i) for i in range(cmap.N)]
cmap = cmap.from_list('Custom cmap', cmaplist, cmap.N)

cmap2 = plt.cm.get_cmap('Greens')
cmaplist2 = [cmap2(i) for i in range(cmap2.N)]
cmap2 = cmap2.from_list('Custom cmap', cmaplist2, cmap2.N)
cmap2.set_under('white')

fig_scale = 2
fig = plt.figure(figsize=(4*fig_scale,3*fig_scale))
gs1 = plt.GridSpec(4, 3, height_ratios=[1, 1, 1, 0.05])

gs1.update(wspace=0.05, hspace=0.001) #1) 

name = 'Deforestation Fraction'
unit = '%'    
i = 0; j = 0; m = 0


for model in models:
    print 'i = ' + str(i) + ' j = ' + str(j)
    #### created with time series scripts (or ToE_ts)
    if mistral == True:
        savepath = "/Users/lenaboysen/mistral/LUMIP/" 
        path="/Users/lenaboysen/mistral/LUMIP/"+model+"/"
    else:
        savepath = "/Users/lenaboysen/Documents/Crescendo_LUMIP/LUMIP/DATA/"
        path = "/Users/lenaboysen/Documents/Crescendo_LUMIP/LUMIP/DATA/"+model+"/"
    
    file_out = savepath + 'def_mask_regions.obj'
    if os.path.isfile(file_out):
        print 'loading def mask for ' +  model
        def_mask_regions = pickle.load(open(file_out,'rb'))
        def_mask = def_mask_regions[model][model+'_defmask_glob']
        diff = def_mask_regions[model][model+'_deffrac_glob']
        cellarea = def_mask_regions[model][model+'_cellarea']
#            lat = deffrac.lat
        
        ### make remaining forest unicolor
        init = def_mask_regions[model][model+'_init_forest']
        init = init.where(init>1, np.nan)
        init = init*0.0001+20
        
#        plt.pcolormesh(init,cmap=cmap2)
#        plt.colorbar()
#        plt.show()       
        
    diffplot = diff.where(diff < -0.1)
    defarea = diffplot * cellarea /100/ 10**9 #10**3 km2
    
#    plt.pcolormesh(init,cmap=cmap2)
#    plt.pcolormesh(diffplot,cmap=cmap)
#    plt.colorbar()
#    plt.show()       
    
    axes = plt.subplot(gs1[j,i], projection=ccrs.Robinson(central_longitude=0.0))

    plot = init.T.plot.pcolormesh(transform = ccrs.PlateCarree(), 
                              ax=axes,cmap=cmap2,
                              #vmin = -100, vmax= -0.1,
                              vmin = 1, vmax= 100,
                              add_colorbar=False)

    plot = diffplot.T.plot.pcolormesh(transform = ccrs.PlateCarree(), 
                              ax=axes,cmap=cmap,
                              #vmin = -100, vmax= -0.1,
                              vmin = -100, vmax= -1,
                              add_colorbar=False)
    
    cp = def_mask.T.plot.contour(ax=axes, levels=1, transform = ccrs.PlateCarree(),
          add_colorbar=False,vmin = 0, vmax = 1.1,colors='k', origin="lower" ,linewidths=0.5)
    
    axes.coastlines(color='grey',linewidth=0.5)
    axes.outline_patch.set_linewidth(0.75)
    axes.set_title('',fontsize=14)
    axes.gridlines(color='grey',linewidth=0.5, alpha=0.5, linestyle='--')
    
    axes.text(0.5,1.07,
    list(models_name[model])[0],
    color='slategrey',
    horizontalalignment='center',
    verticalalignment='center',
    transform=axes.transAxes,
    fontsize=cexx) #,
#    bbox={'facecolor':'white', 'alpha':0.7, 'pad':5})
    
    
    axes.text(0.1,1.07,
    ['(a)','(b)','(c)','(d)','(e)','(f)','(g)','(h)','(i)'][m],
    color='black',
    horizontalalignment='center',
    verticalalignment='center',
    transform=axes.transAxes,
    fontsize=cexx) #,

    
    i = i + 1
    
    if m in [2,5,8]:
        i = 0
        j = j + 1
    
    m = m + 1

    
### --- Colorbar ----
axes = plt.subplot(gs1[3, :])
axes.set_visible(False)
#cbar_ax = fig.add_axes([0.12, 0.1, 0.8, 0.03]) ## x0, y0, width, height   
cbar_ax = fig.add_axes([0.22, 0.1, 0.6, 0.02]) ## x0, y0, width, height    

#tcks = np.arange(-100,10,10)
tcks = np.arange(-100,1,10)
bar = plt.colorbar(plot, cax=cbar_ax, 
                   ticks=tcks, boundaries=tcks,
                   orientation='horizontal',extend='both')

#    for label in bar.ax.xaxis.get_ticklabels()[::2]:
#        label.set_visible(False)

bar.ax.tick_params(labelsize=10,length=2, width=1)
bar.outline.set_visible(False)
bar.set_label(name + ' ['+unit+']',fontsize=12,labelpad=-40)
#bar.set_label('Deforestation Area [10$^3$ km$^2$]',fontsize=14,labelpad=-50)

p_fancy = mpatches.FancyBboxPatch((0.8, 0.1),
                                 abs(0.3), abs(0.5),
                                 boxstyle="round, pad=0.1",
                                 fc='white',
                                 ec='black',
                                 label = '$\Delta$F',
                                 linewidth = 0.5)
plt.legend(handles=[p_fancy], fontsize=cexx-2,
               bbox_to_anchor=(-0.15, 3.0), loc=2, borderaxespad=0.)

plt.show()
#
#fig.tight_layout()                
pngfile='/Users/lenaboysen/Google Drive/LUMIP/Maps/Deforestation_ALL.png' 
#pngfile='/Users/lenaboysen/Google Drive/LUMIP/Maps/Deforestation_area_ALL.png' 
fig.savefig(pngfile,dpi=300, bbox_inches='tight',width=400,height=300)
plt.close()      


#%% All in one plot  INITIAL FOREST COVER
    
cmap = plt.cm.get_cmap('Greens')
cmaplist = [cmap(i) for i in range(cmap.N)]
cmap = cmap.from_list('Custom cmap', cmaplist, cmap.N)

cmap2 = plt.cm.get_cmap('Greens')
cmaplist2 = [cmap2(i) for i in range(cmap2.N)]
cmap2 = cmap2.from_list('Custom cmap', cmaplist2, cmap2.N)
cmap2.set_under('white')

fig_scale = 2
fig = plt.figure(figsize=(4*fig_scale,3*fig_scale))
gs1 = plt.GridSpec(4, 3, height_ratios=[1, 1, 1, 0.05])

gs1.update(wspace=0.05, hspace=0.001) #1) 

name = 'Deforestation Fraction'
unit = '%'    
i = 0; j = 0; m = 0


for model in models:
    print 'i = ' + str(i) + ' j = ' + str(j)
    #### created with time series scripts (or ToE_ts)
    if mistral == True:
        savepath = "/Users/lenaboysen/mistral/LUMIP/" 
        path="/Users/lenaboysen/mistral/LUMIP/"+model+"/"
    else:
        savepath = "/Users/lenaboysen/Documents/Crescendo_LUMIP/LUMIP/DATA/"
        path = "/Users/lenaboysen/Documents/Crescendo_LUMIP/LUMIP/DATA/"+model+"/"
    
    file_out = savepath + 'def_mask_regions.obj'
    if os.path.isfile(file_out):
        print 'loading def mask for ' +  model
        def_mask_regions = pickle.load(open(file_out,'rb'))
        def_mask = def_mask_regions[model][model+'_defmask_glob']
        diff = def_mask_regions[model][model+'_deffrac_glob']
        cellarea = def_mask_regions[model][model+'_cellarea']
#            lat = deffrac.lat
        init = def_mask_regions[model][model+'_init_forest']
        
    diffplot = diff.where(diff < -0.1)
    defarea = diffplot * cellarea /100/ 10**9 #10**3 km2
    
#    plt.pcolormesh(init,cmap=cmap2)
#    plt.pcolormesh(diffplot,cmap=cmap)
#    plt.colorbar()
#    plt.show()       
    
    axes = plt.subplot(gs1[j,i], projection=ccrs.Robinson(central_longitude=0.0))

    plot = init.T.plot.pcolormesh(transform = ccrs.PlateCarree(), 
                              ax=axes,cmap=cmap2,
                              #vmin = -100, vmax= -0.1,
                              vmin = 1, vmax= 100,
                              add_colorbar=False)

    cp = def_mask.T.plot.contour(ax=axes, levels=1, transform = ccrs.PlateCarree(),
          add_colorbar=False,vmin = 0, vmax = 1.1,colors='k', origin="lower" ,linewidths=0.5)
    
    axes.coastlines(color='grey',linewidth=0.5)
    axes.outline_patch.set_linewidth(0.75)
    axes.set_title('',fontsize=14)
    axes.gridlines(color='grey',linewidth=0.5, alpha=0.5, linestyle='--')
    
    axes.text(0.5,1.07,
    list(models_name[model])[0],
    color='slategrey',
    horizontalalignment='center',
    verticalalignment='center',
    transform=axes.transAxes,
    fontsize=cexx) #,
#    bbox={'facecolor':'white', 'alpha':0.7, 'pad':5})
    
    
    axes.text(0.1,1.07,
    ['(a)','(b)','(c)','(d)','(e)','(f)','(g)','(h)','(i)'][m],
    color='black',
    horizontalalignment='center',
    verticalalignment='center',
    transform=axes.transAxes,
    fontsize=cexx) #,

    
    i = i + 1
    
    if m in [2,5,8]:
        i = 0
        j = j + 1
    
    m = m + 1

    
### --- Colorbar ----
axes = plt.subplot(gs1[3, :])
axes.set_visible(False)
#cbar_ax = fig.add_axes([0.12, 0.1, 0.8, 0.03]) ## x0, y0, width, height   
cbar_ax = fig.add_axes([0.22, 0.1, 0.6, 0.02]) ## x0, y0, width, height    

#tcks = np.arange(-100,10,10)
tcks = np.arange(0,110,10)
bar = plt.colorbar(plot, cax=cbar_ax, 
                   ticks=tcks, boundaries=tcks,
                   orientation='horizontal')

#    for label in bar.ax.xaxis.get_ticklabels()[::2]:
#        label.set_visible(False)

bar.ax.tick_params(labelsize=10,length=2, width=1)
bar.outline.set_visible(False)
bar.set_label('Initial Forest Fraction ['+unit+']',fontsize=12,labelpad=-40)
#bar.set_label('Deforestation Area [10$^3$ km$^2$]',fontsize=14,labelpad=-50)

p_fancy = mpatches.FancyBboxPatch((0.8, 0.1),
                                 abs(0.3), abs(0.5),
                                 boxstyle="round, pad=0.1",
                                 fc='white',
                                 ec='black',
                                 label = '$\Delta$F',
                                 linewidth = 0.5)
plt.legend(handles=[p_fancy], fontsize=cexx-2,
               bbox_to_anchor=(-0.15, 3.0), loc=2, borderaxespad=0.)


plt.show()
#
#fig.tight_layout()                
pngfile='/Users/lenaboysen/Google Drive/LUMIP/Maps/Initial_Forest_ALL.png' 
#pngfile='/Users/lenaboysen/Google Drive/LUMIP/Maps/Deforestation_area_ALL.png' 
fig.savefig(pngfile,dpi=300, bbox_inches='tight',width=400,height=300)
plt.close()          
    
#%% All in one plot (2 rows, 3 columns)  ***  DEFOREST - INITIAL FOREST ***
    
cmap = plt.cm.get_cmap('YlOrBr')
cmaplist = [cmap(i) for i in range(cmap.N)]
cmap = cmap.from_list('Custom cmap', cmaplist, cmap.N)
cmap.set_under('white')

cmap2 = plt.cm.get_cmap('Greens')
cmaplist2 = [cmap2(i) for i in range(cmap2.N)]
cmap2 = cmap2.from_list('Custom cmap', cmaplist2, cmap2.N)
cmap2.set_under('white')

fig_scale = 2
fig = plt.figure(figsize=(4*fig_scale,3*fig_scale))
gs1 = plt.GridSpec(4, 3, height_ratios=[1, 1, 1, 0.05])

gs1.update(wspace=0.05, hspace=0.001) #1) 

name = 'Remaining Fraction'
unit = '%'    
i = 0; j = 0; m = 0


for model in models:
    print 'i = ' + str(i) + ' j = ' + str(j)
    #### created with time series scripts (or ToE_ts)
    if mistral == True:
        savepath = "/Users/lenaboysen/mistral/LUMIP/" 
        path="/Users/lenaboysen/mistral/LUMIP/"+model+"/"
    else:
        savepath = "/Users/lenaboysen/Documents/Crescendo_LUMIP/LUMIP/DATA/"
        path = "/Users/lenaboysen/Documents/Crescendo_LUMIP/LUMIP/DATA/"+model+"/"
    
    file_out = savepath + 'def_mask_regions.obj'
    if os.path.isfile(file_out):
        print 'loading def mask for ' +  model
        def_mask_regions = pickle.load(open(file_out,'rb'))
        def_mask = def_mask_regions[model][model+'_defmask_glob']
        diff = def_mask_regions[model][model+'_deffrac_glob']
        cellarea = def_mask_regions[model][model+'_cellarea']
#            lat = deffrac.lat
        init = def_mask_regions[model][model+'_init_forest']
        
    diffplot = init + diff#.where(diff < -0.1) 
    defarea = diffplot * cellarea /100/ 10**9 #10**3 km2
#    
#    plt.pcolormesh(init.T,cmap=cmap2)
#    plt.pcolormesh(diffplot.T,cmap=cmap)
#    plt.colorbar()
#    plt.show()       
    
    axes = plt.subplot(gs1[j,i], projection=ccrs.Robinson(central_longitude=0.0))

    plot = diffplot.T.plot.pcolormesh(transform = ccrs.PlateCarree(), 
                              ax=axes,cmap=cmap,
                              #vmin = -100, vmax= -0.1,
                              vmin = 1, vmax= 100,
                              add_colorbar=False)
    
    cp = def_mask.T.plot.contour(ax=axes, levels=1, transform = ccrs.PlateCarree(),
          add_colorbar=False,vmin = 0, vmax = 1.1,colors='k', origin="lower" ,linewidths=0.5)

    axes.coastlines(color='grey',linewidth=0.5)
    axes.outline_patch.set_linewidth(0.75)
    axes.set_title('',fontsize=14)
    axes.gridlines(color='grey',linewidth=0.5, alpha=0.5, linestyle='--')
    
    axes.text(0.5,1.07,
    list(models_name[model])[0],
    color='slategrey',
    horizontalalignment='center',
    verticalalignment='center',
    transform=axes.transAxes,
    fontsize=cexx) #,
#    bbox={'facecolor':'white', 'alpha':0.7, 'pad':5})
    
    
    axes.text(0.1,1.07,
    ['(a)','(b)','(c)','(d)','(e)','(f)','(g)','(h)','(i)'][m],
    color='black',
    horizontalalignment='center',
    verticalalignment='center',
    transform=axes.transAxes,
    fontsize=cexx) #,

    
    i = i + 1
    
    if m in [2,5,8]:
        i = 0
        j = j + 1
    
    m = m + 1

    
### --- Colorbar ----
axes = plt.subplot(gs1[3, :])
axes.set_visible(False)
#cbar_ax = fig.add_axes([0.12, 0.1, 0.8, 0.03]) ## x0, y0, width, height   
cbar_ax = fig.add_axes([0.22, 0.1, 0.6, 0.02]) ## x0, y0, width, height    

#tcks = np.arange(-100,10,10)
tcks = np.arange(0,100,10)
bar = plt.colorbar(plot, cax=cbar_ax, 
                   ticks=tcks, boundaries=tcks,
                   orientation='horizontal')

#    for label in bar.ax.xaxis.get_ticklabels()[::2]:
#        label.set_visible(False)

bar.ax.tick_params(labelsize=10,length=2, width=1)
bar.outline.set_visible(False)
bar.set_label(name + ' ['+unit+']',fontsize=12,labelpad=-40)
#bar.set_label('Deforestation Area [10$^3$ km$^2$]',fontsize=14,labelpad=-50)

plt.show()
#
#fig.tight_layout()                
pngfile='/Users/lenaboysen/Google Drive/LUMIP/Maps/Remaining_Forest_ALL.png' 
#pngfile='/Users/lenaboysen/Google Drive/LUMIP/Maps/Deforestation_area_ALL.png' 
fig.savefig(pngfile,dpi=300, bbox_inches='tight',width=400,height=300)
plt.close()      
        