#!/bin/bash
set -vex


for model in  MPI MIROC UKESM CNRM Can BCC CESM2 IPSL EC
do


base="/work/mj0060/m300178/LUMIP/"
#base="/scratch/local1/m300178/LUMIP/"
out_dir="${base}analysis/${model}"

#constants
kg_C_per_kg_CO2=0.272918
g_C_perMol=12.011
kg_C_perMol=0.012011
C_to_K=-273.15
missingValue=-99999

## ==============

if [ ${model} == "IPSL" ]; then
vars=( cLand tas cellarea pr \
	cVeg cSoil cSoilSlow cSoilFast cLitter\
	 treeFrac grassFrac \
	evspsbl tran \
	gpp npp rh \
	ts hfss hfls rsds rsus rlds clt rldscs )
varsout=( cLand tas cellarea  pr \
        cVeg cSoil cSoilSlow cSoilFast cLitter\
        treeFrac grassFrac \
        evspsbl tran \
        gpp npp rh\
	ts hfss hfls rsds rsus rlds clt rldscs )
vmax=23
elif [ ${model} == "MPI" ]; then
vars=( cLand tas cellarea pr \
        cVeg cSoil cSoilSlow cSoilFast cLitter\
        burntArea fFire treeFrac grassFrac \
        evspsbl tran \
        gpp npp rhSoil\
	ts hfss hfls rsds rsus rlds clt rldscs )
varsout=( cLand tas cellarea pr \
        cVeg cSoil cSoilSlow cSoilFast cLitter\
        burntArea fFire treeFrac grassFrac \
        evspsbl tran \
        gpp npp rh\
	ts hfss hfls rsds rsus rlds clt rldscs )
vmax=25
elif [ ${model} == "CESM" ]; then 
vars=( cLand tas cellarea pr \
        cVeg cSoilSlow cSoilFast cSoilMedium cLitter\
        burntFractionAll fFire treeFrac treeFracBdlDcd treeFracNdlEvg grassFrac \
        evspsblsoi evspsblveg tran \
        gpp npp rh)
varsout=( cLand tas cellarea pr \
        cVeg cSoilSlow cSoilFast cSoilMedium cLitter\
        burntFrac fFire treeFrac treeFracBdlDcd treeFracNdlEvg grassFrac \
        evspsblsoi evspsblveg tran \
        gpp npp rh )
vmax=19

elif [ ${model} == "CESM2" ]; then
vars=( cLand tas pr cellarea \
        cVeg cSoilSlow cSoilFast cSoilMedium cSoil cLitter\
        fFire treeFrac \
        evspsbl \
        gpp npp rh \
	ts hfss hfls rsds rsus rlds clt rldscs )
varsout=( cLand tas pr cellarea \
        cVeg cSoilSlow cSoilFast cSoilMedium cSoil cLitter\
        fFire treeFrac \
        evspsbl \
        gpp npp rh \
	ts hfss hfls rsds rsus rlds clt rldscs )
vmax=22

elif [ ${model} == "BCC" ]; then
vars=( cLand tas cellarea pr \
        cVeg cSoil cSoilSlow cSoilMedium \
	cLitter cLitterAbove cLitterBelow \
        cLand evspsbl \
        gpp npp rh \
	hfss hfls rsds rsus rlds ts clt rldscs )
varsout=( cLand tas cellarea pr \
        cVeg cSoil cSoilSlow cSoilMedium \
	cLitter cLitterAbove cLitterBelow \
        cLand evspsbl
        gpp npp rh \
	hfss hfls rsds rsus rlds ts clt rldscs )
vmax=23

elif [ ${model} == "Can" ]; then
vars=( cLand tas cellarea pr \
        cVeg cSoil  cLitter \
        evspsbl treeFrac grassFrac \
        gpp npp rh \
	ts hfss hfls rsds rsus rlds clt rldscs )
varsout=( cLand tas cellarea pr \
        cVeg cSoil cLitter \
        evspsbl treeFrac grassFrac \
        gpp npp rh \
	ts hfss hfls rsds rsus rlds clt rldscs )
vmax=20

elif [ ${model} == "CNRM" ]; then
vars=( cLand tas cellarea pr \
        cVeg cSoilSlow cSoilFast cSoilMedium cSoil \
	cLitter  cLitterBelow \
        treeFrac grassFrac \
        evspsbl \
        gpp npp rh\
	ts hfss hfls rsds rsus rlds clt rldscs )
varsout=( cLand tas cellarea pr \
        cVeg cSoilSlow cSoilFast cSoilMedium cSoil\
	cLitter cLitterBelow \
        treeFrac grassFrac \
        evspsbl \
        gpp npp rh\
	ts hfss hfls rsds rsus rlds clt rldscs )
vmax=24

elif [ ${model} == "MIROC" ]; then
vars=( cLand tas cellarea pr \
        cVeg cSoilSlow cSoilFast cSoilMedium cSoil \
        cLitter \
        treeFrac \
        evspsbl \
        gpp npp rh\
	ts hfss hfls rsds rsus rlds clt rldscs )
varsout=( cLand tas cellarea pr \
        cVeg cSoilSlow cSoilFast cSoilMedium cSoil\
        cLitter \
        treeFrac \
        evspsbl \
        gpp npp rh\
	ts hfss hfls rsds rsus rlds clt rldscs )
vmax=22

elif [ ${model} == "UKESM" ]; then
vars=( cLand tas cellarea pr \
        cVeg cSoilFast cSoilMedium cSoil \
        treeFrac \
        evspsbl \
        gpp npp rh\
	ts hfss hfls rsds rsus rlds clt rldscs )
varsout=( cLand tas cellarea pr \
        cVeg cSoilFast cSoilMedium cSoil\
        treeFrac \
        evspsbl \
        gpp npp rh\
	ts hfss hfls rsds rsus rlds clt rldscs )
vmax=20
elif [ ${model} == "EC" ]; then
vars=( cLand tas cellarea pr \
        cVeg cLitter cSoil cLand \
        treeFrac \
        evspsbl \
        gpp npp rh fFireNat \
        ts hfss hfls rsds rsus rlds clt rldscs )
varsout=( cLand tas cellarea pr \
        cVeg cLitter cSoil cLand \
        treeFrac \
        evspsbl \
        gpp npp rh fFire \
        ts hfss hfls rsds rsus rlds clt rldscs )
vmax=21
fi


for vv in  `seq 0 ${vmax}`
do
	for timstat in  "year" # "mon" 
	do


#if [ ${varsout[${vv}]} != "clt" ] ;then
#	continue
#fi


### --------------------------------------


if [ ${model} == "MPI" ];then
exp=`seq 1 8`
elif [ ${model} == "CESM" ];then
exp=( 1 piControl)
elif [ ${model} == "CESM2" ];then
exp=( 1 2 3 piControl )
elif [ ${model} == "IPSL" ];then
exp=( 1 2 3 piControl )
elif [ ${model} == "Can" ];then
exp=( 2 piControl ) # 1 piControl2 )
elif [ ${model} == "CNRM" ] ||  [ ${model} == "UKESM" ] ;then
exp=( 2 piControl)
else
exp=( 1 piControl)
fi

		for xx in ${exp[@]}
		do


#        	if [ -f ${ofile} ]; then
#                	continue
#        	fi

if [[ ${model} == "CESM2" && ${vars[${vv}]} == "treeFrac" ]] && [[ ${model} == "CESM2" && ${xx} == "piControl" ]]; then
	continue
fi

if [[ ${model} == "MIROC" && ${vars[${vv}]} == "treeFrac" && ${xx} == "piControl" ]]; then 
	continue
fi


if [ ${model} == "MPI" ] && [ ${xx} == 8 ];then
outname="piControl"
elif [ ${xx} == "piControl" ]; then
outname="piControl"
elif [ ${xx} == "piControl2" ]; then
outname="piControl2"
elif [[ ${model} == "Can" && ${xx} == 1 ]] || [[ ${model} == "Can" && ${xx} == 2 ]] ;then
outname="r1i1p${xx}f1"
#elif [ ${model} == "Can" ] && [ ${xx} == "piControl2" ]; then
#outname="ri1p2f1"
elif [[ ${model} == "CNRM" && ${xx} == 2 ]] ; then
outname="r1i1p1f${xx}"
elif [[ ${model} == "UKESM" && ${xx} == 2 ]]; then 
outname="r1i1p1f2"
else
outname="r${xx}i1p1f1"
fi


#if [[ ${model} == "CESM" && ${xx} == 1 && ${vars[${vv}]} == "treeFracBdlDcd" ]]; then
#continue
#elif [[ ${model} == "CESM" && ${xx} == 1 && ${vars[${vv}]} == "treeFracNdlEvg" ]]; then
#continue
#fi

#if [[ ${model} == "CESM" && ${xx} == "piControl" && ${vars[${vv}]} == "treeFrac" ]];then
#continue
#fi




opath=${base}/${model}/${outname}/

if [ ! -d ${opath}/ ]; then
	mkdir -p ${opath}
fi

ofile="${opath}/${varsout[${vv}]}_${model}_${timstat}.nc"

## skip variable if exists
if [ -f ${ofile} ]; then 
	continue
fi
## or overwrite variable
#if [ -f $ofile ];then
#	rm $ofile
#fi

if [ ${vars[${vv}]} != "cellarea" ]; then
	
	if [ ${model} == "MPI" ] && [ $vv -le 23 ] ;then
	fpath="/work/mj0060/m300618/LUMIP/mpiesm-landveg-cmip6/experiments/"
	ifileCTL=${fpath}/alu000${xx}/cmor/outdata_land/alu000${xx}_${vars[${vv}]}.nc
	elif [ ${model} == "BCC" ] && [ ${vars[${vv}]} == "cLand" ];then	
	fpath="/work/mj0060/m300178/LUMIP/${model}/${outname}/"
	elif [ ${model} == "EC" ] && [ ${vars[${vv}]} == "cLand" ];then
        fpath="/work/mj0060/m300178/LUMIP/${model}/${outname}/"
	elif [ ${model} == "MIROC" ] && [ ${vars[${vv}]} == "cSoil" ];then
        fpath="/work/mj0060/m300178/LUMIP/${model}/${outname}/"
	elif [ ${model} == "EC" ] && [ ${xx} == 1 ]; then
        fpath="/work/mj0060/m300178/LUMIP/${model}/${outname}/"
        ifileCTL=`ls ${opath}/deforestGlobe_${vars[${vv}]}_*`
	else
	fpath="/work/mj0060/m300178/LUMIP/${model}/${outname}/"
	ifileCTL=`ls ${opath}/${vars[${vv}]}_*`
	fi
	
fi

## two files for CESM2 piControl and r2 and r3
#if [[ ${model} == "CESM2" && ${xx} != 1 ]] && [ ${vars[${vv}]} != "cellarea" ];then # &&  [ ${vars[${vv}]} != "cLand" ]; then
if [[ ${model} == "CESM2" ]] && [ ${vars[${vv}]} != "cellarea" ];then #
##ifileCTL2=($(ls ${opath}/${vars[${vv}]}_*))
##ifileCTL=${ifileCTL2[0]}
	ifiles=($(ls ${fpath}${vars[${vv}]}*))
	ifileCTL=${ifiles[0]}
	ifileCTL2=${ifiles[1]}
	ofileCTL=`echo $ifileCTL | sed 's|\(.*\)_.*|\1|'`
	if [ ! -f ${ofileCTL}_185001-192912.nc ] ; then
		cdo mergetime $ifileCTL $ifileCTL2 ${ofileCTL}_185001-192912.nc
	fi 
	ifileCTL="${ofileCTL}_185001-192912.nc"
##	ofileCTL=`echo $ifiles | sed 's|\(.*\)_.*|\1|'`
##	cdo selyear,501/580 $ifiles ${ofileCTL}_185001-192912.nc fi
fi



if [[ ( ${model} == "MPI"  &&  ${vars[${vv}]} == "grassFrac" ) || \
	( ${model} == "MPI"  &&  ${vars[${vv}]} == "treeFrac" ) || \
	( ${model} == "MPI"  &&  ${vars[${vv}]} == "shrubFrac" ) ||
	( ${model} == "MPI"  &&  ${vars[${vv}]} == "rsus"  && ${xx} != 'piControl' ) ]] ;then
	fpath="/work/mj0060/m300178/LUMIP/MPI/"
	ifileCTL=${fpath}/alu000${xx}/alu000${xx}_${vars[${vv}]}.nc
fi


#ofile="${opath}/${varsout[${vv}]}_${model}_${timstat}.nc"

if [ $xx == 'piControl' ]; then
	additional="-seltimestep,1/90"
else
	additional=" "
fi

if [[ ( $xx == 'piControl'  && $model == "BCC" ) ]]; then
	additional="-selyear,2290/2360"
fi


if [[ (  $model == 'MIROC' && $xx != "piControl" ) ]]; then
	additional="${additional} -invertlat"
fi


if [[ (  $model == 'EC' && ${varsout[${vv}]} == 'evspsbl' ) ]]; then
        additional="${additional} -mulc,-1"
fi


### mask land data with sftlf to nan-oceans; done only once!
#if [ $model == 'EC' ]; then
#	myArray=(gpp npp cLand cVeg cSoil cLitter  rh treeFrac)
#	sftlf=${fpath}sftlf_nan.nc
#	if [[ ${myArray[*]} =~ ${varsiout[${vv}]} ]]; then
#		cp ${ifileCTL} ${fpath}..	 
#		cdo ifthen ${sftlf} ${ifileCTL} ${ifileCTL%.*}_sftlf.nc
#		mv ${ifileCTL%.*}_sftlf.nc ${ifileCTL} 
#
#	fi
#
#fi 


### multiply org data with sftlf; done only once!!!
#if  [ $model == 'Can' ]; then
#        myArray=(cLand cVeg cSoil cLitter cSoilMedium cSoilFast cSoilSlow npp gpp rh)
#        sftlf=${fpath}sftlf_nan.nc
#        if [[ ${myArray[*]} =~ ${varsiout[${vv}]} ]]; then
# #               cp ${ifileCTL} ${fpath}..
#                cdo divc,100 -mul ${sftlf} ${ifileCTL} ${ifileCTL%.*}_sftlf.nc
#                mv ${ifileCTL%.*}_sftlf.nc ${ifileCTL}
#        fi
#
#fi






#	if [ -f ${ofile} ]; then
#		continue
#	fi	
	
	if [ ${vars[${vv}]}  = "gpp" ] || [ ${vars[${vv}]} = "npp" ] ||  \
		[ ${vars[${vv}]} = "rhSoil" ] || [ ${vars[${vv}]} = "rh" ];then
		cdo -O ${additional} -setname,${varsout[${vv}]} -setunit,"kgC/m2yr" -${timstat}sum -mulc,86400 -muldpm ${ifileCTL} ${ofile}
	elif [ ${vars[${vv}]} = "pr" ] || [ ${vars[${vv}]} = "evapotrans" ] || [ ${vars[${vv}]} = "tran" ] || \
		[ ${vars[${vv}]} = "evspsbl" ] || [ ${vars[${vv}]} = "evspsblsoi" ] || [ ${vars[${vv}]} = "evspsblveg" ] ;then
		if [ $timstat = "year" ]; then
			cdo -O ${additional} -setname,${varsout[${vv}]} -setunit,"mm/yr" -mulc,86400 -muldpm -yearsum ${ifileCTL} ${ofile}	 
		elif [ $timstat = "mon" ]; then # is already per month
			cdo -O ${additional} -setname,${varsout[${vv}]} -setunit,"mm/mon" -mulc,86400 ${ifileCTL} ${ofile}
		fi
	elif [ ${vars[${vv}]} = "burntArea" ] ;then
		cdo -O ${additional} -setname,${varsout[${vv}]} -setunit,"m2/yr" -${timstat}sum ${ifileCTL} ${ofile}
	elif [ ${vars[${vv}]} = "alpha" ] ;then # %/mon
		cdo -O ${additional} -setname,${varsout[${vv}]} -setunit,"yearmax" -${timstat}max  ${opath}/dummy.nc ${ofile}
	elif [ ${vars[${vv}]} = "treeFrac" ] || [ ${vars[${vv}]} = "grassFrac" ] ||\
			[ ${vars[${vv}]} = "treeFracBdlDcd" ] || [ ${vars[${vv}]} = "treeFracNdlEvg" ];then
		cdo -O ${additional} -setname,${varsout[${vv}]} -setunit,"m2/yr" -${timstat}mean ${ifileCTL} ${ofile}
		if [ ${vars[${vv}]} = "treeFracNdlEvg" ]; then 
			cdo -O -setname,treeFrac -add ${opath}/treeFracBdlDcd_${model}_${timstat}.nc ${opath}/treeFracNdlEvg_${model}_${timstat}.nc ${opath}/treeFrac_${model}_${timstat}.nc
		fi
	elif [ ${vars[${vv}]} = "cellarea" ]; then
		cdo -O -seltimestep,1 -setname,${varsout[${vv}]} -gridarea ${opath}/tas_${model}_${timstat}.nc ${ofile}
	
	

	elif [ ${vars[${vv}]} = "cLand" ] && [ $model == "UKESM" ];then
		cdo -O -setname,cLand -add ${opath}/cSoil_${model}_${timstat}.nc ${opath}/cVeg_${model}_${timstat}.nc  ${opath}/cLand_${model}_${timstat}.nc
        elif [ ${vars[${vv}]} = "cLand" ] ;then
                cdo -O -setname,cLand -add ${opath}/cSoil_${model}_${timstat}.nc  -add ${opath}/cLitter_${model}_${timstat}.nc ${opath}/cVeg_${model}_${timstat}.nc ${opath}/cLand_${model}_${timstat}.nc



	elif [ ${vars[${vv}]} = "cSoil" ] && [ $model == "CESM2" ] ;then
		cdo -O -setname,cSoil -add ${opath}/cSoilSlow_${model}_${timstat}.nc  -add ${opath}/cSoilMedium_${model}_${timstat}.nc ${opath}/cSoilFast_${model}_${timstat}.nc ${opath}/cSoil_${model}_${timstat}.nc
	elif [ ${vars[${vv}]} = "cSoil" ] && [ $model == "CNRM" ] ;then
                cdo -O -setname,cSoil -add ${opath}/cSoilSlow_${model}_${timstat}.nc  -add ${opath}/cSoilMedium_${model}_${timstat}.nc ${opath}/cSoilFast_${model}_${timstat}.nc ${opath}/cSoil_${model}_${timstat}.nc
        elif [ ${vars[${vv}]} = "cSoil" ] && [ $model == "MIROC" ] ;then
                cdo -O -setname,cSoil -add ${opath}/cSoilSlow_${model}_${timstat}.nc  -add ${opath}/cSoilMedium_${model}_${timstat}.nc ${opath}/cSoilFast_${model}_${timstat}.nc ${opath}/cSoil_${model}_${timstat}.nc
	elif [ ${varsout[${vv}]} = "fFire" ] && [ $model == "EC" ] ;then
		cdo -O ${additional} -setname,${varsout[${vv}]} -setunit,"kgC/m2yr" -${timstat}sum -mulc,86400 -muldpm -selmon,12 ${ifileCTL} ${ofile}	
	else
		cdo -O ${additional} -setname,${varsout[${vv}]} -${timstat}mean ${ifileCTL} ${ofile}
	fi

	if [ ${vars[${vv}]} = "treeFrac" ] && [ $model == "EC" ] ;then
                cdo -O ${additional}  -setname,${varsout[${vv}]} -setunit,"m2/yr" -${timstat}max ${ifileCTL} ${ofile}
	fi


	


	if [ ${varsout[${vv}]} = "rh" ] && [ !-f ${opath}/nep_${model}_${timstat}.nc ]  ;then
		cdo -O -setname,nep -sub ${opath}/npp_${model}_${timstat}.nc ${opath}/rh_${model}_${timstat}.nc ${opath}/nep_${model}_${timstat}.nc
	fi
		
	if [ ${varsout[${vv}]} = "rh" ];then
		if  [ $model == "MPI" ] || [ $model == "CESM2" ] || [ $model == "EC" ]; then 
		cdo -O -setname,NEPfFire -sub ${opath}/nep_${model}_${timstat}.nc ${opath}/fFire_${model}_${timstat}.nc ${opath}/NEPfFire_${model}_${timstat}.nc
		fi
	fi


			done ## xx
		done ## timstat
	done ## v in vars

done ## model



exit
fi
