#!/bin/bash
#2020-05-02 peter anthoni (peter.anthoni@kit.edu)
#determine deforested tree area for LUMIP EC-Earth-Veg deforest globe experiment

cutoff=0.0
if [[ $# -gt 0 ]]; then
  cutoff=$1  
fi
echo "edge cut off limit $cutoff"

cdoopt="-s -L"

#Note: in LPJ-GUESS deforestation gross transition of year 1850 is performed on 1st Jan 1851 (our deforestation is shifted by a year!)
#psl=Natural, grasslands, bare ground (the LPJ-GUESS NATURAL category)
cdo ${cdoopt} --reduce_dim sellevel,1 -selyear,1851 deforestGlobe_fracOutLut_1850-1929.nc deforestGlobe_fracOutPsl_1851.nc

#generate a deforested grid mask
#one could choose a cut off (e.g. 0.00001) to remove edge grid cells, that originate from the remapping from the ECE reduced gaussian to CMIP6 netcdf format
cdo ${cdoopt} expr,"deforested=(fracOutLut>${cutoff})" deforestGlobe_fracOutPsl_1851.nc deforestGlobe_deforestPsl_cutoff${cutoff}.nc

#Note: divc,100 convert treeFrac % to fraction, divc,1e12 convert m2 to Mkm2
#only in grid cell that are getting deforested
cdo ${cdoopt} outputkey,year,month,value -divc,1e12 -fldsum -mul gridarea.nc -mul deforestGlobe_deforestPsl_cutoff${cutoff}.nc -divc,100 deforestGlobe_treeFrac_1850-1929_ECE.nc | sed 's,# ,,' > treeFrac_mularea_fdlsum_deforested_year_month_cutoff${cutoff}.txt

#in all gridcells (without deforested grid mask) 
cdo ${cdoopt} outputkey,year,month,value -divc,1e12 -fldsum -mul gridarea.nc -divc,100 deforestGlobe_treeFrac_1850-1929_ECE.nc  | sed 's,# ,,' > treeFrac_mularea_fdlsum_year_month.txt

#select yearly maximum value (the foliage projected cover of trees that is used to determin treeFrac has a seasonal component)
#only in grid cell that are getting deforested  (converted treeFrac % to fraction and m2 to Mkm2)
cdo ${cdoopt} outputkey,year,value -yearmax -divc,1e12 -fldsum -mul gridarea.nc -mul deforestGlobe_deforestPsl_cutoff${cutoff}.nc -divc,100 deforestGlobe_treeFrac_1850-1929_ECE.nc | sed 's,# ,,' > treeFrac_mularea_fdlsum_deforested_yearmax_cutoff${cutoff}.txt

#in all gridcells (without deforested grid mask) 
cdo ${cdoopt} outputkey,year,value -yearmax -divc,1e12 -fldsum -mul gridarea.nc -divc,100 deforestGlobe_treeFrac_1850-1929_ECE.nc  | sed 's,# ,,' > treeFrac_mularea_fdlsum_yearmax.txt

echo "change in tree fraction area in all gridcells"
year1=1850
year2=1902
for fn in $(ls treeFrac_mularea_fdlsum*yearmax.txt)
do
  grep -H "year\|${year2}\|${year1}" ${fn}
  area_year1=$(grep "${year1}" $fn | awk '{print $2}')
  area_year2=$(grep "${year2}" $fn | awk '{print $2}')
  area_delta=$(perl -E "say ${area_year1} - ${area_year2}")
  echo "$fn delta ${area_delta}"
done

echo "change in tree fraction area in deforested gridcells with edge cutoff of ${cutoff}"
for fn in $(ls treeFrac_mularea_fdlsum*yearmax*cutoff${cutoff}.txt)
do
  grep -H "year\|${year2}\|${year1}" ${fn}
  area_year1=$(grep "${year1}" $fn | awk '{print $2}')
  area_year2=$(grep "${year2}" $fn | awk '{print $2}')
  area_delta=$(perl -E "say ${area_year1} - ${area_year2}")
  echo "$fn delta ${area_delta}"
done
